@@ -1,10 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998-2000  Andrew Clausen, Lennert Buytenhek and Red Hat Inc.
-
-	Andrew Clausen			<clausen@alphalink.com.au>
-	Lennert Buytenhek		<buytenh@scotty.dsv.nl>
-        Matt Wilson, Red Hat Inc.       <msw@redhat.com>
+    Copyright (C) 1998-2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,8 +28,7 @@ typedef struct _PedDiskType		PedDiskType;
 
 #include <parted/device.h>
 #include <parted/filesys.h>
-
-#define PED_MAX_PART_PER_DISK	16
+#include <parted/natmath.h>
 
 struct _PedGeometry {
 	PedDisk*		disk;
@@ -50,21 +45,37 @@ typedef enum {
 	PED_PARTITION_METADATA		= 0x08
 } PedPartitionType;
 
+typedef enum {
+	PED_PARTITION_BOOT=1,
+	PED_PARTITION_ROOT=2,
+	PED_PARTITION_SWAP=3,
+	PED_PARTITION_HIDDEN=4,
+	PED_PARTITION_RAID=5,
+	PED_PARTITION_LVM=6,
+	PED_PARTITION_LBA=7
+} PedPartitionFlag;
+#define PED_PARTITION_FIRST_FLAG	PED_PARTITION_BOOT
+#define PED_PARTITION_LAST_FLAG		PED_PARTITION_LBA
+
+typedef enum {
+	PED_DISK_TYPE_EXTENDED=1,
+	PED_DISK_TYPE_PARTITION_NAME=2
+} PedDiskTypeFeature;
+#define PED_DISK_TYPE_FIRST_FEATURE	PED_PARTITION_EXTENDED
+#define PED_DISK_TYPE_LAST_FEATURE	PED_PARTITION_NAME
+
 struct _PedPartition {
 	PedPartition*		prev;
 	PedPartition*		next;
 
 	PedGeometry		geom;
 	int			num;
-	int			hidden;
 
 	PedPartitionType	type;
-	PedPartition*		part_list;
+	const PedFileSystemType* fs_type;
+	PedPartition*		part_list;	/* for extended partitions */
 
-	int			system;
-	int			bootable;
-
-	PedFileSystemType*	fs_type;
+	void*			disk_specific;
 };
 
 struct _PedDisk {
@@ -72,34 +83,58 @@ struct _PedDisk {
 	PedDiskType*		type;
 	PedPartition*		part_list;
 
+	void*			disk_specific;
+
 /* office use only ;-) */
 	int			update_mode;
 };
 
-/* align_geometry() should only be called on partitions that have geometry,
- * a PedPartitionType, and a number.
- * All of the bottom four operations are "private", and should only be called
- * by functions in disk.c.
- */
 struct _PedDiskOps {
+	/* disk label operations */
 	int (*probe) (const PedDevice *dev);
-
 	PedDisk* (*open) (PedDevice* dev);
 	PedDisk* (*create) (PedDevice* dev);
+	int (*clobber) (PedDevice* dev);
 	int (*close) (PedDisk* disk);
 	int (*read) (PedDisk* disk);
 	int (*write) (PedDisk* disk);
 
-	int (*align_partition) (PedDisk* disk, PedPartition* part);
-	int (*enumerate_partition) (PedDisk* disk, PedPartition* part);
+	/* partition operations */
+	PedPartition* (*partition_new) (
+		const PedDisk* disk,
+	       	PedPartitionType part_type,
+		const PedFileSystemType* fs_type,
+		PedSector start,
+		PedSector end);
+	void (*partition_destroy) (PedPartition* part);
+	int (*partition_set_flag) (
+		PedPartition* part,
+	       	PedPartitionFlag flag,
+		int state);
+	int (*partition_get_flag) (
+		const PedPartition* part,
+		PedPartitionFlag flag);
+	int (*partition_is_flag_available) (
+		const PedPartition* part,
+		PedPartitionFlag flag);
+	void (*partition_set_name) (PedPartition* part, const char* name);
+	const char* (*partition_get_name) (const PedPartition* part);
+	int (*partition_align) (PedPartition* part,
+				const PedConstraint* constraint);
+	int (*partition_enumerate) (PedPartition* part);
+	int (*partition_set_extended_system) (PedPartition* part);
+
+	/* other */
 	int (*alloc_metadata) (PedDisk* disk);
-	int (*get_extended_system) ();
+	int (*get_max_primary_partition_count) (const PedDisk* disk);
 };
 
 struct _PedDiskType {
 	PedDiskType*		next;
 	const char*		name;
 	PedDiskOps* const	ops;
+
+	PedDiskTypeFeature	features;
 };
 
 extern void ped_register_disk_type (PedDiskType* type);
@@ -107,25 +142,39 @@ extern void ped_unregister_disk_type (PedDiskType* type);
 extern PedDiskType* ped_disk_type_get_next (PedDiskType* type);
 extern PedDiskType* ped_disk_type_get (const char* name);
 extern PedDiskType* ped_disk_probe (const PedDevice* dev);
+extern int ped_disk_type_check_feature (const PedDiskType* disk_type,
+	       				PedDiskTypeFeature feature);
 
 extern PedDisk* ped_disk_open (PedDevice* dev);
 extern PedDisk* ped_disk_create (PedDevice* dev, PedDiskType* type);
-extern PedDisk* ped_disk_alloc (PedDevice* dev, PedDiskType* type);
+extern int ped_disk_clobber (PedDisk* disk);
 extern int ped_disk_close (PedDisk* disk);
 extern int ped_disk_read (PedDisk* disk);
 extern int ped_disk_write (PedDisk* disk);
 extern int ped_disk_is_busy (const PedDisk* disk);
+extern int ped_disk_get_max_primary_partition_count (const PedDisk* disk);
+extern PedDisk* ped_disk_alloc (PedDevice* dev, PedDiskType* type);
+extern void ped_disk_free (PedDisk* disk);
 
+extern int ped_geometry_init (PedGeometry* geom, const PedDisk* disk,
+			      PedSector start, PedSector length);
 extern PedGeometry* ped_geometry_new (const PedDisk* disk, PedSector start,
 				      PedSector length);
 extern PedGeometry* ped_geometry_duplicate (const PedGeometry* geom);
+extern PedGeometry* ped_geometry_intersect (const PedGeometry* a,
+	       				    const PedGeometry* b);
 extern void ped_geometry_destroy (PedGeometry* geom);
-extern void ped_geometry_set (PedGeometry* geom, PedSector start,
-			      PedSector length);
-extern void ped_geometry_set_start (PedGeometry* geom, PedSector start);
-extern void ped_geometry_set_end (PedGeometry* geom, PedSector end);
-extern int ped_geometry_test_overlap (PedGeometry* a, PedGeometry* b);
-extern int ped_geometry_test_inside (PedGeometry* a, PedGeometry* b);
+extern int ped_geometry_set (PedGeometry* geom, PedSector start,
+			     PedSector length);
+extern int ped_geometry_set_start (PedGeometry* geom, PedSector start);
+extern int ped_geometry_set_end (PedGeometry* geom, PedSector end);
+extern int ped_geometry_test_overlap (const PedGeometry* a,
+	       			      const PedGeometry* b);
+extern int ped_geometry_test_inside (const PedGeometry* a,
+				     const PedGeometry* b);
+extern int ped_geometry_test_equal (const PedGeometry* a, const PedGeometry* b);
+extern int ped_geometry_test_sector_inside (const PedGeometry* geom,
+	       				    PedSector sect);
 
 extern int ped_geometry_read (const PedGeometry* geom, void* buffer,
 			      PedSector offset, PedSector count);
@@ -141,28 +190,47 @@ extern PedSector ped_geometry_map (const PedGeometry* dst,
 				   const PedGeometry* src,
 				   PedSector sector);
 
+extern PedPartition* ped_partition_alloc (const PedDisk* disk,
+					  PedPartitionType type,
+					  const PedFileSystemType* fs_type,
+					  PedSector start,
+					  PedSector end);
 extern PedPartition* ped_partition_new (const PedDisk* disk,
 					PedPartitionType type,
-					PedFileSystemType* fs_type,
+					const PedFileSystemType* fs_type,
 					PedSector start,
 					PedSector end);
-extern PedPartition* ped_partition_duplicate (const PedPartition* part);
 extern void ped_partition_destroy (PedPartition* part);
 extern int ped_partition_is_active (const PedPartition* part);
-extern int ped_partition_get_hide_state (const PedPartition* part);
-extern int ped_partition_set_hide_state (PedPartition* part, int state);
+extern int ped_partition_set_flag (PedPartition* part, PedPartitionFlag flag,
+	       			   int state);
+extern int ped_partition_get_flag (const PedPartition* part,
+	       			   PedPartitionFlag flag);
+extern int ped_partition_is_flag_available (const PedPartition* part,
+	       			      	    PedPartitionFlag flag);
 extern int ped_partition_set_system (PedPartition* part,
-				     PedFileSystemType* fs_type);
+				     const PedFileSystemType* fs_type);
+extern int ped_partition_set_name (PedPartition* part, const char* name);
+extern const char* ped_partition_get_name (const PedPartition* part);
 extern int ped_partition_is_busy (const PedPartition* part);
 
-char* ped_partition_type_get_name (PedPartitionType type);
+extern const char* ped_partition_type_get_name (PedPartitionType part_type);
+extern const char* ped_partition_flag_get_name (PedPartitionFlag flag);
+extern PedPartitionFlag ped_partition_flag_get_by_name (const char* name);
+extern PedPartitionFlag ped_partition_flag_next (PedPartitionFlag flag);
 
-extern int ped_disk_add_partition (PedDisk* disk, PedPartition* part);
+extern int ped_disk_add_partition (PedDisk* disk, PedPartition* part,
+				   const PedConstraint* constraint);
+extern int ped_disk_remove_partition (PedDisk* disk, PedPartition* part);
 extern int ped_disk_delete_partition (PedDisk* disk, PedPartition* part);
 extern int ped_disk_delete_all (PedDisk* disk);
 extern int ped_disk_set_partition_geom (PedDisk* disk, PedPartition* part,
+					const PedConstraint* constraint,
 					PedSector start, PedSector end);
-extern int ped_disk_maximize_partition (PedDisk* disk, PedPartition* part);
+extern int ped_disk_maximize_partition (PedDisk* disk, PedPartition* part,
+					const PedConstraint* constraint);
+extern PedGeometry* ped_disk_get_max_partition_geometry (PedDisk* disk,
+		PedPartition* part, const PedConstraint* constraint);
 extern int ped_disk_minimize_extended_partition (PedDisk* disk);
 
 extern PedPartition* ped_disk_next_partition (const PedDisk* disk,
@@ -171,8 +239,6 @@ extern PedPartition* ped_disk_get_partition (const PedDisk* disk, int num);
 extern PedPartition* ped_disk_get_partition_by_sector (const PedDisk* disk,
 						       PedSector sect);
 extern PedPartition* ped_disk_extended_partition (const PedDisk* disk);
-extern PedPartition* ped_disk_get_boot_partition (const PedDisk* disk);
-extern int ped_disk_set_boot_partition (PedDisk* disk, PedPartition* part);
 
 #endif /* PED_DISK_H_INCLUDED */
 
