@@ -1,10 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998-2000  Andrew Clausen and Lennert Buytenhek
-
-	Andrew Clausen			<clausen@alphalink.com.au>
-	Lennert Buytenhek		<buytenh@scotty.dsv.nl>
-        Matt Wilson, Red Hat Inc.       <msw@redhat.com>
+    Copyright (C) 1998-2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
