@@ -1,10 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998-1999  Andrew Clausen, Lennert Buytenhek and Red Hat Inc.
-
-	Andrew Clausen			<clausen@gnu.org>
-	Lennert Buytenhek		<buytenh@gnu.org>
-        Matt Wilson, Red Hat Inc.       <msw@redhat.com>
+    Copyright (C) 1998-2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,21 +26,23 @@ typedef const struct _PedFileSystemOps	PedFileSystemOps;
 
 #include <parted/disk.h>
 #include <parted/exception.h>
+#include <parted/constraint.h>
 #include <stdio.h>
 
 struct _PedFileSystemOps {
 	int (*probe) (const PedGeometry* geom);
+	int (*clobber) (PedGeometry* geom);
 
 	PedFileSystem* (*open) (const PedGeometry* geom);
-	PedFileSystem* (*create) (const PedGeometry* geom);
+	PedFileSystem* (*create) (PedGeometry* geom);
 	int (*close) (PedFileSystem* fs);
 	int (*check) (PedFileSystem* fs);
-	int (*copy) (PedFileSystem* fs, PedGeometry* geom);
-	int (*resize) (PedFileSystem* fs, PedGeometry* geom);
+	int (*copy) (const PedFileSystem* fs, PedGeometry* geom);
+	int (*resize) (PedFileSystem* fs, const PedGeometry* geom);
 
-	PedSector (*get_min_size) (const PedFileSystem* fs);
-	char (*get_system) (const PedFileSystem* fs, const PedPartition* part,
-			    PedDiskType* disk_type);
+	PedConstraint* (*get_resize_constraint) (const PedFileSystem* fs);
+	int (*set_system) (const PedFileSystem* fs, PedPartition* part,
+			   const PedDiskType* disk_type);
 };
 
 struct _PedFileSystemType {
@@ -54,9 +52,9 @@ struct _PedFileSystemType {
 };
 
 struct _PedFileSystem {
-	PedFileSystemType*	type;
-	PedGeometry*		geom;
-	void*			type_specific;
+	PedFileSystemType*		type;
+	PedGeometry*			geom;
+	void*				type_specific;
 };
 
 extern void ped_file_system_type_register (PedFileSystemType* type);
@@ -67,17 +65,16 @@ extern PedFileSystemType*
 ped_file_system_type_get_next (PedFileSystemType* fs_type);
 
 extern PedFileSystemType* ped_file_system_probe (const PedGeometry* geom);
+extern int ped_file_system_clobber (PedGeometry* geom);
 
 extern PedFileSystem* ped_file_system_open (PedGeometry* geom);
 extern PedFileSystem* ped_file_system_create (PedGeometry* geom,
 					      PedFileSystemType* type);
 extern int ped_file_system_close (PedFileSystem* fs);
 extern int ped_file_system_check (PedFileSystem* fs);
-extern int ped_file_system_copy (PedFileSystem* fs, PedGeometry* geom);
-extern int ped_file_system_resize (PedFileSystem* fs, PedGeometry* geom);
-extern PedSector ped_file_system_get_min_size (const PedFileSystem* fs);
-extern char ped_file_system_get_system (const PedFileSystem* fs,
-					const PedPartition* part,
-					PedDiskType* disk_type);
+extern int ped_file_system_copy (const PedFileSystem* fs, PedGeometry* geom);
+extern int ped_file_system_resize (PedFileSystem* fs, const PedGeometry* geom);
+extern PedConstraint* ped_file_system_get_resize_constraint (
+				const PedFileSystem* fs);
 
 #endif /* PED_FILESYS_H_INCLUDED */
