@@ -1,10 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998-2000  Andrew Clausen, Lennert Buytenhek and Red Hat Inc.
-
-	Andrew Clausen			<clausen@alphalink.com.au>
-	Lennert Buytenhek		<buytenh@scotty.dsv.nl>
-	Matt Wilson, Red Hat Inc.	<msw@redhat.com>
+    Copyright (C) 1998-2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,16 +28,20 @@ extern "C" {
 // #define VERBOSE
 // #define READONLY
 
+#include <parted/constraint.h>
 #include <parted/debug.h>
 #include <parted/device.h>
 #include <parted/disk.h>
 #include <parted/exception.h>
 #include <parted/filesys.h>
+#include <parted/natmath.h>
 
 #include <stdlib.h>
 
 extern int ped_init ();
 extern void ped_done ();
+extern const char* ped_get_version ();
+
 extern void* ped_malloc (size_t size);
 extern int ped_realloc (void** ptr, size_t size);
 extern void ped_free (void* ptr);
