@@ -1,10 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998-2000  Andrew Clausen, Lennert Buytenhek and Red Hat Inc.
-
-	Andrew Clausen			<clausen@gnu.org>
-	Lennert Buytenhek		<buytenh@gnu.org>
-	Matt Wilson, Red Hat Inc.	<msw@redhat.com>
+    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,21 +22,19 @@
 #include <libintl.h>
 #define N_(String) String
 #if ENABLE_NLS
-#  define _(String) gettext (String)
+#  define _(String) dgettext (PACKAGE, String)
 #else
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
 
 #include "llseek.h"
 
+#include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <string.h>
 #include <string.h>
 #include <unistd.h>
-#include <linux/hdreg.h>
-#include <linux/major.h>
-#include <linux/fs.h>		/* FIXME: autoconfuse this */
 #include <sys/ioctl.h>
 #include <sys/sysmacros.h>
 #include <sys/stat.h>
@@ -52,16 +46,64 @@
 #define SCSI_IOCTL_SEND_COMMAND 1
 #endif
 
+#ifdef linux
+#include <linux/hdreg.h>
+
+/* from <linux/fs.h> */
+#define BLKRRPART  _IO(0x12,95)	/* re-read partition table */
+#define BLKGETSIZE _IO(0x12,96)	/* return device size */
+#define BLKFLSBUF  _IO(0x12,97)	/* flush buffer cache */
+#define BLKSSZGET  _IO(0x12,104) /* get block device sector size */
+
+/* from <linux/major.h> */
+#define IDE0_MAJOR		3
+#define IDE1_MAJOR		22
+#define IDE2_MAJOR		33
+#define IDE3_MAJOR		34
+#define IDE4_MAJOR		56
+#define IDE5_MAJOR		57
+#define SCSI_CDROM_MAJOR	11
+#define SCSI_DISK0_MAJOR	8
+#define SCSI_DISK1_MAJOR	65
+#define SCSI_DISK2_MAJOR	66
+#define SCSI_DISK3_MAJOR	67
+#define SCSI_DISK4_MAJOR	68
+#define SCSI_DISK5_MAJOR	69
+#define SCSI_DISK6_MAJOR	70
+#define SCSI_DISK7_MAJOR	71
+#define COMPAQ_SMART2_MAJOR	72
+#define COMPAQ_SMART2_MAJOR1	73
+#define COMPAQ_SMART2_MAJOR2	74
+#define COMPAQ_SMART2_MAJOR3	75
+#define COMPAQ_SMART2_MAJOR4	76
+#define COMPAQ_SMART2_MAJOR5	77
+#define COMPAQ_SMART2_MAJOR6	78
+#define COMPAQ_SMART2_MAJOR7	79
+#define DAC960_MAJOR		48
+
+#define SCSI_BLK_MAJOR(M) (						\
+		(M) == SCSI_DISK0_MAJOR 				\
+		|| (M) == SCSI_CDROM_MAJOR				\
+		|| ((M) >= SCSI_DISK1_MAJOR && (M) <= SCSI_DISK7_MAJOR))
+
+#endif /* linux */
+
 static PedDevice*	devices = NULL;
 
 static PedDevice*
 ped_device_new (const char* path);
 
+/* O(n) add function, hehe */
 static void
 ped_device_add (PedDevice* dev)
 {
-	dev->next = devices;
-	devices = dev;
+	PedDevice*	walk;
+	for (walk = devices; walk && walk->next; walk = walk->next);
+	if (walk)
+		walk->next = dev;
+	else
+		devices = dev;
+	dev->next = NULL;
 }
 
 static void
@@ -96,6 +138,9 @@ PedDevice*
 ped_device_get (const char* path)
 {
 	PedDevice*	walk;
+
+	PED_ASSERT (path != NULL, return NULL);
+
 	for (walk = devices; walk != NULL; walk = walk->next) {
 		if (!strcmp (walk->path, path))
 			return walk;
@@ -111,6 +156,9 @@ ped_device_get (const char* path)
 static int
 ped_device_stat (PedDevice* dev, struct stat * dev_stat)
 {
+	PED_ASSERT (dev != NULL, return 0);
+	PED_ASSERT (!dev->external_mode, return 0);
+
 	while (1) {
 		if (!stat (dev->path, dev_stat)) {
 			return 1;
@@ -145,65 +193,82 @@ strip_name(char* n)
 	return (strdup(n));
 }
 
+static int
+_is_ide_major (int major)
+{
+	switch (major) {
+		case IDE0_MAJOR:
+		case IDE1_MAJOR:
+		case IDE2_MAJOR:
+		case IDE3_MAJOR:
+		case IDE4_MAJOR:
+		case IDE5_MAJOR:
+			return 1;
+
+		default:
+			return 0;
+	}
+}
+
+static int
+_is_cpqarray_major (int major)
+{
+	return COMPAQ_SMART2_MAJOR <= major && major <= COMPAQ_SMART2_MAJOR7;
+}
+
 static int
 device_probe_type (PedDevice* dev)
 {
 	struct stat		dev_stat;
 	int			dev_major;
+	int			dev_minor;
 	PedExceptionOption	ex_status;
 
 	if (!ped_device_stat (dev, &dev_stat))
 		return 0;
 
 	if (!S_ISBLK(dev_stat.st_mode)) {
-		ex_status = ped_exception_throw (
-				PED_EXCEPTION_WARNING,
-				PED_EXCEPTION_IGNORE_CANCEL,
-				_("%s is not a block device."),
-				dev->path);
-		switch (ex_status) {
-			case PED_EXCEPTION_CANCEL:
-				return 0;
-
-			case PED_EXCEPTION_UNHANDLED:
-				ped_exception_catch ();
-			case PED_EXCEPTION_IGNORE:
-				dev->type = PED_DEVICE_UNKNOWN;
-				return 1;
-		}
+		dev->type = PED_DEVICE_FILE;
+		return 1;
 	}
 
 	dev_major = major (dev_stat.st_rdev);
+	dev_minor = minor (dev_stat.st_rdev);
 
-	if (SCSI_BLK_MAJOR (dev_major) && ((dev_stat.st_rdev & 0x0f) == 0)) {
+	if (SCSI_BLK_MAJOR (dev_major) && (dev_minor % 0x10 == 0)) {
 		dev->type = PED_DEVICE_SCSI;
-		return 1;
-	}
-
-	if (((dev_major == IDE0_MAJOR) || (dev_major == IDE1_MAJOR)
-		  || (dev_major == IDE2_MAJOR) || (dev_major == IDE3_MAJOR))
-	    && ((dev_stat.st_rdev & 0x3f) == 0)) {
+	} else if (_is_ide_major (dev_major) && (dev_minor % 0x40 == 0)) {
 		dev->type = PED_DEVICE_IDE;
-		return 1;
+	} else if (dev_major == DAC960_MAJOR && (dev_minor % 0x8 == 0)) {
+		dev->type = PED_DEVICE_DAC960;
+	} else if (_is_cpqarray_major (dev_major) && (dev_minor % 0x10 == 0)) {
+		dev->type = PED_DEVICE_CPQARRAY;
+	} else {
+		dev->type = PED_DEVICE_UNKNOWN;
 	}
 
-	ex_status = ped_exception_throw (
-			PED_EXCEPTION_WARNING,
-			PED_EXCEPTION_IGNORE_CANCEL,
-			_("Device %s is neither a SCSI nor IDE drive."),
-			dev->path);
-	switch (ex_status) {
-		case PED_EXCEPTION_CANCEL:
-			return 0;
-
-		case PED_EXCEPTION_UNHANDLED:
-			ped_exception_catch ();
-		case PED_EXCEPTION_IGNORE:
-			dev->type = PED_DEVICE_UNKNOWN;
-			return 1;
-	}
+	return 1;
 }
 
+#ifdef linux
+static int
+_get_linux_version ()
+{
+	FILE*	f;
+	int	major;
+	int	minor;
+
+	f = fopen ("/proc/sys/kernel/osrelease", "r");
+	if (!f)
+		return 0;
+	if (fscanf (f, "%d.%d.", &major, &minor) != 2)
+		return 0;
+	fclose (f);
+
+	return major * 0x100 + minor;
+}
+#endif /* linux */
+
 static int
 device_get_sector_size (PedDevice* dev)
 {
@@ -211,7 +276,9 @@ device_get_sector_size (PedDevice* dev)
 
 	PED_ASSERT (dev->open_count, return 0);
 
-#ifdef USE_BLKSSZGET
+#ifdef linux
+	if (_get_linux_version() < 0x203)    /* BLKSSZGET is broken < 2.3.x */
+		return 512;
 	if (ioctl (dev->fd, BLKSSZGET, &sector_size))
 		return 512;
 
@@ -229,11 +296,15 @@ device_get_sector_size (PedDevice* dev)
 		else
 			return 0;
 	}
-#else /* USE_BLKSSZGET */
+
+	return sector_size;
+
+#else /* linux */
 	return 512;
-#endif /* !USE_BLKSSZGET */
+#endif /* !linux */
 }
 
+/* TODO: do a binary search if BLKGETSIZE doesn't work?! */
 static PedSector
 device_get_length (PedDevice* dev)
 {
@@ -404,7 +475,28 @@ error:
 }
 
 static int
-init_unknown (PedDevice* dev)
+init_file (PedDevice* dev)
+{
+	struct stat		dev_stat;
+
+	if (!ped_device_stat (dev, &dev_stat))
+		goto error;
+
+	dev->length = dev_stat.st_size / 512;
+	dev->cylinders = dev->length / 4 / 32;
+	dev->heads = 4;
+	dev->sectors = 32;
+	dev->sector_size = 512;
+	dev->geom_known = 0;
+	dev->model = "";
+	return 1;
+
+error:
+	return NULL;
+}
+
+static int
+init_generic (PedDevice* dev, char* model_name)
 {
 	struct stat		dev_stat;
 	struct hd_geometry	geometry;
@@ -445,8 +537,11 @@ init_unknown (PedDevice* dev)
 		dev->heads = 4;
 		dev->sectors = 32;
 		dev->sector_size = 512;
+		dev->geom_known = 0;
 	}
 
+	dev->model = model_name;
+
 	ped_device_close (dev);
 	return 1;
 
@@ -469,8 +564,11 @@ ped_device_new (const char* path)
 
 	dev->path = strdup (path);
 	dev->open_count = 0;
+	dev->external_mode = 0;
 	dev->dirty = 0;
+	dev->boot_dirty = 0;
 	dev->geom_known = 1;
+	dev->geom_already_guessed = 0;
 
 	if (!device_probe_type (dev))
 		goto error_free_dev;
@@ -486,8 +584,30 @@ ped_device_new (const char* path)
 			goto error_free_dev;
 		break;
 
+	case PED_DEVICE_DAC960:
+		if (!init_generic (dev, _("DAC960 RAID controller")))
+			goto error_free_dev;
+		break;
+
+	case PED_DEVICE_CPQARRAY:
+		if (!init_generic (dev, _("Compaq Smart Array")))
+			goto error_free_dev;
+		break;
+
+	case PED_DEVICE_FILE:
+		if (!init_file (dev))
+			goto error_free_dev;
+		break;
+
 	case PED_DEVICE_UNKNOWN:
-		if (!init_unknown (dev))
+		if (ped_exception_throw (
+			PED_EXCEPTION_WARNING,
+			PED_EXCEPTION_IGNORE_CANCEL,
+			_("Device %s is neither a SCSI nor IDE drive."),
+			dev->path)
+				== PED_EXCEPTION_CANCEL)
+			goto error_free_dev;
+		if (!init_generic (dev, _("Unknown")))
 			goto error_free_dev;
 		break;
 
@@ -517,59 +637,158 @@ ped_device_destroy (PedDevice* dev)
 	ped_free (dev);
 }
 
-int
-ped_device_open (PedDevice* dev)
+#ifdef linux
+static void
+_flush_cache (PedDevice* dev)
 {
-	PED_ASSERT (dev != NULL, return 0);
+	int		i;
+	char*		name;
+	int		fd;
 
-	if (dev->open_count++)
-		return 1;
+	if (dev->read_only)
+		return;
 
-	dev->fd = open (dev->path, O_RDWR);
-	if (dev->fd == -1) {
-		ped_exception_throw (
-			PED_EXCEPTION_ERROR,
-			PED_EXCEPTION_CANCEL,
-			"Error opening %s: %s",
-			dev->path, strerror (errno));
-		return 0;
+	ioctl (dev->fd, BLKFLSBUF);
+
+	name = (char*) ped_malloc (strlen (dev->path) + 3);
+	if (!name)
+		return;
+
+	for (i = 0; i < 16; i++) {
+		sprintf (name, "%s%d", dev->path, i);
+		fd = open (name, O_WRONLY, 0);
+		if (fd == -1)
+			continue;
+		ioctl (fd, BLKFLSBUF);
+		close (fd);
+	}
+
+	ped_free (name);
+}
+
+static int
+_kernel_reread_part_table (PedDevice* dev)
+{
+	int	retry_count = 5;
+
+	sync();
+	while (ioctl (dev->fd, BLKRRPART)) {
+		retry_count--;
+		sync();
+	    	if (!retry_count)
+			return 0;
 	}
 
 	return 1;
 }
 
-int
-ped_device_close (PedDevice* dev)
+#endif /* linux */
+
+static int
+_do_refresh_open (PedDevice* dev)
 {
-	int			retry_count = 5;	/* a bad guess */
-	PedExceptionOption	ex_status;
+#ifdef linux
+	_flush_cache (dev);
+#endif
+	return 1;
+}
 
-	PED_ASSERT (dev != NULL, return 0);
+static int
+_do_open (PedDevice* dev)
+{
+	char*		rw_error_msg;
 
-	if (--dev->open_count) return 1;
+retry:
+	dev->fd = open (dev->path, O_RDWR);
+	if (dev->fd == -1) {
+		rw_error_msg = strerror (errno);
+
+		dev->fd = open (dev->path, O_RDONLY);
+		if (dev->fd == -1) {
+			if (ped_exception_throw (
+				PED_EXCEPTION_ERROR,
+				PED_EXCEPTION_RETRY_CANCEL,
+				_("Error opening %s: %s"),
+				dev->path, strerror (errno))
+					!= PED_EXCEPTION_RETRY)
+				return 0;
+			else
+				goto retry;
+		} else {
+			ped_exception_throw (
+				PED_EXCEPTION_WARNING,
+				PED_EXCEPTION_OK,
+				_("Unable to open %s read-write (%s).  %s has "
+				  "been opened read-only."),
+				dev->path, rw_error_msg, dev->path);
+			dev->read_only = 1;
+		}
+	} else {
+		dev->read_only = 0;
+	}
 
 #ifdef linux
-	if (dev->dirty && dev->type != PED_DEVICE_UNKNOWN) {
-		sync();
-		while (ioctl (dev->fd, BLKRRPART)) {
-			retry_count--;
-			sync();
-	    		if (!retry_count) {
-				ex_status = ped_exception_throw (
-					PED_EXCEPTION_WARNING,
-					PED_EXCEPTION_IGNORE,
-		_("The kernel was unable to re-read the partition "
-		  "table on %s (%s).  This means Linux knows nothing "
-		  "about any modifications you made.  You should "
-		  "reboot your computer before doing anything with %s."),
-					dev->path, strerror (errno), dev->path);
+	_flush_cache (dev);
+#endif
 
-				if (ex_status == PED_EXCEPTION_UNHANDLED)
-					ped_exception_catch ();
+	return 1;
+}
 
-				break;
-	    		}
-		}
+static int
+_do_refresh_close (PedDevice* dev)
+{
+	return 1;
+}
+
+static int
+_do_close (PedDevice* dev)
+{
+#ifdef linux
+	_flush_cache (dev);
+
+	if (dev->dirty && dev->type != PED_DEVICE_FILE) {
+		if (_kernel_reread_part_table (dev))
+			dev->dirty = 0;
+	}
+
+	if (dev->dirty && dev->boot_dirty && dev->type != PED_DEVICE_FILE) {
+		/* ouch! */
+		ped_exception_throw (
+			PED_EXCEPTION_WARNING,
+			PED_EXCEPTION_OK,
+			_("The kernel was unable to re-read your partition "
+			  "table, so you need to reboot before mounting any "
+			  "modified partitions.  You also need to reinstall "
+			  "your boot loader before you reboot (which may "
+			  "require mounting modified partitions).  It is "
+			  "impossible do both things!  So you'll need to "
+			  "boot off a rescue disk, and reinstall your boot "
+			  "loader from the rescue disk.  Read section 4 of "
+			  "the Parted User documentation for more "
+			  "information."));
+		close (dev->fd);
+		return 1;
+	}
+
+	if (dev->dirty && dev->type != PED_DEVICE_FILE) {
+		ped_exception_throw (
+			PED_EXCEPTION_WARNING,
+			PED_EXCEPTION_IGNORE,
+			_("The kernel was unable to re-read the partition "
+			  "table on %s (%s).  This means Linux knows nothing "
+			  "about any modifications you made.  You should "
+			  "reboot your computer before doing anything with "
+			  "%s."),
+			dev->path, strerror (errno), dev->path);
+	}
+
+	if (dev->boot_dirty && dev->type != PED_DEVICE_FILE) {
+		ped_exception_throw (
+			PED_EXCEPTION_WARNING,
+			PED_EXCEPTION_OK,
+			_("You should reinstall your boot loader before "
+			  "rebooting.  Read section 4 of the Parted User "
+			  "documentation for more information."));
 	}
 #endif /* linux */
 
@@ -577,10 +796,62 @@ ped_device_close (PedDevice* dev)
 	return 1;
 }
 
+int
+ped_device_open (PedDevice* dev)
+{
+	PED_ASSERT (dev != NULL, return 0);
+	PED_ASSERT (!dev->external_mode, return 0);
+
+	if (dev->open_count++)
+		return _do_refresh_open (dev);
+
+	return _do_open (dev);
+}
+
+int
+ped_device_close (PedDevice* dev)
+{
+	PED_ASSERT (dev != NULL, return 0);
+	PED_ASSERT (!dev->external_mode, return 0);
+
+	if (--dev->open_count)
+		return _do_refresh_close (dev);
+
+	return _do_close (dev);
+
+}
+
+int
+ped_device_begin_external_access (PedDevice* dev)
+{
+	PED_ASSERT (dev != NULL, return 0);
+	PED_ASSERT (!dev->external_mode, return 0);
+
+	dev->external_mode = 1;
+	if (dev->open_count)
+		_do_close (dev);
+
+	return 1;
+}
+
+int
+ped_device_end_external_access (PedDevice* dev)
+{
+	PED_ASSERT (dev != NULL, return 0);
+	PED_ASSERT (dev->external_mode, return 0);
+
+	dev->external_mode = 0;
+	if (dev->open_count)
+		_do_open (dev);
+
+	return 1;
+}
+
 static int
 ped_device_seek (const PedDevice* dev, PedSector sector)
 {
 	PED_ASSERT (dev != NULL, return 0);
+	PED_ASSERT (!dev->external_mode, return 0);
 
 #if SIZEOF_OFF_T < 8 && defined(linux)
 	if (sizeof (off_t) < 8) {
@@ -603,6 +874,7 @@ ped_device_read (const PedDevice* dev, void* buffer, PedSector start,
 	size_t			read_length = count * PED_SECTOR_SIZE;
 
 	PED_ASSERT (dev != NULL, return 0);
+	PED_ASSERT (!dev->external_mode, return 0);
 	PED_ASSERT (buffer != NULL, return 0);
 
 	while (1) {
@@ -669,6 +941,10 @@ ped_device_sync (PedDevice* dev)
 	PedExceptionOption	ex_status;
 
 	PED_ASSERT (dev != NULL, return 0);
+	PED_ASSERT (!dev->external_mode, return 0);
+
+	if (dev->read_only)
+		return 1;
 
 	while (1) {
 		status = fsync (dev->fd);
@@ -706,8 +982,21 @@ ped_device_write (PedDevice* dev, const void* buffer, PedSector start,
 	size_t			write_length = count * PED_SECTOR_SIZE;
 
 	PED_ASSERT (dev != NULL, return 0);
+	PED_ASSERT (!dev->external_mode, return 0);
 	PED_ASSERT (buffer != NULL, return 0);
 
+	if (dev->read_only) {
+		if (ped_exception_throw (
+			PED_EXCEPTION_ERROR,
+			PED_EXCEPTION_IGNORE_CANCEL,
+			_("Can't write to %s, because it is opened read-only."),
+			dev->path)
+				!= PED_EXCEPTION_IGNORE)
+			return 0;
+		else
+			return 1;
+	}
+
 	while (1) {
 		if (ped_device_seek (dev, start))
 			break;
@@ -778,6 +1067,7 @@ ped_device_check (PedDevice* dev, void* buffer, PedSector start,
 	int			done = 0;
 
 	PED_ASSERT (dev != NULL, return 0);
+	PED_ASSERT (!dev->external_mode, return 0);
 	PED_ASSERT (buffer != NULL, return 0);
 
 	if (!ped_device_seek (dev, start))
@@ -832,25 +1122,61 @@ error:
 	ped_exception_leave_all ();
 }
 
-/* FIXME */
+static int
+probe_proc_partitions ()
+{
+	FILE*		proc_part_file;
+	int		major, minor, size;
+	char		part_name [256];
+	char		dev_name [256];
+
+	proc_part_file = fopen ("/proc/partitions", "r");
+	if (!proc_part_file)
+		return 0;
+
+	fgets (part_name, 256, proc_part_file);
+	fgets (part_name, 256, proc_part_file);
+
+	while (fscanf (proc_part_file, "%d %d %d %255s",
+		       &major, &minor, &size, part_name) == 4) {
+		if (isdigit (part_name [strlen (part_name) - 1]))
+			continue;
+		strcpy (dev_name, "/dev/");
+		strcat (dev_name, part_name);
+		probe (dev_name);
+	}
+
+	fclose (proc_part_file);
+	return 1;
+}
+
+static int
+probe_standard_devices ()
+{
+	probe ("/dev/sda");
+	probe ("/dev/sdb");
+	probe ("/dev/sdc");
+	probe ("/dev/sdd");
+	probe ("/dev/sde");
+	probe ("/dev/sdf");
+
+	probe ("/dev/hda");
+	probe ("/dev/hdb");
+	probe ("/dev/hdc");
+	probe ("/dev/hdd");
+	probe ("/dev/hde");
+	probe ("/dev/hdf");
+	probe ("/dev/hdg");
+	probe ("/dev/hdh");
+
+	return 1;
+}
+
 void
 ped_device_probe_all ()
 {
-	probe ("/dev/hdh");
-	probe ("/dev/hdg");
-	probe ("/dev/hdf");
-	probe ("/dev/hde");
-	probe ("/dev/hdd");
-	probe ("/dev/hdc");
-	probe ("/dev/hdb");
-	probe ("/dev/hda");
-
-	probe ("/dev/sdf");
-	probe ("/dev/sde");
-	probe ("/dev/sdd");
-	probe ("/dev/sdc");
-	probe ("/dev/sdb");
-	probe ("/dev/sda");
+	probe_proc_partitions ();
+	probe_standard_devices ();
 }
 
 void
