@@ -1,10 +1,6 @@
  /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998-2000  Andrew Clausen, Lennert Buytenhek and Red Hat Inc.
-
-	Andrew Clausen			<clausen@gnu.org>
-	Lennert Buytenhek		<buytenh@gnu.org>
-	Matt Wilson, Red Hat Inc.	<msw@redhat.com>
+    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,9 +23,11 @@
 
 #include <libintl.h>
 #if ENABLE_NLS
-#  define _(String) gettext (String)
+#  define _(String) dgettext (PACKAGE, String)
+#  define N_(String) (String)
 #else
 #  define _(String) (String)
+#  define N_(String) (String)
 #endif /* ENABLE_NLS */
 
 #include <errno.h>
@@ -38,16 +36,16 @@
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 
-#ifdef linux
-#include <linux/fs.h>
-#endif
-
 /* UPDATE MODE functions */
+static int _disk_check_sanity (PedDisk* disk);
+static void _disk_push_update_mode (PedDisk* disk);
+static void _disk_pop_update_mode (PedDisk* disk);
 static int _disk_raw_insert_before (PedDisk* disk, PedPartition* loc,
 				    PedPartition* part);
 static int _disk_raw_insert_after (PedDisk* disk, PedPartition* loc,
 				   PedPartition* part);
 static int _disk_raw_remove (PedDisk* disk, PedPartition* part);
+static int _disk_raw_add (PedDisk* disk, PedPartition* part);
 
 static PedDiskType*	disk_types = NULL;
 
@@ -92,6 +90,9 @@ PedDiskType*
 ped_disk_type_get (const char* name)
 {
 	PedDiskType*	walk = NULL;
+
+	PED_ASSERT (name != NULL, return NULL);
+
 	while (1) {
 		walk = ped_disk_type_get_next (walk);
 		if (!walk) break;
@@ -107,7 +108,7 @@ ped_disk_probe (const PedDevice* dev)
 
 	PED_ASSERT (dev != NULL, return NULL);
 
-	ped_exception_leave_all ();
+	ped_exception_fetch_all ();
 
 	while (1) {
 		walk = ped_disk_type_get_next (walk);
@@ -117,7 +118,7 @@ ped_disk_probe (const PedDevice* dev)
 
 	if (ped_exception)
 		ped_exception_catch ();
-	ped_exception_fetch_all ();
+	ped_exception_leave_all ();
 	return walk;
 }
 
@@ -125,6 +126,7 @@ PedDisk*
 ped_disk_open (PedDevice* dev)
 {
 	PedDiskType*	type;
+	PedDisk*	disk;
 
 	PED_ASSERT (dev != NULL, return NULL);
 
@@ -135,21 +137,35 @@ ped_disk_open (PedDevice* dev)
 			dev->path);
 		return 0;
 	}
-	return type->ops->open (dev);
+	disk = type->ops->open (dev);
+	if (!disk)
+		goto error;
+	return disk;
+
+error:
+	return NULL;
 }
 
 PedDisk*
 ped_disk_create (PedDevice* dev, PedDiskType* type)
 {
+	PedDiskType*	old_type;
+	PedDisk*	disk;
+
 	PED_ASSERT (dev != NULL, return NULL);
 	PED_ASSERT (type != NULL, return NULL);
 
+	while ((old_type = ped_disk_probe (dev))) {
+		PED_ASSERT (old_type->ops->clobber != NULL, return NULL);
+		old_type->ops->clobber (dev);
+	}
+
 	if (!type->ops->create) {
 		ped_exception_throw (PED_EXCEPTION_NO_FEATURE,
 			PED_EXCEPTION_CANCEL,
 			_("Creating new %s disklabels is not implemented yet."),
 			type->name);
-		return 0;
+		return NULL;
 	}
 	return type->ops->create (dev);
 }
@@ -161,20 +177,40 @@ ped_disk_alloc (PedDevice* dev, PedDiskType* disk_type)
 
 	disk = (PedDisk*) ped_malloc (sizeof (PedDisk));
 	if (!disk)
-		return NULL;
+		goto error;
 
 	disk->dev = dev;
 	disk->type = disk_type;
-	disk->part_list = NULL;
 	disk->update_mode = 0;
 
+	disk->part_list = ped_partition_new (
+		disk, PED_PARTITION_FREESPACE, NULL,
+		0, dev->length - 1);
+	if (!disk->part_list)
+		goto error_free_disk;
+
 	return disk;
+
+error_free_disk:
+	ped_free (disk);
+error:
+	return NULL;
+}
+
+void
+ped_disk_free (PedDisk* disk)
+{
+	_disk_push_update_mode (disk);
+	ped_disk_delete_all (disk);
+	ped_free (disk);
 }
 
 int
 ped_disk_close (PedDisk* disk)
 {
 	PED_ASSERT (disk != NULL, return 0);
+	PED_ASSERT (!disk->update_mode, return 0);
+
 	return disk->type->ops->close (disk);
 }
 
@@ -182,6 +218,8 @@ int
 ped_disk_read (PedDisk* disk)
 {
 	PED_ASSERT (disk != NULL, return 0);
+	PED_ASSERT (!disk->update_mode, return 0);
+
 	return disk->type->ops->read (disk);
 }
 
@@ -189,6 +227,7 @@ int
 ped_disk_write (PedDisk* disk)
 {
 	PED_ASSERT (disk != NULL, return 0);
+	PED_ASSERT (!disk->update_mode, return 0);
 
 	if (!disk->type->ops->write) {
 		ped_exception_throw (
@@ -205,34 +244,96 @@ ped_disk_write (PedDisk* disk)
 	return 1;
 }
 
-static int
-ped_disk_align_partition (PedDisk* disk, PedPartition* part)
+int
+ped_disk_type_check_feature (const PedDiskType* disk_type,
+			     PedDiskTypeFeature feature)
 {
-	PED_ASSERT (disk != NULL, return 0);
+	return (disk_type->features & feature) != 0;
+}
+
+int
+ped_disk_get_max_primary_partition_count (const PedDisk* disk)
+{
+	PED_ASSERT (disk->type != NULL, return 0);
+	PED_ASSERT (disk->type->ops->get_max_primary_partition_count != NULL,
+		    return 0);
+
+	return disk->type->ops->get_max_primary_partition_count (disk);
+}
+
+/* We turned a really nasty beaurocracy problem into an elegant maths
+ * problem :-)  Basically, there are some constraints to a partition's
+ * geometry:
+ *
+ * (1) it must start and end on a "disk" block, determined by the disk label
+ * (not the hardware).  (constraint represented by a PedAlignment)
+ *
+ * (2) if we're resizing a partition, we MIGHT need to keep each block aligned.
+ * Eg: if an ext2 file system has 4k blocks, then we can only move the start
+ * by a multiple of 4k.  (constraint represented by a PedAlignment)
+ *
+ * (3) we need to keep the start and end within the device's physical
+ * boundaries.  (constraint represented by a PedGeometry)
+ *
+ * Satisfying (1) and (2) simultaneously required a bit of fancy maths ;-)  See
+ * ped_alignment_intersect()
+ *
+ * The application of these constraints is in disk_*.c's *_partition_align()
+ * function.
+ */
+static int
+_partition_align (PedPartition* part, const PedConstraint* constraint)
+{
+	PedDiskType*	disk_type;
+ 	int		status;
+
 	PED_ASSERT (part != NULL, return 0);
+	PED_ASSERT (part->num != -1, return 0);
+	PED_ASSERT (part->geom.disk != NULL, return 0);
+	disk_type = part->geom.disk->type;
+	PED_ASSERT (disk_type != NULL, return 0);
+	PED_ASSERT (disk_type->ops->partition_align != NULL, return 0);
+	PED_ASSERT (part->geom.disk->update_mode, return 0);
 
-	return disk->type->ops->align_partition (disk, part);
+	status = disk_type->ops->partition_align (part, constraint);
+ 	PED_ASSERT (part->geom.start <= part->geom.end, return 0);
+ 	return status;
 }
 
 static int
-ped_disk_enumerate_partition (PedDisk* disk, PedPartition* part)
+_partition_enumerate (PedPartition* part)
 {
-	PED_ASSERT (disk != NULL, return 0);
+	PedDiskType*	disk_type;
+
 	PED_ASSERT (part != NULL, return 0);
+	PED_ASSERT (part->geom.disk != NULL, return 0);
+	disk_type = part->geom.disk->type;
+	PED_ASSERT (disk_type != NULL, return 0);
+	PED_ASSERT (disk_type->ops->partition_enumerate != NULL, return 0);
 
-	return disk->type->ops->enumerate_partition (disk, part);
-}
-
-static char
-ped_disk_get_extended_system (const PedDisk* disk)
-{
-	PED_ASSERT (disk != NULL, return 0);
-
-	return disk->type->ops->get_extended_system ();
+	return disk_type->ops->partition_enumerate (part);
 }
 
 static int
-ped_disk_last_partition_num (PedDisk* disk)
+_disk_count_primary_partitions (PedDisk* disk)
+{
+	PedPartition*	walk;
+	int		count = 0;
+
+	PED_ASSERT (disk != NULL, return 0);
+
+	for (walk = disk->part_list; walk;
+	     walk = ped_disk_next_partition (disk, walk)) {
+		if (walk->type == PED_PARTITION_PRIMARY
+		    || walk->type == PED_PARTITION_EXTENDED)
+			count++;
+	}
+
+	return count;
+}
+
+static int
+_disk_last_partition_num (PedDisk* disk)
 {
 	PedPartition*	walk;
 	int		highest = -1;
@@ -264,19 +365,19 @@ ped_disk_enumerate_partitions (PedDisk* disk)
  * is removed, then all logical partitions that were number higher MUST be
  * renumbered)
  */
-	end = ped_disk_last_partition_num (disk);
+	end = _disk_last_partition_num (disk);
 	for (i=1; i<=end; i++) {
 		walk = ped_disk_get_partition (disk, i);
 		if (!walk)
 			continue;
-		ped_disk_enumerate_partition (disk, walk);
+		_partition_enumerate (walk);
 	}
 
 /* now, number un-numbered partitions */
 	for (walk = disk->part_list; walk;
 	     walk = ped_disk_next_partition (disk, walk)) {
 		if (ped_partition_is_active (walk)) {
-			if (!ped_disk_enumerate_partition (disk, walk))
+			if (!_partition_enumerate (walk))
 				return 0;
 		}
 	}
@@ -328,8 +429,10 @@ _disk_remove_freespace (PedDisk* disk)
 	for (; walk; walk = next) {
 		next = ped_disk_next_partition (disk, walk);
 
-		if (walk->type & PED_PARTITION_FREESPACE)
+		if (walk->type & PED_PARTITION_FREESPACE) {
 			_disk_raw_remove (disk, walk);
+			ped_partition_destroy (walk);
+		}
 	}
 
 	return 1;
@@ -432,9 +535,13 @@ static void
 _disk_push_update_mode (PedDisk* disk)
 {
 	if (!disk->update_mode) {
+		_disk_check_sanity (disk);
+
 		_disk_remove_freespace (disk);
 		disk->update_mode++;
 		_disk_remove_metadata (disk);
+
+		_disk_check_sanity (disk);
 	} else {
 		disk->update_mode++;
 	}
@@ -443,81 +550,107 @@ _disk_push_update_mode (PedDisk* disk)
 static void
 _disk_pop_update_mode (PedDisk* disk)
 {
+	PED_ASSERT (disk->update_mode, return);
+
 	if (disk->update_mode == 1) {
 	/* re-allocate metadata BEFORE leaving update mode, to prevent infinite
 	 * recursion (metadata allocation requires update mode)
 	 */
+		_disk_check_sanity (disk);
+
 		_disk_alloc_metadata (disk);
 		disk->update_mode--;
 		_disk_alloc_freespace (disk);
+
+		_disk_check_sanity (disk);
 	} else {
 		disk->update_mode--;
 	}
 }
 
 PedPartition*
-ped_partition_new (const PedDisk* disk, PedPartitionType type,
-		   PedFileSystemType* fs_type, PedSector start, PedSector end)
+ped_partition_alloc (const PedDisk* disk, PedPartitionType type,
+		     const PedFileSystemType* fs_type,
+		     PedSector start, PedSector end)
 {
 	PedPartition*	part;
 
 	PED_ASSERT (disk != NULL, return 0);
 
-	if (end < start) {
-		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-		   _("Can't create a partition with the start after the end."));
-		return NULL;
-	}
-
 	part = (PedPartition*) ped_malloc (sizeof (PedPartition));
 	if (!part)
-		return NULL;
+		goto error;
 
 	part->prev = NULL;
 	part->next = NULL;
 
 	part->geom.disk = (PedDisk*) disk;
-	ped_geometry_set (&part->geom, start, end - start + 1);
+	if (!ped_geometry_set (&part->geom, start, end - start + 1))
+		goto error_free_part;
 
 	part->type = type;
 	part->part_list = NULL;
+	part->fs_type = fs_type;
 
-	part->bootable = 0;
-	part->hidden = 0;
-	if (type & PED_PARTITION_EXTENDED) {
-		part->system = ped_disk_get_extended_system (disk);
-	} else if (fs_type) {
-		part->system = fs_type->ops->get_system (NULL, part,
-							 disk->type);
-	} else {
-		part->system = 0;
-	}
-
-	part->fs_type = NULL;
 	return part;
+
+error_free_part:
+	ped_free (part);
+error:
+	return NULL;
 }
 
 PedPartition*
-ped_partition_duplicate (const PedPartition* part)
+ped_partition_new (const PedDisk* disk, PedPartitionType type,
+		   const PedFileSystemType* fs_type, PedSector start,
+		   PedSector end)
 {
-	PedPartition*	result;
+	int		supports_extended;
+	PedPartition*	part;
 
-	PED_ASSERT (part != NULL, return 0);
+	PED_ASSERT (disk != NULL, return NULL);
+	PED_ASSERT (disk->type->ops->partition_new != NULL, return NULL);
 
-	result = ped_partition_new (part->geom.disk, part->type, NULL,
-				    part->geom.start, part->geom.end);
-	if (!result)
-		return NULL;
-	result->system = part->system;
-	return result;
+	supports_extended = ped_disk_type_check_feature (disk->type,
+			    	PED_DISK_TYPE_EXTENDED);
+
+	if (!supports_extended && type == PED_PARTITION_EXTENDED) {
+		ped_exception_throw (
+			PED_EXCEPTION_ERROR,
+			PED_EXCEPTION_CANCEL,
+			_("%s disk labels do not support extended "
+			  "partitions."),
+			disk->type->name);
+		goto error;
+	}
+
+	part = disk->type->ops->partition_new (disk, type, fs_type, start, end);
+	if (!part)
+		goto error;
+
+	if (fs_type || part->type == PED_PARTITION_EXTENDED) {
+		if (!ped_partition_set_system (part, fs_type))
+			goto error;
+	}
+	return part;
+
+error_destroy_part:
+	ped_partition_destroy (part);
+error:
+	return NULL;
 }
 
 void
 ped_partition_destroy (PedPartition* part)
 {
-	PED_ASSERT (part != NULL, return);
+	PedDisk*	disk;
 
-	ped_free (part);
+	PED_ASSERT (part != NULL, return);
+	disk = part->geom.disk;
+	PED_ASSERT (disk != NULL, return);
+	PED_ASSERT (disk->type->ops->partition_new != NULL, return);
+
+	disk->type->ops->partition_destroy (part);
 }
 
 int
@@ -530,63 +663,94 @@ ped_partition_is_active (const PedPartition* part)
 }
 
 int
-ped_partition_get_hide_state (const PedPartition* part)
+ped_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 {
-	PED_ASSERT (part != NULL, return 0);
+	PedDisk*	disk;
 
-	return part->hidden;
+	PED_ASSERT (part != NULL, return 0);
+	disk = part->geom.disk;
+	PED_ASSERT (disk != NULL, return 0);
+	PED_ASSERT (disk->type->ops->partition_set_flag != NULL, return 0);
+	PED_ASSERT (disk->type->ops->partition_is_flag_available != NULL,
+		    return 0);
+	PED_ASSERT (ped_partition_is_active (part), return 0);
+
+	if (!disk->type->ops->partition_is_flag_available (part, flag)) {
+		ped_exception_throw (
+			PED_EXCEPTION_ERROR,
+			PED_EXCEPTION_CANCEL,
+			"The flag '%s' is not available for %s disk labels.",
+			ped_partition_flag_get_name (flag),
+			disk->type->name);
+		return 0;
+	}
+
+	if (!disk->type->ops->partition_set_flag (part, flag, state))
+		return 0;
+	return 1;
 }
 
 int
-ped_partition_set_hide_state (PedPartition* part, int state)
+ped_partition_get_flag (const PedPartition* part, PedPartitionFlag flag)
 {
-	PedFileSystem*		fs;
-	char			old_hide_state = part->hidden;
-	char			new_system;
+	PedDisk*	disk;
 
 	PED_ASSERT (part != NULL, return 0);
+	disk = part->geom.disk;
+	PED_ASSERT (disk != NULL, return 0);
+	PED_ASSERT (disk->type->ops->partition_get_flag != NULL, return 0);
+	PED_ASSERT (ped_partition_is_active (part), return 0);
 
-	if (part->type & PED_PARTITION_EXTENDED)
-		return 0;
-	part->hidden = state;
+	return disk->type->ops->partition_get_flag (part, flag);
+}
 
-	ped_exception_fetch_all();
-	fs = ped_file_system_open (&part->geom);
-	if (!fs) {
-		ped_exception_catch();
-		ped_exception_leave_all();
+int
+ped_partition_is_flag_available (const PedPartition* part,
+	       			 PedPartitionFlag flag)
+{
+	PedDisk*	disk;
 
-		if (part->fs_type) {
-			/* hack! */
-			new_system = part->fs_type->ops->get_system (NULL, part,
-							part->geom.disk->type);
-			if (new_system) {
-				part->system = new_system;
-				return 1;
-			}
-		}
+	PED_ASSERT (part != NULL, return 0);
+	disk = part->geom.disk;
+	PED_ASSERT (disk != NULL, return 0);
+	PED_ASSERT (disk->type->ops->partition_is_flag_available != NULL,
+		    return 0);
+	PED_ASSERT (ped_partition_is_active (part), return 0);
 
-		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-				_("No filesystem found on partition.  Can't "
-				  "determine what partition system to use."));
-		goto error;
+	return disk->type->ops->partition_is_flag_available (part, flag);
+}
+
+static int
+_fs_set_system (const PedFileSystemType* fs_type, PedFileSystem* fs,
+		PedPartition* part)
+{
+	PedDiskType*	disk_type = part->geom.disk->type;
+	if (fs)
+		PED_ASSERT (fs->type == fs_type, return 0);
+
+	if (fs_type->ops->set_system (fs, part, disk_type)) {
+		return 1;
+	} else {
+		const PedFileSystemType*	ext2;
+
+		if (ped_exception_throw (
+			PED_EXCEPTION_NO_FEATURE,
+			PED_EXCEPTION_IGNORE_CANCEL,
+			_("The %s file system code doesn't support %s disk "
+			  "labels."),
+			fs_type->name, disk_type->name)
+				== PED_EXCEPTION_CANCEL)
+			return 0;
+
+		/* Hack: set the system to SOMETHING.  Otherwise, it might be
+		 * set to "non-existant"
+		 */
+		ext2 = ped_file_system_type_get ("ext2");
+		if (ext2)
+			return ext2->ops->set_system (NULL, part, disk_type);
+		else
+			return 0;
 	}
-	ped_exception_leave_all();
-
-	new_system = ped_file_system_get_system (fs, part,
-						 part->geom.disk->type);
-	if (!new_system)
-		goto error_close_fs;
-	part->system = new_system;
-
-	ped_file_system_close (fs);
-	return 1;
-
-error_close_fs:
-	ped_file_system_close (fs);
-error:
-	part->hidden = old_hide_state;
-	return 0;
 }
 
 /* Sets the system (i.e. partition system type) on a partition.  It attempts
@@ -597,23 +761,31 @@ error:
  * one anyway, and guesses the best partition system.
  */
 int
-ped_partition_set_system (PedPartition* part, PedFileSystemType* fs_type)
+ped_partition_set_system (PedPartition* part, const PedFileSystemType* fs_type)
 {
-	char			new_system;
+	PedDiskType*		disk_type;
 	PedFileSystem*		fs;
 
 	PED_ASSERT (part != NULL, return 0);
-	if (!(part->type & PED_PARTITION_EXTENDED))
-		PED_ASSERT (fs_type != NULL, return 0);
+
+	part->fs_type = fs_type;
 
 	if (part->type & PED_PARTITION_EXTENDED) {
-		new_system = ped_disk_get_extended_system (part->geom.disk);
-		if (!new_system)
-			goto error;
-		part->system = new_system;
-		return 1;
+		PED_ASSERT (part->geom.disk != NULL, return 0);
+		disk_type = part->geom.disk->type;
+		PED_ASSERT (disk_type != NULL, return 0);
+		PED_ASSERT (disk_type->ops != NULL, return 0);
+		PED_ASSERT (disk_type->ops->partition_set_extended_system
+				!= NULL,
+			    return 0);
 	}
 
+	if (part->type & PED_PARTITION_EXTENDED)
+		return disk_type->ops->partition_set_extended_system (part);
+
+	PED_ASSERT (fs_type != NULL, return 0);
+	PED_ASSERT (fs_type->ops->set_system != NULL, return 0);
+
 	ped_exception_fetch_all();
 	fs = ped_file_system_open (&part->geom);
 	if (fs && fs->type != fs_type) {
@@ -624,26 +796,68 @@ ped_partition_set_system (PedPartition* part, PedFileSystemType* fs_type)
 		ped_exception_catch();
 		ped_exception_leave_all();
 
-		new_system = fs_type->ops->get_system (NULL, part,
-							part->geom.disk->type);
-		if (!new_system)
-			goto error;
-		return 1;
+		return _fs_set_system (fs_type, NULL, part);
 	}
 	ped_exception_leave_all();
 
-	new_system = ped_file_system_get_system (fs, part,
-						 part->geom.disk->type);
+	if (!_fs_set_system (fs_type, fs, part))
+		goto error_close_fs;
+
 	ped_file_system_close (fs);
-	if (!new_system)
-		goto error;
-	part->system = new_system;
 	return 1;
 
+error_close_fs:
+	ped_file_system_close (fs);
 error:
 	return 0;
 }
 
+static int
+_assert_partition_name_feature (PedDiskType* disk_type)
+{
+	if (!ped_disk_type_check_feature (
+			disk_type, PED_DISK_TYPE_PARTITION_NAME)) {
+		ped_exception_throw (
+			PED_EXCEPTION_ERROR,
+			PED_EXCEPTION_CANCEL,
+			"%s disk labels do not support partition names.",
+			disk_type->name);
+		return 0;
+	}
+	return 1;
+}
+
+int
+ped_partition_set_name (PedPartition* part, const char* name)
+{
+	PedDisk*	disk;
+
+	PED_ASSERT (part != NULL, return 0);
+	disk = part->geom.disk;
+	PED_ASSERT (disk != NULL, return 0);
+	if (!_assert_partition_name_feature (disk->type))
+		return 0;
+	PED_ASSERT (disk->type->ops->partition_set_name != NULL, return 0);
+
+	disk->type->ops->partition_set_name (part, name);
+	return 1;
+}
+
+const char*
+ped_partition_get_name (const PedPartition* part)
+{
+	PedDisk*	disk;
+
+	PED_ASSERT (part != NULL, return NULL);
+	disk = part->geom.disk;
+	PED_ASSERT (disk != NULL, return NULL);
+	if (!_assert_partition_name_feature (disk->type))
+		return NULL;
+	PED_ASSERT (disk->type->ops->partition_get_name != NULL, return NULL);
+
+	return disk->type->ops->partition_get_name (part);
+}
+
 #ifdef linux
 
 /* FIXME: should this be part of the API?  Should it report a warning if the
@@ -652,7 +866,7 @@ static int
 _partition_is_root_device (const PedPartition* part)
 {
 	char		part_name [256];
-	int		root_dev;
+	unsigned int	root_dev;
 	struct stat	part_stat;
 	FILE*		proc_real_root_dev;
 
@@ -722,11 +936,10 @@ error:
 	return -1;
 }
 
-int
-ped_partition_is_busy (const PedPartition* part)
+static int
+_partition_is_mounted (const PedPartition* part)
 {
 	char		part_name [256];
-	int		is_root_dev;
 	int		status;
 
 	PED_ASSERT (part != NULL, return 0);
@@ -749,7 +962,6 @@ ped_partition_is_busy (const PedPartition* part)
 		if (status != -1)
 			return status;
 	}
-
 #endif
 
 	status = _check_mount_table_is_busy ("/etc/mtab", part_name);
@@ -764,6 +976,28 @@ error:
 				!= PED_EXCEPTION_IGNORE;
 }
 
+int
+ped_partition_is_busy (const PedPartition* part)
+{
+	PedPartition*	walk;
+
+	PED_ASSERT (part != NULL, return 0);
+
+	if (_partition_is_mounted (part))
+		return 1;
+
+	if (part->type == PED_PARTITION_EXTENDED) {
+		for (walk = part->part_list; walk; walk = walk->next) {
+			if (!ped_partition_is_active (walk))
+				continue;
+			if (ped_partition_is_busy (walk))
+				return 1;
+		}
+	}
+
+	return 0;
+}
+
 int
 ped_disk_is_busy (const PedDisk* disk)
 {
@@ -794,35 +1028,6 @@ ped_disk_extended_partition (const PedDisk* disk)
 	return walk;
 }
 
-PedPartition*
-ped_disk_get_boot_partition (const PedDisk* disk)
-{
-	PedPartition*		walk;
-
-	PED_ASSERT (disk != NULL, return 0);
-
-	for (walk = disk->part_list; walk; walk = walk->next) {
-		if (walk->bootable)
-			break;
-	}
-	return walk;
-}
-
-int
-ped_disk_set_boot_partition (PedDisk* disk, PedPartition* part)
-{
-	PedPartition*		old_boot_part;
-
-	PED_ASSERT (disk != NULL, return 0);
-	PED_ASSERT (part != NULL, return 0);
-
-	old_boot_part = ped_disk_get_boot_partition (disk);
-	if (old_boot_part)
-		old_boot_part->bootable = 0;
-	part->bootable = 1;
-	return 1;
-}
-
 /* returns the next partition.  If the current partition is an extended
  * partition, returns the first logical partition.
  */
@@ -834,7 +1039,7 @@ ped_disk_next_partition (const PedDisk* disk, const PedPartition* part)
 	if (!part)
 		return disk->part_list;
 	if (part->type == PED_PARTITION_EXTENDED)
-		return part->part_list;
+		return part->part_list ? part->part_list : part->next;
 	if (part->next)
 		return part->next;
 	if (part->type & PED_PARTITION_LOGICAL)
@@ -853,6 +1058,11 @@ _disk_check_sanity (PedDisk* disk)
 		PED_ASSERT (!(walk->type & PED_PARTITION_LOGICAL), return 0);
 		if (walk->prev)
 			PED_ASSERT (walk->prev->next == walk, return 0);
+
+		if (!strcasecmp (disk->type->name, "msdos")
+			&& disk->dev->geom_known
+			&& walk->type & PED_PARTITION_FREESPACE)
+			PED_ASSERT (walk->geom.length > 63 * 5, return 0);
 	}
 
 	if (!ped_disk_extended_partition (disk))
@@ -910,7 +1120,6 @@ ped_disk_get_partition_by_sector (const PedDisk* disk, PedSector sect)
 }
 
 /* I'm beginning to agree with Sedgewick :-/
- * UPDATE MODE ONLY
  */
 static int
 _disk_raw_insert_before (PedDisk* disk, PedPartition* loc, PedPartition* part)
@@ -934,9 +1143,6 @@ _disk_raw_insert_before (PedDisk* disk, PedPartition* loc, PedPartition* part)
 	return 1;
 }
 
-/* inserts a partition *after* loc.
- * UPDATE MODE ONLY
- */
 static int
 _disk_raw_insert_after (PedDisk* disk, PedPartition* loc, PedPartition* part)
 {
@@ -953,8 +1159,6 @@ _disk_raw_insert_after (PedDisk* disk, PedPartition* loc, PedPartition* part)
 	return 1;
 }
 
-/* UPDATE MODE ONLY
- */
 static int
 _disk_raw_remove (PedDisk* disk, PedPartition* part)
 {
@@ -979,6 +1183,45 @@ _disk_raw_remove (PedDisk* disk, PedPartition* part)
 	return 1;
 }
 
+/* UPDATE MODE ONLY
+ */
+static int
+_disk_raw_add (PedDisk* disk, PedPartition* part)
+{
+	PedPartition*	walk;
+	PedPartition*	last;
+	PedPartition*	ext_part;
+
+	PED_ASSERT (disk->update_mode, return 0);
+
+	ext_part = ped_disk_extended_partition (disk);
+
+	last = NULL;
+	walk = (part->type & PED_PARTITION_LOGICAL) ?
+			ext_part->part_list : disk->part_list;
+
+	for (; walk; last = walk, walk = walk->next) {
+		if (walk->geom.start > part->geom.end)
+			break;
+	}
+
+	if (walk) {
+		return _disk_raw_insert_before (disk, walk, part);
+	} else {
+		if (last) {
+			return _disk_raw_insert_after (disk, last, part);
+		} else {
+			if (part->type & PED_PARTITION_LOGICAL)
+				ext_part->part_list = part;
+			else
+				disk->part_list = part;
+		} 
+	}
+
+	return 1;
+}
+
+
 /* returns 0 if the partition, "part" overlaps with any partitions on the
  * "disk".  The geometry of "part" is taken to be "geom", NOT "part->geom"
  * (the idea here is to check if "geom" is valid, before changing "part").
@@ -995,6 +1238,10 @@ ped_disk_check_overlap_new_geom (PedDisk* disk, PedPartition* part,
 {
 	PedPartition*	walk;
 
+	PED_ASSERT (disk != NULL, return 0);
+	PED_ASSERT (part != NULL, return 0);
+	PED_ASSERT (geom != NULL, return 0);
+
 	for (walk = ped_disk_next_partition (disk, NULL); walk;
 	     walk = ped_disk_next_partition (disk, walk)) {
 		if (walk->type & PED_PARTITION_FREESPACE)
@@ -1020,6 +1267,9 @@ ped_disk_check_overlap_new_geom (PedDisk* disk, PedPartition* part,
 int
 ped_disk_check_overlap (PedDisk* disk, PedPartition* part)
 {
+	PED_ASSERT (disk != NULL, return 0);
+	PED_ASSERT (part != NULL, return 0);
+
 	return ped_disk_check_overlap_new_geom (disk, part, &part->geom);
 }
 
@@ -1028,6 +1278,8 @@ _check_new_partition_basic_sanity (PedDisk* disk, PedPartition* part)
 {
 	PedPartition*	ext_part = ped_disk_extended_partition (disk);
 
+	PED_ASSERT (part->geom.start <= part->geom.end, return 0);
+
 	if (part->geom.start < 0 || part->geom.end >= disk->dev->length) {
 		ped_exception_throw (
 			PED_EXCEPTION_ERROR,
@@ -1096,42 +1348,9 @@ _check_new_partition (PedDisk* disk, PedPartition* part)
 	return 1;
 }
 
-static int
-_add_partition (PedDisk* disk, PedPartition* part)
-{
-	PedPartition*	walk;
-	PedPartition*	last;
-	PedPartition*	ext_part;
-
-	ext_part = ped_disk_extended_partition (disk);
-
-	last = NULL;
-	walk = (part->type & PED_PARTITION_LOGICAL) ?
-			ext_part->part_list : disk->part_list;
-
-	for (; walk; last = walk, walk = walk->next) {
-		if (walk->geom.start > part->geom.end)
-			break;
-	}
-
-	if (walk) {
-		return _disk_raw_insert_before (disk, walk, part);
-	} else {
-		if (last) {
-			return _disk_raw_insert_after (disk, last, part);
-		} else {
-			if (part->type & PED_PARTITION_LOGICAL)
-				ext_part->part_list = part;
-			else
-				disk->part_list = part;
-		} 
-	}
-
-	return 1;
-}
-
 int
-ped_disk_add_partition (PedDisk* disk, PedPartition* part)
+ped_disk_add_partition (PedDisk* disk, PedPartition* part,
+			const PedConstraint* constraint)
 {
 	PED_ASSERT (disk != NULL, return 0);
 	PED_ASSERT (part != NULL, return 0);
@@ -1143,20 +1362,44 @@ ped_disk_add_partition (PedDisk* disk, PedPartition* part)
 		(int) part->system);
 #endif
 
+	if (!ped_disk_type_check_feature (disk->type, PED_DISK_TYPE_EXTENDED)
+	    && (part->type == PED_PARTITION_EXTENDED
+		    || part->type == PED_PARTITION_LOGICAL)) {
+		ped_exception_throw (
+			PED_EXCEPTION_ERROR,
+			PED_EXCEPTION_CANCEL,
+			_("%s disk labels don't support logical or extended "
+			  "partitions."),
+			disk->type->name);
+		return 0;
+	}
+
+	if (part->type == PED_PARTITION_PRIMARY
+	    || part->type == PED_PARTITION_EXTENDED) {
+		if (_disk_count_primary_partitions (disk) + 1
+		    > ped_disk_get_max_primary_partition_count (disk)) {
+			ped_exception_throw (
+				PED_EXCEPTION_ERROR,
+				PED_EXCEPTION_CANCEL,
+				"Too many primary partitions.");
+			return 0;
+		}
+	}
+
 	if (!_check_new_partition_basic_sanity (disk, part))
 		return 0;
 
 	_disk_push_update_mode (disk);
 
 	if (ped_partition_is_active (part)) {
-		if (!ped_disk_enumerate_partition (disk, part))
+		if (!_partition_enumerate (part))
 			goto error;
-		if (!ped_disk_align_partition (disk, part))
+		if (!_partition_align (part, constraint))
 			goto error;
 	}
 	if (!_check_new_partition (disk, part))
 		goto error;
-	if (!_add_partition (disk, part))
+	if (!_disk_raw_add (disk, part))
 		goto error;
 
 	_disk_pop_update_mode (disk);
@@ -1169,30 +1412,40 @@ error:
 	return 0;
 }
 
+int
+ped_disk_remove_partition (PedDisk* disk, PedPartition* part)
+{
+	PED_ASSERT (disk != NULL, return 0);
+	PED_ASSERT (part != NULL, return 0);
+
+	_disk_push_update_mode (disk);
+	PED_ASSERT (part->part_list == NULL, goto error);
+	_disk_raw_remove (disk, part);
+	_disk_pop_update_mode (disk);
+	ped_disk_enumerate_partitions (disk);
+	return 1;
+
+error:
+	_disk_pop_update_mode (disk);
+	return 0;
+}
+
 static int
 ped_disk_delete_all_logical (PedDisk* disk);
 
 int
 ped_disk_delete_partition (PedDisk* disk, PedPartition* part)
 {
-	int	is_metadata_part;
-
 	PED_ASSERT (disk != NULL, return 0);
 	PED_ASSERT (part != NULL, return 0);
 
-	is_metadata_part = part->type & PED_PARTITION_METADATA;
-
 	_disk_push_update_mode (disk);
-
-	if (part->type & PED_PARTITION_EXTENDED)
+	if (part->type == PED_PARTITION_EXTENDED)
 		ped_disk_delete_all_logical (disk);
-	_disk_raw_remove (disk, part);
+	ped_disk_remove_partition (disk, part);
 	ped_partition_destroy (part);
-
 	_disk_pop_update_mode (disk);
 
-	ped_disk_enumerate_partitions (disk);
-
 	return 1;
 }
 
@@ -1210,10 +1463,8 @@ ped_disk_delete_all_logical (PedDisk* disk)
 	for (walk = ext_part->part_list; walk; walk = next) {
 		next = walk->next;
 
-		if (!(walk->type & PED_PARTITION_FREESPACE)) {
-			if (!ped_disk_delete_partition (disk, walk))
-				return 0;
-		}
+		if (!ped_disk_delete_partition (disk, walk))
+			return 0;
 	}
 	return 1;
 }
@@ -1242,16 +1493,18 @@ ped_disk_delete_all (PedDisk* disk)
 
 static PedGeometry*
 _calc_aligned_geom (PedDisk* disk, PedPartition* part,
+		    const PedConstraint* constraint,
 		    PedSector start, PedSector end)
 {
 	PedGeometry	old_geom;
 	PedGeometry*	aligned_geom;
 
 	old_geom = part->geom;
-	ped_geometry_set (&part->geom, start, end - start + 1);
+	if (!ped_geometry_set (&part->geom, start, end - start + 1))
+		goto error;
 
 	if (ped_partition_is_active (part)) {
-		if (!ped_disk_align_partition (disk, part))
+		if (!_partition_align (part, constraint))
 			goto error;
 	}
 
@@ -1300,14 +1553,7 @@ _check_changed_geometry (PedDisk* disk, PedPartition* part,
 	PED_ASSERT (part != NULL, return 0);
 	PED_ASSERT (new_geom != NULL, return 0);
 
-	if (new_geom->start > new_geom->end) {
-		ped_exception_throw (
-			PED_EXCEPTION_ERROR,
-			PED_EXCEPTION_CANCEL,
-			_("Can't move the end of the partition behind the "
-			  "start."));
-		goto error;
-	}
+	PED_ASSERT (new_geom->start <= new_geom->end, return 0);
 
 	if (part->type & PED_PARTITION_LOGICAL) {
 		ext_part = ped_disk_extended_partition (disk);
@@ -1330,8 +1576,7 @@ _check_changed_geometry (PedDisk* disk, PedPartition* part,
 		ped_exception_throw (
 			PED_EXCEPTION_ERROR,
 			PED_EXCEPTION_CANCEL,
-			_("Can't grow partition onto used space, or outside "
-			  "the disk."));
+			_("Can't grow a partition onto used space."));
 		goto error;
 	}
 
@@ -1343,65 +1588,123 @@ error:
 
 int
 ped_disk_set_partition_geom (PedDisk* disk, PedPartition* part,
+			     const PedConstraint* constraint,
 			     PedSector start, PedSector end)
 {
-	PedPartition*		free_space;
-	PedPartition*		ext_free_space;
 	PedGeometry*		new_geom;
 
 	PED_ASSERT (disk != NULL, return 0);
 	PED_ASSERT (part != NULL, return 0);
 
-	new_geom = _calc_aligned_geom (disk, part, start, end);
-	if (!new_geom)
-		goto error;
-
 	_disk_push_update_mode (disk);
 
+	new_geom = _calc_aligned_geom (disk, part, constraint, start, end);
+	if (!new_geom)
+		goto error_pop_update_mode;
 	if (!_check_changed_geometry (disk, part, new_geom))
-		goto error;
+		goto error_destroy_new_geom;
 
+	/* remove and add, to ensure the ordering gets updated if necessary */
+	_disk_raw_remove (disk, part);
 	part->geom = *new_geom;
-	ped_geometry_destroy (new_geom);
-
+	_disk_raw_add (disk, part);
 	_disk_pop_update_mode (disk);
 
+	ped_geometry_destroy (new_geom);
 	return 1;
 
-error:
+error_destroy_new_geom:
+	ped_geometry_destroy (new_geom);
+error_pop_update_mode:
 	_disk_pop_update_mode (disk);
+error:
 	return 0;
 }
 
 int
-ped_disk_maximize_partition (PedDisk* disk, PedPartition* part)
+ped_disk_maximize_partition (PedDisk* disk, PedPartition* part,
+			     const PedConstraint* constraint)
 {
 	PedGeometry	old_geom;
+	PedSector	global_min_start;
+	PedSector	global_max_end;
+	PedSector	new_start;
+	PedSector	new_end;
+	PedPartition*	ext_part = ped_disk_extended_partition (disk);
+	PedConstraint*	constraint_any;
 
 	PED_ASSERT (disk != NULL, return 0);
 	PED_ASSERT (part != NULL, return 0);
 
+	if (part->type & PED_PARTITION_LOGICAL) {
+		PED_ASSERT (ext_part != NULL, return 0);
+		global_min_start = ext_part->geom.start;
+		global_max_end = ext_part->geom.end;
+	} else {
+		global_min_start = 0;
+		global_max_end = disk->dev->length - 1;
+	}
+
 	old_geom = part->geom;
 
-	if (part->prev && part->prev->type & PED_PARTITION_FREESPACE) {
-		if (!ped_disk_set_partition_geom (
-				disk, part, part->prev->geom.start,
-				part->geom.end))
-			goto error;
-	}
-	if (part->next && part->next->type & PED_PARTITION_FREESPACE) {
-		if (!ped_disk_set_partition_geom (
-				disk, part, part->geom.start,
-				part->next->geom.end))
-			goto error;
-	}
+	_disk_push_update_mode (disk);
+
+	if (part->prev)
+		new_start = part->prev->geom.end + 1;
+	else
+		new_start = global_min_start;
+
+	if (part->next)
+		new_end = part->next->geom.start - 1;
+	else
+		new_end = global_max_end;
+
+	if (!ped_disk_set_partition_geom (disk, part, constraint, new_start,
+					  new_end))
+		goto error;
+
+	_disk_pop_update_mode (disk);
 	return 1;
 
 error:
-	ped_disk_set_partition_geom (disk, part, old_geom.start, old_geom.end);
+	constraint_any = ped_constraint_any (disk);
+	ped_disk_set_partition_geom (disk, part, constraint_any,
+				     old_geom.start, old_geom.end);
+	ped_constraint_destroy (constraint_any);
+	_disk_pop_update_mode (disk);
 	return 0;
 }
 
+PedGeometry*
+ped_disk_get_max_partition_geometry (PedDisk* disk, PedPartition* part,
+				     const PedConstraint* constraint)
+{
+	PedGeometry	old_geom;
+	PedGeometry	max_geom;
+	PedConstraint*	constraint_any;
+
+	PED_ASSERT(disk != NULL, return NULL);
+	PED_ASSERT(part != NULL, return NULL);
+
+	old_geom = part->geom;
+	if (!ped_disk_maximize_partition (disk, part, constraint))
+		return NULL;
+	max_geom = part->geom;
+
+	constraint_any = ped_constraint_any (disk);
+	ped_disk_set_partition_geom (disk, part, constraint,
+				     old_geom.start, old_geom.end);
+	ped_constraint_destroy (constraint_any);
+
+	/* this assertion should never fail, because the old
+	 * geometry was valid
+	 */
+	PED_ASSERT (ped_geometry_test_equal (&part->geom, &old_geom),
+		    return NULL);
+
+	return ped_geometry_duplicate (&max_geom);
+}
+
 /* Reduces the size of the extended partition to wrap the logical partitions.
  * If there are no logical partitions, it removes the extended partition.
  */
@@ -1412,6 +1715,8 @@ ped_disk_minimize_extended_partition (PedDisk* disk)
 	PedPartition*		last_logical;
 	PedPartition*		walk;
 	PedPartition*		ext_part;
+	PedConstraint*		constraint;
+	int			status;
 
 	PED_ASSERT (disk != NULL, return 0);
 
@@ -1419,33 +1724,98 @@ ped_disk_minimize_extended_partition (PedDisk* disk)
 	if (!ext_part)
 		return 1;
 
+	_disk_push_update_mode (disk);
+
 	first_logical = ext_part->part_list;
-	if (!first_logical)
+	if (!first_logical) {
+		_disk_pop_update_mode (disk);
 		return ped_disk_delete_partition (disk, ext_part);
+	}
 
 	for (walk = first_logical; walk->next; walk = walk->next);
 	last_logical = walk;
 
-	return ped_disk_set_partition_geom (disk, ext_part,
-					    first_logical->geom.start,
-					    last_logical->geom.end);
+	constraint = ped_constraint_any (disk);
+	status = ped_disk_set_partition_geom (disk, ext_part, constraint,
+					      first_logical->geom.start,
+					      last_logical->geom.end);
+	ped_constraint_destroy (constraint);
+
+	_disk_pop_update_mode (disk);
+	return status;
 }
 
-char*
+const char*
 ped_partition_type_get_name (PedPartitionType type)
 {
 	if (type & PED_PARTITION_METADATA)
-		return _("metadata");
+		return N_("metadata");
 	else if (type & PED_PARTITION_FREESPACE)
-		return _("free");
+		return N_("free");
 	else if (type & PED_PARTITION_EXTENDED)
-		return _("extended");
+		return N_("extended");
 	else if (type & PED_PARTITION_LOGICAL)
-		return _("logical");
+		return N_("logical");
 	else
-		return _("primary");
+		return N_("primary");
 }
 
+/* returns the English name of the flag.  Get the native name by calling
+ * dgettext("parted", XXX) on the English name.
+ */
+const char*
+ped_partition_flag_get_name (PedPartitionFlag flag)
+{
+	switch (flag) {
+	case PED_PARTITION_BOOT:
+		return N_("boot");
+	case PED_PARTITION_ROOT:
+		return N_("root");
+	case PED_PARTITION_SWAP:
+		return N_("swap");
+	case PED_PARTITION_HIDDEN:
+		return N_("hidden");
+	case PED_PARTITION_RAID:
+		return N_("raid");
+	case PED_PARTITION_LVM:
+		return N_("lvm");
+	case PED_PARTITION_LBA:
+		return N_("lba");
+
+	default:
+		ped_exception_throw (
+			PED_EXCEPTION_BUG,
+			PED_EXCEPTION_CANCEL,
+			_("Unknown flag partition flag, %d."),
+			flag);
+		return NULL;
+	}
+}
+
+PedPartitionFlag
+ped_partition_flag_next (PedPartitionFlag flag)
+{
+	return (flag + 1) % (PED_PARTITION_LAST_FLAG + 1);
+}
+
+PedPartitionFlag
+ped_partition_flag_get_by_name (const char* name)
+{
+	PedPartitionFlag	flag;
+	const char*		flag_name;
+
+	for (flag = ped_partition_flag_next (0); flag;
+	     		flag = ped_partition_flag_next (flag)) {
+		flag_name = ped_partition_flag_get_name (flag);
+		if (strcasecmp (name, flag_name) == 0
+		    || strcasecmp (name, _(flag_name)) == 0)
+			return flag;
+	}
+
+	return 0;
+}
+
+
 void
 ped_partition_print (PedPartition* part)
 {
