@@ -1,7 +1,7 @@
 /*
     ext2_resize.c -- ext2 resizer
-    Copyright (C) 1998,99 Lennert Buytenhek <buytenh@gnu.org>
-
+    Copyright (C) 1998-2000 Free Software Foundation, Inc.
+  
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -37,14 +37,15 @@ static int ext2_add_group(struct ext2_fs *fs, blk_t groupsize)
 		fprintf(stderr, "ext2_add_group\n");
 
 	if (!ped_realloc (&fs->gd,
-			  (fs->numgroups+1) * sizeof(struct ext2_group_desc))
-			      + fs->blocksize);
+			  (fs->numgroups+1) * sizeof(struct ext2_group_desc)
+			      + fs->blocksize))
 		return 0;
 
 	if (fs->opt_debug)
 	{
-		if (fs->sb.s_blocks_count !=
-		    fs->sb.s_first_data_block + fs->numgroups * fs->sb.s_blocks_per_group)
+		if (EXT2_SUPER_BLOCKS_COUNT(fs->sb) !=
+		    EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb)
+		    + fs->numgroups * EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb))
 		{
 			fprintf(stderr,
 				"ext2_add_group: last (existing) group "
@@ -56,7 +57,8 @@ static int ext2_add_group(struct ext2_fs *fs, blk_t groupsize)
 
 	group = fs->numgroups;
 	sparse = ext2_is_group_sparse(fs, group);
-	groupstart = fs->sb.s_first_data_block + group * fs->sb.s_blocks_per_group;
+	groupstart = EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb)
+		     + group * EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
 
 	admin = fs->adminblocks;
 	if (!sparse)
@@ -65,7 +67,7 @@ static int ext2_add_group(struct ext2_fs *fs, blk_t groupsize)
 	if (fs->opt_debug)
 	{
 		if (groupsize < fs->adminblocks ||
-		    groupsize > fs->sb.s_blocks_per_group)
+		    groupsize > EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb))
 		{
 			fprintf(stderr,
 				"ext2_add_group: groups of %i blocks are "
@@ -75,7 +77,9 @@ static int ext2_add_group(struct ext2_fs *fs, blk_t groupsize)
 		}
 	}
 
-	newgdblocks = howmany((fs->numgroups + 1) * sizeof(struct ext2_group_desc), fs->blocksize);
+	newgdblocks = howmany((fs->numgroups + 1)
+					* sizeof(struct ext2_group_desc),
+			      fs->blocksize);
 	if (newgdblocks != fs->gdblocks)
 	{
 		int i;
@@ -85,8 +89,10 @@ static int ext2_add_group(struct ext2_fs *fs, blk_t groupsize)
 			{
 				blk_t start;
 
-				start = fs->sb.s_first_data_block + i * fs->sb.s_blocks_per_group;
-				ext2_set_block_state(fs, start + fs->gdblocks + 1, 1, 1);
+				start = EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb)
+				      + i * EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
+				ext2_set_block_state(fs,
+						start + fs->gdblocks + 1, 1, 1);
 			}
 
 		fs->gdblocks++;
@@ -97,10 +103,16 @@ static int ext2_add_group(struct ext2_fs *fs, blk_t groupsize)
 
 	fs->numgroups++;
 
-	fs->sb.s_inodes_count += fs->sb.s_inodes_per_group;
-	fs->sb.s_blocks_count += groupsize;
-	fs->sb.s_free_blocks_count += groupsize - admin;
-	fs->sb.s_free_inodes_count += fs->sb.s_inodes_per_group;
+	fs->sb.s_inodes_count = PED_CPU_TO_LE32(
+		EXT2_SUPER_INODES_COUNT(fs->sb)
+		+ EXT2_SUPER_INODES_PER_GROUP(fs->sb));
+	fs->sb.s_blocks_count = PED_CPU_TO_LE32(
+		EXT2_SUPER_BLOCKS_COUNT(fs->sb) + groupsize);
+	fs->sb.s_free_blocks_count = PED_CPU_TO_LE32(
+		EXT2_SUPER_FREE_BLOCKS_COUNT(fs->sb) + groupsize - admin);
+	fs->sb.s_free_inodes_count = PED_CPU_TO_LE32(
+		EXT2_SUPER_FREE_INODES_COUNT(fs->sb)
+	        + EXT2_SUPER_INODES_PER_GROUP(fs->sb));
 	fs->metadirty |= EXT2_META_SB;
 
 	{
@@ -112,28 +124,34 @@ static int ext2_add_group(struct ext2_fs *fs, blk_t groupsize)
 
 		if (sparse)
 		{
-			fs->gd[group].bg_block_bitmap = sparseoff;
-			fs->gd[group].bg_inode_bitmap = sparseoff + 1;
+			fs->gd[group].bg_block_bitmap
+				= PED_CPU_TO_LE32(sparseoff);
+			fs->gd[group].bg_inode_bitmap
+				= PED_CPU_TO_LE32(sparseoff + 1);
 		}
 		else
 		{
-			fs->gd[group].bg_block_bitmap = off;
-			fs->gd[group].bg_inode_bitmap = off + 1;
+			fs->gd[group].bg_block_bitmap
+				= PED_CPU_TO_LE32(off);
+			fs->gd[group].bg_inode_bitmap
+				= PED_CPU_TO_LE32(off + 1);
 		}
 
-		fs->gd[group].bg_inode_table = sparseoff + 2;			/* Hey, I don't know _why_ either */
+		/* Hey, I don't know _why_ either */
+		fs->gd[group].bg_inode_table = PED_CPU_TO_LE32(sparseoff + 2);
 	}
 
-	fs->gd[group].bg_free_blocks_count = groupsize - admin;
-	fs->gd[group].bg_free_inodes_count = fs->sb.s_inodes_per_group;
-	fs->gd[group].bg_used_dirs_count   = 0;
+	fs->gd[group].bg_free_blocks_count = PED_CPU_TO_LE16(groupsize - admin);
+	fs->gd[group].bg_free_inodes_count = PED_CPU_TO_LE16(
+		EXT2_SUPER_INODES_PER_GROUP(fs->sb));
+	fs->gd[group].bg_used_dirs_count = 0;
 	fs->metadirty |= EXT2_META_SB | EXT2_META_GD;
 
 	{
 		struct ext2_buffer_head *bh;
-		int i;
+		blk_t i;
 
-		bh = ext2_bcreate(fs, fs->gd[group].bg_block_bitmap);
+		bh = ext2_bcreate(fs, EXT2_GROUP_BLOCK_BITMAP(fs->gd[group]));
 		if (!bh)
 			return 0;
 
@@ -144,29 +162,30 @@ static int ext2_add_group(struct ext2_fs *fs, blk_t groupsize)
 				bh->data[i>>3] |= _bitmap[i&7];
 		}
 
-		i = fs->gd[group].bg_block_bitmap - groupstart;
+		i = EXT2_GROUP_BLOCK_BITMAP(fs->gd[group]) - groupstart;
 		bh->data[i>>3] |= _bitmap[i&7];
 
-		i = fs->gd[group].bg_inode_bitmap - groupstart;
+		i = EXT2_GROUP_INODE_BITMAP(fs->gd[group]) - groupstart;
 		bh->data[i>>3] |= _bitmap[i&7];
 
 		for (i=0;i<fs->inodeblocks;i++)
 		{
 			blk_t j;
 
-			j = fs->gd[group].bg_inode_table - groupstart + i;
+			j = EXT2_GROUP_INODE_TABLE(fs->gd[group])
+			    - groupstart + i;
 			bh->data[j>>3] |= _bitmap[j&7];
 		}
 
-		for (i=groupsize;i<fs->sb.s_blocks_per_group;i++)
+		for (i=groupsize;i<EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);i++)
 			bh->data[i>>3] |= _bitmap[i&7];
 
 		ext2_brelse(bh, 0);         /* this is a block bitmap */
 	}
 
-	if (!ext2_zero_blocks(fs, fs->gd[group].bg_inode_bitmap, 1))
+	if (!ext2_zero_blocks(fs, EXT2_GROUP_INODE_BITMAP(fs->gd[group]), 1))
 		return 0;
-	if (!ext2_zero_blocks(fs, fs->gd[group].bg_inode_table,
+	if (!ext2_zero_blocks(fs, EXT2_GROUP_INODE_TABLE(fs->gd[group]),
 			      fs->inodeblocks))
 		return 0;
 
@@ -195,9 +214,11 @@ static int ext2_del_group(struct ext2_fs *fs)
 	if (!sparse)
 		admin -= fs->gdblocks + 1;
 
-	groupsize = fs->sb.s_blocks_count - fs->sb.s_first_data_block - group * fs->sb.s_blocks_per_group;
+	groupsize = EXT2_SUPER_BLOCKS_COUNT(fs->sb)
+		  - EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb)
+		  - group * EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
 
-	if (fs->sb.s_free_blocks_count < groupsize - admin)
+	if (EXT2_SUPER_FREE_BLOCKS_COUNT(fs->sb) < groupsize - admin)
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 			_("Filesystem is too occupied to remove a group!"));
@@ -205,7 +226,8 @@ static int ext2_del_group(struct ext2_fs *fs)
 		return 0;
 	}
 
-	if (fs->sb.s_free_inodes_count < fs->sb.s_inodes_per_group)
+	if (EXT2_SUPER_FREE_INODES_COUNT(fs->sb)
+		< EXT2_SUPER_INODES_PER_GROUP(fs->sb))
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 			_("Filesystem has too many allocated inodes to "
@@ -215,8 +237,8 @@ static int ext2_del_group(struct ext2_fs *fs)
 
 	if (fs->opt_debug)
 	{
-		if (fs->gd[group].bg_free_inodes_count !=
-		    fs->sb.s_inodes_per_group)
+		if (EXT2_GROUP_FREE_INODES_COUNT(fs->gd[group]) !=
+		    EXT2_SUPER_INODES_PER_GROUP(fs->sb))
 		{
 			fprintf(stderr,
 				"ext2_del_group: this should not "
@@ -238,8 +260,8 @@ static int ext2_del_group(struct ext2_fs *fs)
 			{
 				blk_t start;
 
-				start = fs->sb.s_first_data_block +
-					i * fs->sb.s_blocks_per_group;
+				start = EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb) +
+					i * EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
 				ext2_set_block_state(fs,
 						     start + fs->gdblocks,
 						     0, 1);
@@ -253,15 +275,16 @@ static int ext2_del_group(struct ext2_fs *fs)
 
 	if (fs->opt_debug)
 	{
-		if (fs->gd[group].bg_free_blocks_count != groupsize - admin)
+		if (EXT2_GROUP_FREE_BLOCKS_COUNT(fs->gd[group])
+				!= groupsize - admin)
 		{
 			blk_t i;
 			blk_t num;
 			blk_t offset;
 
-			offset = fs->sb.s_first_data_block +
-				group * fs->sb.s_blocks_per_group;
-			num = fs->sb.s_blocks_per_group;
+			offset = EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb) +
+				group * EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
+			num = EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
 
 			for (i=0;i<num;i++)
 				if (ext2_is_data_block(fs, offset+i) &&
@@ -280,10 +303,16 @@ static int ext2_del_group(struct ext2_fs *fs)
 
 	fs->numgroups--;
 
-	fs->sb.s_inodes_count -= fs->sb.s_inodes_per_group;
-	fs->sb.s_blocks_count -= groupsize;
-	fs->sb.s_free_blocks_count -= groupsize - admin;
-	fs->sb.s_free_inodes_count -= fs->sb.s_inodes_per_group;
+	fs->sb.s_inodes_count = PED_CPU_TO_LE32(
+		EXT2_SUPER_INODES_COUNT(fs->sb)
+		- EXT2_SUPER_INODES_PER_GROUP(fs->sb));
+	fs->sb.s_blocks_count = PED_CPU_TO_LE32(
+		EXT2_SUPER_BLOCKS_COUNT(fs->sb) - groupsize);
+	fs->sb.s_free_blocks_count = PED_CPU_TO_LE32(
+		EXT2_SUPER_FREE_BLOCKS_COUNT(fs->sb) - (groupsize - admin));
+	fs->sb.s_free_inodes_count = PED_CPU_TO_LE32(
+		EXT2_SUPER_FREE_INODES_COUNT(fs->sb)
+	        - EXT2_SUPER_INODES_PER_GROUP(fs->sb));
 	fs->metadirty |= EXT2_META_SB;
 
 	if (fs->opt_safe)
@@ -307,8 +336,9 @@ static int ext2_grow_group(struct ext2_fs *fs, blk_t newsize)
 		fprintf(stderr, "ext2_grow_group\n");
 
 	group = fs->numgroups - 1;
-	groupoff = group * fs->sb.s_blocks_per_group + fs->sb.s_first_data_block;
-	gblocks = fs->sb.s_blocks_count - groupoff;
+	groupoff = group * EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb)
+		   + EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb);
+	gblocks = EXT2_SUPER_BLOCKS_COUNT(fs->sb) - groupoff;
 
 	if (fs->opt_debug)
 	{
@@ -330,7 +360,8 @@ static int ext2_grow_group(struct ext2_fs *fs, blk_t newsize)
 	for (i=gblocks;i<newsize;i++)
 		ext2_set_block_state(fs, groupoff + i, 0, 1);
 
-	fs->sb.s_blocks_count += newsize - gblocks;
+	fs->sb.s_blocks_count = PED_CPU_TO_LE32(
+		EXT2_SUPER_BLOCKS_COUNT(fs->sb) + newsize - gblocks);
 	fs->metadirty |= EXT2_META_SB;
 
 	if (fs->opt_safe)
@@ -345,7 +376,7 @@ static int ext2_shrink_group(struct ext2_fs *fs, blk_t newsize)
 	int   group;
 	blk_t groupoff;
 	blk_t gblocks;
-	int   i;
+	blk_t i;
 
 	if (fs->opt_verbose)
 		fprintf(stderr, "ext2_shrink_group\n");
@@ -356,8 +387,9 @@ static int ext2_shrink_group(struct ext2_fs *fs, blk_t newsize)
 	if (!ext2_is_group_sparse(fs, group))
 		admin -= fs->gdblocks + 1;
 
-	groupoff = group * fs->sb.s_blocks_per_group + fs->sb.s_first_data_block;
-	gblocks = fs->sb.s_blocks_count - groupoff;
+	groupoff = group * EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb)
+		   + EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb);
+	gblocks = EXT2_SUPER_BLOCKS_COUNT(fs->sb) - groupoff;
 
 	if (fs->opt_debug)
 	{
@@ -403,9 +435,12 @@ static int ext2_shrink_group(struct ext2_fs *fs, blk_t newsize)
 	}
 
 	i = gblocks - newsize;
-	fs->sb.s_blocks_count -= i;
-	fs->sb.s_free_blocks_count -= i;
-	fs->gd[group].bg_free_blocks_count -= i;
+	fs->sb.s_blocks_count = PED_CPU_TO_LE32(
+		EXT2_SUPER_BLOCKS_COUNT(fs->sb) - i);
+	fs->sb.s_free_blocks_count = PED_CPU_TO_LE32(
+		EXT2_SUPER_FREE_BLOCKS_COUNT(fs->sb) - i);
+	fs->gd[group].bg_free_blocks_count = PED_CPU_TO_LE16(
+		EXT2_GROUP_FREE_BLOCKS_COUNT(fs->gd[group]) - i);
 
 	fs->metadirty |= EXT2_META_SB | EXT2_META_GD;
 
@@ -434,17 +469,18 @@ static int ext2_grow_fs(struct ext2_fs *fs, blk_t newsize)
 	if (!ext2_metadata_push(fs, newsize))
 		return 0;
 
-	diff = newsize - fs->sb.s_blocks_count;
-	sizelast = fs->sb.s_blocks_count - fs->sb.s_first_data_block
-		   - (fs->numgroups-1)*fs->sb.s_blocks_per_group;
+	diff = newsize - EXT2_SUPER_BLOCKS_COUNT(fs->sb);
+	sizelast = EXT2_SUPER_BLOCKS_COUNT(fs->sb)
+		   - EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb)
+		   - (fs->numgroups-1) * EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
 
-	if (sizelast != fs->sb.s_blocks_per_group)
+	if (sizelast != EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb))
 	{
 		blk_t growto;
 
 		growto = sizelast + diff;
-		if (growto > fs->sb.s_blocks_per_group)
-			growto = fs->sb.s_blocks_per_group;
+		if (growto > EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb))
+			growto = EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
 
 		if (!ext2_grow_group(fs, growto))
 			return 0;
@@ -454,7 +490,7 @@ static int ext2_grow_fs(struct ext2_fs *fs, blk_t newsize)
 
 	while (diff)
 	{
-		sizelast = min(diff, fs->sb.s_blocks_per_group);
+		sizelast = PED_MIN(diff, EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb));
 		if (!ext2_add_group(fs, sizelast))
 			return 0;
 
@@ -473,8 +509,10 @@ static int ext2_shrink_fs(struct ext2_fs *fs, blk_t newsize)
 	if (fs->opt_verbose)
 		fprintf(stderr, "ext2_shrink_fs\n");
 
-	newgroups = howmany(newsize - fs->sb.s_first_data_block, fs->sb.s_blocks_per_group);
-	if (fs->sb.s_blocks_count - fs->sb.s_free_blocks_count > newsize)
+	newgroups = howmany(newsize - EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb),
+			    EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb));
+	if (EXT2_SUPER_BLOCKS_COUNT(fs->sb)
+	    - EXT2_SUPER_FREE_BLOCKS_COUNT(fs->sb) > newsize)
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 			_("Your filesystem is too occupied to resize it to %i "
@@ -482,7 +520,9 @@ static int ext2_shrink_fs(struct ext2_fs *fs, blk_t newsize)
 		return 0;
 	}
 
-	if (fs->sb.s_inodes_count - fs->sb.s_free_inodes_count > newgroups * fs->sb.s_inodes_per_group)
+	if (EXT2_SUPER_INODES_COUNT(fs->sb)
+	    - EXT2_SUPER_FREE_INODES_COUNT(fs->sb)
+	    		> newgroups * EXT2_SUPER_INODES_PER_GROUP(fs->sb))
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 			_("Your filesystem has too much occupied inodes to "
@@ -496,12 +536,14 @@ static int ext2_shrink_fs(struct ext2_fs *fs, blk_t newsize)
 	if (!ext2_block_relocate(fs, newsize))
 		return 0;
 
-	diff = fs->sb.s_blocks_count - newsize;
+	diff = EXT2_SUPER_BLOCKS_COUNT(fs->sb) - newsize;
 
 	while (diff > 0)
 	{
-		sizelast = fs->sb.s_blocks_count - fs->sb.s_first_data_block -
-			(fs->numgroups - 1) * fs->sb.s_blocks_per_group;
+		sizelast = EXT2_SUPER_BLOCKS_COUNT(fs->sb)
+			   - EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb) -
+			   (fs->numgroups - 1)
+			   	* EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
 
 		if (diff < sizelast)
 		{
@@ -531,25 +573,28 @@ int ext2_resize_fs(struct ext2_fs *fs, blk_t newsize)
 	if (fs->opt_verbose)
 		fprintf(stderr, "ext2_resize_fs\n");
 
-	residue = (newsize - fs->sb.s_first_data_block) % fs->sb.s_blocks_per_group;
+	residue = (newsize - EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb))
+		   % EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
 	if (residue && residue <= fs->adminblocks)
 	{
+/*
 		fprintf(stderr, "ext2_resize_fs: %i is an impossible size for an ext2 fs! rounding down to %i\n",
 			newsize, newsize-residue);
+*/
 		newsize -= residue;
 	}
 
-	if (newsize == fs->sb.s_blocks_count)
+	if (newsize == EXT2_SUPER_BLOCKS_COUNT(fs->sb))
 		return 1;
 
-	if ((fs->relocator_pool = (unsigned char *)malloc(ext2_relocator_pool_size << 10)) == NULL)
-	{
-		fprintf(stderr, "ext2_resize_fs: error allocating relocator pool!\n");
+	fs->relocator_pool
+		= (unsigned char *)ped_malloc(ext2_relocator_pool_size << 10);
+	if (!fs->relocator_pool)
 		return 0;
-	}
-	fs->relocator_pool_end = fs->relocator_pool + (ext2_relocator_pool_size << 10);
+	fs->relocator_pool_end
+		= fs->relocator_pool + (ext2_relocator_pool_size << 10);
 
-	if (newsize < fs->sb.s_blocks_count)
+	if (newsize < EXT2_SUPER_BLOCKS_COUNT(fs->sb))
 		status = ext2_shrink_fs(fs, newsize);
 	else
 		status = ext2_grow_fs(fs, newsize);
