@@ -1,7 +1,7 @@
 /*
     interface.c -- parted binding glue to libext2resize
-    Copyright (C) 1999 Lennert Buytenhek <buytenh@gnu.org>
-
+    Copyright (C) 1998-2000 Free Software Foundation, Inc.
+  
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -26,6 +26,12 @@ static const char _interface_c[] = "$Id: interface.c,v 1.6 1999/10/02 20:56:47 a
 #include "config.h"
 
 #include <parted/parted.h>
+#include <parted/disk_dos.h>
+#include <parted/disk_loop.h>
+#include <parted/disk_pc98.h>
+#include <parted/disk_mac.h>
+#include <parted/disk_bsd.h>
+#include <parted/disk_sun.h>
 #include "ext2.h"
 #include "parted_io.h"
 
@@ -33,46 +39,48 @@ struct ext2_dev_handle* ext2_make_dev_handle_from_parted_geometry(PedGeometry* g
 
 
 
-int
+static int
 _ext2_probe (const PedGeometry* geom);
 
-PedFileSystem*
+static int
+_ext2_clobber (PedGeometry* geom);
+
+static PedFileSystem*
 _ext2_open (const PedGeometry *geom);
 
-PedFileSystem*
-_ext2_create (const PedGeometry *geom);
+static PedFileSystem*
+_ext2_create (PedGeometry *geom);
 
-int
+static int
 _ext2_close (PedFileSystem *fs);
 
-int
+static int
 _ext2_check (PedFileSystem *fs);
 
-int
-_ext2_resize (PedFileSystem* fs, PedGeometry* geom);
-
-PedSector
-_ext2_get_min_size (const PedFileSystem* fs);
-
-char
-_ext2_get_system (const PedFileSystem* fs, const PedPartition* part,
-		  PedDiskType* disk_type);
+static int
+_ext2_resize (PedFileSystem* fs, const PedGeometry* geom);
 
+static PedConstraint*
+_ext2_get_resize_constraint (const PedFileSystem* fs);
 
+static int
+_ext2_set_system (const PedFileSystem* fs, PedPartition* part,
+		  const PedDiskType* disk_type);
 
 
 
 
 static PedFileSystemOps _ext2_ops = {
 	probe:		_ext2_probe,
+	clobber:	_ext2_clobber,
 	open:		_ext2_open,
 	create:         _ext2_create,
 	close:		_ext2_close,
 	check:          _ext2_check,
 	resize:		_ext2_resize,
 	copy:           NULL,
-	get_min_size:	_ext2_get_min_size,
-	get_system:	_ext2_get_system
+	get_resize_constraint:	_ext2_get_resize_constraint,
+	set_system:	_ext2_set_system
 };
 
 static PedFileSystemType _ext2_type = {
@@ -93,19 +101,33 @@ void ped_file_system_ext2_done ()
 
 
 
-int
+static int
 _ext2_probe (const PedGeometry* geom)
 {
 	struct ext2_super_block sb;
 
 	if (!ped_geometry_read(geom, &sb, 2, 2)
-	    || sb.s_magic != EXT2_SUPER_MAGIC)
+	    || EXT2_SUPER_MAGIC(sb) != EXT2_SUPER_MAGIC_CONST)
 		return 0;
 
 	return 1;
 }
 
-PedFileSystem*
+static int
+_ext2_clobber (PedGeometry* geom)
+{
+	struct ext2_super_block sb;
+
+	if (!ped_geometry_read(geom, &sb, 2, 2))
+		return 0;
+	if (EXT2_SUPER_MAGIC(sb) != EXT2_SUPER_MAGIC_CONST)
+		return 1;
+
+	sb.s_magic = 0;
+	return ped_geometry_write(geom, &sb, 2, 2);
+}
+
+static PedFileSystem*
 _ext2_open (const PedGeometry *geom)
 {
 	PedFileSystem*		fs;
@@ -137,8 +159,8 @@ error:
 	return NULL;
 }
 
-PedFileSystem*
-_ext2_create (const PedGeometry *geom)
+static PedFileSystem*
+_ext2_create (PedGeometry *geom)
 {
 	PedFileSystem*		fs;
 	struct ext2_fs*		fs_info;
@@ -169,7 +191,7 @@ error:
 	return NULL;
 }
 
-int
+static int
 _ext2_close (PedFileSystem *fs)
 {
 	struct ext2_dev_handle* handle;
@@ -182,7 +204,7 @@ _ext2_close (PedFileSystem *fs)
 	return 1;
 }
 
-int
+static int
 _ext2_check (PedFileSystem *fs)
 {
 	ped_exception_throw (PED_EXCEPTION_INFORMATION, PED_EXCEPTION_OK,
@@ -191,13 +213,13 @@ _ext2_check (PedFileSystem *fs)
 	return 1;
 }
 
-int
-_ext2_resize (PedFileSystem* fs, PedGeometry* geom)
+static int
+_ext2_resize (PedFileSystem* fs, const PedGeometry* geom)
 {
 	struct ext2_fs* f;
 	PedSector	old_length = fs->geom->length;
 
-	if (fs->geom->disk != geom->disk)
+	if (fs->geom->disk->dev != geom->disk->dev)
 	{
 		fprintf(stderr, "huh? ungrokkable!\n");
 		return 0;
@@ -211,6 +233,8 @@ _ext2_resize (PedFileSystem* fs, PedGeometry* geom)
 		return 0;
 	}
 
+	geom->disk->dev->boot_dirty = 1;
+
 	f = (struct ext2_fs *) fs->type_specific;
 
 /* ensure that the geometry contains the new and old geometry */
@@ -235,22 +259,104 @@ error:
 	return 0;
 }
 
-PedSector
-_ext2_get_min_size (const PedFileSystem* fs)
+static PedConstraint*
+_ext2_get_resize_constraint (const PedFileSystem* fs)
 {
 	struct ext2_fs* f = (struct ext2_fs *) fs->type_specific;
+	PedDisk*	disk = fs->geom->disk;
+	PedAlignment	start_align;
+	PedGeometry	start_sector;
+	PedGeometry	full_disk;
+	PedSector	min_size;
 
-	return f->sb.s_blocks_count - f->sb.s_free_blocks_count;
+	if (!ped_alignment_init (&start_align, fs->geom->start, 0))
+		return NULL;
+	if (!ped_geometry_init (&full_disk, disk, 0, disk->dev->length - 1))
+		return NULL;
+	if (!ped_geometry_init (&start_sector, disk, fs->geom->start, 1))
+		return NULL;
+	min_size = (EXT2_SUPER_BLOCKS_COUNT(f->sb)
+		   	- EXT2_SUPER_FREE_BLOCKS_COUNT(f->sb))
+		   * (f->blocksize / PED_SECTOR_SIZE);
+
+	return ped_constraint_new (&start_align, ped_alignment_any,
+				   &start_sector, &full_disk, min_size);
 }
 
-char
-_ext2_get_system (const PedFileSystem* fs, const PedPartition* part,
-		  PedDiskType* disk_type)
+static int
+_ext2_set_system (const PedFileSystem* fs, PedPartition* part,
+		  const PedDiskType* disk_type)
 {
-	if (part->hidden) {
-		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-				     _("Ext2 partitions can't be hidden."));
-		return 0;
+	if (strcmp (disk_type->name, LOOP_NAME) == 0)
+		return 1;
+
+	if (strcmp (disk_type->name, DOS_NAME) == 0) {
+		DosPartitionData*	dos_data = part->disk_specific;
+
+		PED_ASSERT (dos_data != NULL, return 0);
+
+		if (dos_data->raid) {
+			dos_data->system = 0xfd;
+			return 1;
+		}
+		if (dos_data->lvm) {
+			dos_data->system = 0x8e;
+			return 1;
+		}
+
+		if (dos_data->hidden) {
+			ped_exception_throw (
+				PED_EXCEPTION_ERROR,
+			       	PED_EXCEPTION_FIX,
+				_("Ext2 partitions can't be hidden on msdos "
+				  "disk labels."));
+			dos_data->hidden = 0;
+		}
+
+		dos_data->system = 0x83;
+		return 1;
 	}
-	return 0x83;
+
+	if (strcmp (disk_type->name, PC98_NAME) == 0) {
+		PC98PartitionData*	pc98_data = part->disk_specific;
+
+		PED_ASSERT (pc98_data != NULL, return 0);
+
+		pc98_data->boot = 1;
+		pc98_data->system = 0xa0e2;
+		return 1;
+	}
+
+	if (strcmp (disk_type->name, MAC_NAME) == 0) {
+		MacPartitionData*	mac_data = part->disk_specific;
+
+		PED_ASSERT (mac_data != NULL, return 0);
+
+		if (mac_data->is_boot)
+			strcpy (mac_data->system_name, "Apple_Bootstrap");
+		else
+			strcpy (mac_data->system_name, "Apple_UNIX_SVR2");
+		mac_data->status = 0x33;
+		return 1;
+	}
+
+	if (strcmp (disk_type->name, BSD_NAME) == 0) {
+		BSDPartitionData* bsd_data = part->disk_specific;
+		bsd_data->type = 0x8;
+		return 1;
+	}
+
+	if (strcmp (disk_type->name, SUN_NAME) == 0) {
+		SunPartitionData* sun_data = part->disk_specific;
+
+		if (sun_data->is_lvm) {
+			sun_data->type = 0x8e;
+			return 1;
+		}
+
+		sun_data->type = 0x83;
+		return 1;
+	}
+
+	return 0;
 }
