@@ -1,6 +1,6 @@
 /*
     libparted
-    Copyright (C) 1998-2000  Andrew Clausen  <clausen@gnu.org>
+    Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,32 +38,38 @@ fat_boot_sector_read (FatBootSector* bs, const PedGeometry *geom)
 
 	if (PED_LE16_TO_CPU (bs->boot_sign) != 0xAA55) {
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-			_("Filesystem has an invalid signature for a FAT "
-			  "filesystems."));
+			_("File system has an invalid signature for a FAT "
+			  "file systems."));
 		return 0;
 	}
 
 	if (!bs->system_id[0]) {
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-			_("Filesystem has an invalid signature for a FAT "
-			  "filesystem."));
+			_("File system has an invalid signature for a FAT "
+			  "file system."));
 		return 0;
 	}
 
 	if (!bs->sector_size || PED_LE16_TO_CPU (bs->sector_size) % 512) {
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-			_("Filesystem has an invalid sector size for a FAT "
-			  "filesystem."));
+			_("File system has an invalid sector size for a FAT "
+			  "file system."));
 		return 0;
 	}
 
 	if (!bs->reserved) {
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-			_("Filesystem has an invalid number of reserved "
+			_("File system has an invalid number of reserved "
 			  "sectors for a FAT filesystem."));
 		return 0;
 	}
 
+	if (bs->fats < 1 || bs->fats > 4) {
+		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
+			_("File system has an invalid number of FATS."));
+		return 0;
+	}
+
 	return 1;
 }
 
@@ -105,6 +111,7 @@ fat_boot_sector_analyse (FatBootSector* bs, PedFileSystem* fs)
 {
 	FatSpecific*		fs_info = FAT_SPECIFIC (fs);
 	PedExceptionOption	ex_status;
+	int			fat_entry_size;
 
 	PED_ASSERT (bs != NULL, return 0);
 	
@@ -115,20 +122,25 @@ fat_boot_sector_analyse (FatBootSector* bs, PedFileSystem* fs)
 			_("This file system has a logical sector size of %d.  "
 			"GNU Parted is known not to work properly with sector "
 			"sizes other than 512 bytes."),
-			PED_LE16_TO_CPU (bs->sector_size))
+			(int) PED_LE16_TO_CPU (bs->sector_size))
 				!= PED_EXCEPTION_IGNORE)
 			return 0;
 	}
 
 	fs_info->logical_sector_size = PED_LE16_TO_CPU (bs->sector_size) / 512;
-	fs_info->sector_count = PED_LE32_TO_CPU (bs->sector_count)
-					* fs_info->logical_sector_size;
+
+	if (bs->sectors)
+		fs_info->sector_count = PED_LE16_TO_CPU (bs->sectors)
+						* fs_info->logical_sector_size;
+	else
+		fs_info->sector_count = PED_LE32_TO_CPU (bs->sector_count)
+						* fs_info->logical_sector_size;
 
 	if (fs_info->sector_count != fs->geom->length) {
 		ex_status = ped_exception_throw (PED_EXCEPTION_WARNING,
 			PED_EXCEPTION_IGNORE_CANCEL,
 			_("Partition size (%ld sectors) and filesystem "
-			  "size (%d sectors) do not match."),
+			  "size (%ld sectors) do not match."),
 			(long) fs->geom->length,
 			(long) fs_info->sector_count);
 
@@ -138,6 +150,8 @@ fat_boot_sector_analyse (FatBootSector* bs, PedFileSystem* fs)
 
 			case PED_EXCEPTION_UNHANDLED:
 				ped_exception_catch ();
+				if (fs->geom->length < fs_info->sector_count)
+					return 0;
 			case PED_EXCEPTION_IGNORE:
 		}
 	}
@@ -213,6 +227,24 @@ fat_boot_sector_analyse (FatBootSector* bs, PedFileSystem* fs)
 		= (fs_info->sector_count - fs_info->cluster_offset)
 		  / fs_info->cluster_sectors;
 
+	fat_entry_size = fat_table_entry_size (fs_info->fat_type);
+	if (fs_info->cluster_count + 2
+			> fs_info->fat_sectors * 512 / fat_entry_size) {
+		ped_exception_throw (
+			PED_EXCEPTION_ERROR,
+			PED_EXCEPTION_CANCEL,
+			_("The FATs aren't big enough to describe all "
+			  "clusters!  Each FAT is %d sectors.  There are %d "
+			  "clusters, which would require each FAT to be %d "
+			  "sectors.  This is REALLY weird.  You might want to "
+			  "write us an email: bug-parted@gnu.org"),
+			(int) fs_info->fat_sectors,
+			(int) fs_info->cluster_count,
+			(int) div_round_up ((fs_info->cluster_count + 2)
+						* fat_entry_size, 512));
+		return 0;
+	}
+
 	fs_info->dir_entries_per_cluster
 		= fs_info->cluster_size / sizeof (FatDirEntry);
 	return 1;
@@ -273,7 +305,7 @@ fat_boot_sector_generate (FatBootSector* bs, const PedFileSystem* fs)
 		bs->u.fat32.fat_length = PED_CPU_TO_LE32 (fs_info->fat_sectors
 						/ fs_info->logical_sector_size);
 		bs->u.fat32.flags = 0;	/* FIXME: what the hell are these? */
-		bs->u.fat32.version = 0;	/* FIXME ??? */
+		bs->u.fat32.version = 0;  /* must be 0, for Win98 bootstrap */
 		bs->u.fat32.root_dir_cluster
 			= PED_CPU_TO_LE32 (fs_info->root_cluster);
 		bs->u.fat32.info_sector
