@@ -1,6 +1,6 @@
 /*
     libparted
-    Copyright (C) 1998-2000  Andrew Clausen  <clausen@gnu.org>
+    Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,39 +21,6 @@
 
 #include "fat.h"
 
-/* for debugging */
-static int
-search_remap (FatOpContext* ctx, FatCluster start, FatCluster cluster)
-{
-	FatSpecific*    old_fs_info = FAT_SPECIFIC (ctx->old_fs);
-	FatCluster      i;
-
-	PED_ASSERT (start < old_fs_info->cluster_count + 2, return 0);
-	
-	for (i = start; i < old_fs_info->cluster_count + 2; i++) {
-		if (ctx->remap [i] == cluster)
-			return i;
-	}
-
-	return 0;
-}
-
-/* for debugging */
-static int
-search_fat (FatTable* fat, FatCluster start, FatCluster cluster)
-{
-	FatCluster      i;
-
-	PED_ASSERT (start < fat->cluster_count + 2, return 0);
-	
-	for (i = start; i < fat->cluster_count + 2; i++) {
-		if (fat_table_get (fat, i) == cluster)
-			return i;
-	}
-
-	return 0;
-}
-
 static int
 needs_duplicating (const FatOpContext* ctx, FatFragment frag)
 {
@@ -408,8 +375,6 @@ init_remap (FatOpContext* ctx)
 int
 fat_duplicate_clusters (FatOpContext* ctx)
 {
-	FatCluster		i;
-
 	init_remap (ctx);
 
 	ctx->buffer_offset = 0;
