@@ -1,6 +1,6 @@
 /*
     libparted
-    Copyright (C) 1998-2000  Andrew Clausen  <clausen@gnu.org>
+    Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,20 +18,25 @@
 */
 
 #include <string.h>
+#include <parted/disk_dos.h>
+#include <parted/disk_loop.h>
+#include <parted/disk_pc98.h>
+#include <parted/disk_mac.h>
 
 #include "fat.h"
 #include "calc.h"
 
 static PedFileSystemOps fat_ops = {
 	probe:		fat_probe,
+	clobber:	fat_clobber,
 	open:		fat_open,
 	create:		fat_create,
 	close:		fat_close,
 	check:		fat_check,
 	resize:		fat_resize,
 	copy:		fat_copy,
-	get_min_size:	fat_get_min_size,
-	get_system:	fat_get_system
+	get_resize_constraint:	fat_get_resize_constraint,
+	set_system:	fat_set_system
 };
 
 static PedFileSystemType fat_type = {
@@ -145,26 +150,22 @@ fat_set_frag_sectors (PedFileSystem* fs, PedSector frag_sectors)
 int
 fat_probe (const PedGeometry* geom)
 {
-	PedFileSystem*		fs;
-	FatSpecific*		fs_info;
+	FatBootSector		boot_sector;
 
-	fs = fat_alloc ((PedGeometry*) geom);
-	if (!fs)
-		goto error;
-	fs_info = (FatSpecific*) fs->type_specific;
+	return fat_boot_sector_read (&boot_sector, geom);
+}
 
-	fs->type = &fat_type;
+int
+fat_clobber (PedGeometry* geom)
+{
+	FatBootSector		boot_sector;
 
-	if (!fat_boot_sector_read (&fs_info->boot_sector, fs->geom))
-		goto error_free_fs;
+	if (!fat_boot_sector_read (&boot_sector, geom))
+		return 1;
 
-	fat_free (fs);
-	return 1;
-
-error_free_fs:
-	fat_free (fs);
-error:
-	return 0;
+	boot_sector.system_id[0] = 0;
+	boot_sector.boot_sign = 0;
+        return ped_geometry_write (geom, &boot_sector, 0, 1);
 }
 
 static int
@@ -207,7 +208,7 @@ _init_fats (PedFileSystem* fs)
 error_free_copy:
 	fat_table_destroy (fat_copy);
 error_free_fat:
-	ped_free (fs_info->fat);
+	fat_table_destroy (fs_info->fat);
 error:
 	return 0;
 }
@@ -264,10 +265,11 @@ fat_root_dir_clear (PedFileSystem* fs)
 }
 
 PedFileSystem*
-fat_create (const PedGeometry* geom)
+fat_create (PedGeometry* geom)
 {
 	PedFileSystem*		fs;
 	FatSpecific*		fs_info;
+	FatCluster		table_size;
 
 	fs = fat_alloc (geom);
 	if (!fs)
@@ -400,8 +402,9 @@ fat_create (const PedGeometry* geom)
 			  + fs_info->fat_sectors * fs_info->fat_table_count;
 	}
 
-	fs_info->fat = fat_table_new (fs_info->fat_type,
-				      fs_info->cluster_count + 2);
+	table_size = fs_info->fat_sectors * 512
+		     / fat_table_entry_size (fs_info->fat_type);
+	fs_info->fat = fat_table_new (fs_info->fat_type, table_size);
 	if (!fs_info->fat)
 		goto error_free_fs;
 	if (!fat_alloc_buffers (fs)) 
@@ -463,7 +466,7 @@ fat_close (PedFileSystem* fs)
 
 /* Hack: just resize the filesystem outside of it's boundaries! */
 int
-fat_copy (PedFileSystem* fs, PedGeometry* geom)
+fat_copy (const PedFileSystem* fs, PedGeometry* geom)
 {
 	PedFileSystem*		new_fs;
 
@@ -488,7 +491,6 @@ fat_check (PedFileSystem* fs)
 	PedSector	cluster_sectors;
 	FatCluster	cluster_count;
 	PedSector	fat_sectors;
-	FatType		fat_type;
 	PedSector	align_sectors;
 	FatCluster	info_free_clusters;
 
@@ -521,8 +523,8 @@ fat_check (PedFileSystem* fs)
 				"Cluster size is %dk (%dk expected); "
 				"number of clusters is %d (%d expected); "
 				"size of FATs is %d sectors (%d expected)."),
-				fs_info->cluster_sectors / 2,
-					cluster_sectors / 2,
+				(int) fs_info->cluster_sectors / 2,
+					(int) cluster_sectors / 2,
 				(int) fs_info->cluster_count,
 					(int) cluster_count,
 				(int) fs_info->fat_sectors,
@@ -568,79 +570,190 @@ fat_check (PedFileSystem* fs)
 	return 1;	/* existence of fs implies consistency ;-) */
 }
 
-PedSector
-fat_get_min_size (const PedFileSystem* fs)
+static int
+_test_size (const PedFileSystem* fs, PedSector length, PedSector min_data_size)
 {
 	FatSpecific*	fs_info = FAT_SPECIFIC (fs);
-	FatCluster	min_cluster_count;
+	PedGeometry	geom;
+        PedSector	_cluster_sectors;
+	FatCluster	_cluster_count;
+	PedSector	_fat_size;
 
-	min_cluster_count = fs_info->fat->cluster_count
-			    - fs_info->fat->free_cluster_count
-			    + fs_info->total_dir_clusters
-			    + 32;
+	ped_geometry_init (&geom, fs->geom->disk, fs->geom->start, length);
 
-	return fs_info->cluster_offset
-	       + fs_info->cluster_sectors * min_cluster_count;
-}
-
-char
-fat_get_system (const PedFileSystem* fs, const PedPartition* part,
-		PedDiskType* disk_type)
-{
-	PedDevice*	dev;
-	int		is_lba;
-
-	dev = part->geom.disk->dev;
-	is_lba = (part->geom.end >= 1024 * dev->heads * dev->sectors);
-
-	if (!strcasecmp (disk_type->name, "loop"))
+	if (fat_calc_resize_sizes (
+				&geom,
+				fs_info->cluster_sectors,
+				FAT_TYPE_FAT16,
+				fs_info->root_dir_sector_count,
+				fs_info->cluster_sectors,
+				&_cluster_sectors,
+				&_cluster_count,
+				&_fat_size)
+			&& _cluster_sectors * _cluster_count >= min_data_size)
 		return 1;
 
-	if (strcasecmp (disk_type->name, "msdos")) {
-		ped_exception_throw (PED_EXCEPTION_BUG, PED_EXCEPTION_CANCEL,
-			_("FAT filesystems don't have an id for non-msdos "
-			"partition tables"));
-		return 0;
-	}
+	if (fat_calc_resize_sizes (
+				&geom,
+				fs_info->cluster_sectors,
+				FAT_TYPE_FAT32,
+				0,
+				fs_info->cluster_sectors,
+				&_cluster_sectors,
+				&_cluster_count,
+				&_fat_size)
+			&& _cluster_sectors * _cluster_count >= min_data_size)
+		return 1;
 
-	if (!fs) {
-		if (part->hidden) {
-			if (is_lba)
-				return 0x1e;
-			else
-				return 0x16;
-		} else {
-			if (is_lba)
-				return 0x0e;
+	return 0;
+}
+
+/* does a binary search (!) for the mininum size.  Too hard to compute directly
+ * (see calc_sizes() for why!)
+ */
+static PedSector
+_get_min_size (const PedFileSystem* fs, PedSector min_data_size)
+{
+	PedSector	min_length = 0;
+	PedSector	max_length = fs->geom->length;
+	PedSector	length;
+
+	while (min_length < max_length - 1) {
+		length = (min_length + max_length) / 2;
+		if (_test_size (fs, length, min_data_size))
+			max_length = length;
+		else
+			min_length = length;
+	}
+
+	return max_length;
+}
+
+PedConstraint*
+fat_get_resize_constraint (const PedFileSystem* fs)
+{
+	FatSpecific*	fs_info = FAT_SPECIFIC (fs);
+	PedDisk*	disk = fs->geom->disk;
+	PedGeometry	full_disk;
+	PedSector	min_cluster_count;
+	FatCluster	used_clusters;
+	PedSector	min_data_size;
+
+	if (!ped_geometry_init (&full_disk, disk, 0, disk->dev->length - 1))
+		return NULL;
+
+	used_clusters = fs_info->fat->cluster_count
+			- fs_info->fat->free_cluster_count;
+	min_cluster_count = used_clusters + fs_info->total_dir_clusters;
+	min_data_size = min_cluster_count * fs_info->cluster_sectors;
+
+/* adds a bit of leeway (32 sectors), for resolving extra issues, like root
+ * directory allocation, that aren't covered here. 
+ */
+	return ped_constraint_new (ped_alignment_any, ped_alignment_any,
+				   &full_disk,
+				   &full_disk,
+				   _get_min_size (fs, min_data_size) + 32);
+}
+
+int
+fat_set_system (const PedFileSystem* fs, PedPartition* part,
+		const PedDiskType* disk_type)
+{
+	PedDevice*		dev;
+
+	dev = part->geom.disk->dev;
+
+	if (strcasecmp (disk_type->name, LOOP_NAME) == 0)
+		return 1;
+
+	if (strcasecmp (disk_type->name, DOS_NAME) == 0) {
+		DosPartitionData*	dos_data = part->disk_specific;
+
+		PED_ASSERT (dos_data != NULL, return 0);
+
+		if (part->geom.end >= 1024 * dev->heads * dev->sectors)
+			dos_data->lba = 1;
+
+		if (dos_data->raid) {
+			dos_data->system = 0xfd;
+			return 1;
+		}
+		if (dos_data->lvm) {
+			dos_data->system = 0x8e;
+			return 1;
+		}
+
+		if (!fs) {
+			if (dos_data->hidden)
+				dos_data->system = dos_data->lba ? 0x1e : 0x16;
 			else
-				return 0x06;
+				dos_data->system = dos_data->lba ? 0x0e : 0x06;
+			return 1;
 		}
-	}
 
-	if (FAT_SPECIFIC (fs)->fat_type == FAT_TYPE_FAT32) {
-		if (part->hidden) {
-			if (is_lba)
-				return 0x1c;
+		if (FAT_SPECIFIC (fs)->fat_type == FAT_TYPE_FAT32) {
+			if (dos_data->hidden)
+				dos_data->system = dos_data->lba ? 0x1c : 0x1b;
 			else
-				return 0x1b;
+				dos_data->system = dos_data->lba ? 0x0c : 0x0b;
 		} else {
-			if (is_lba)
-				return 0x0c;
+			if (dos_data->hidden)
+				dos_data->system = dos_data->lba ? 0x1e : 0x16;
 			else
-				return 0x0b;
+				dos_data->system = dos_data->lba ? 0x0e : 0x06;
 		}
-	} else {
-		if (part->hidden) {
-			if (is_lba)
-				return 0x1e;
-			else
-				return 0x16;
-		} else {
-			if (is_lba)
-				return 0x0e;
+		return 1;
+	}
+
+	if (strcasecmp (disk_type->name, PC98_NAME) == 0) {
+		PC98PartitionData*	pc98_data = part->disk_specific;
+
+		PED_ASSERT (pc98_data != NULL, return 0);
+
+		if (!fs) {
+			pc98_data->system = 0x2021;
+			return 1;
+		} 
+
+		if ((pc98_data->system & 0x7000) != 0x2000
+		    || !pc98_data->boot
+		    || (FAT_SPECIFIC(fs)->fat_type == FAT_TYPE_FAT32))
+			pc98_data->system = (pc98_data->system & 0xFF) | 0x2000;
+
+		pc98_data->system &= 0xFF00;
+		switch (FAT_SPECIFIC(fs)->fat_type) {
+		case FAT_TYPE_FAT32:
+			pc98_data->system |= 0x61;
+			break;
+		case FAT_TYPE_FAT12:
+			pc98_data->system |= 0x01;
+			break;
+		case FAT_TYPE_FAT16:
+			PED_ASSERT(part != NULL, return 0);
+			if (part->geom.length * PED_SECTOR_SIZE >=
+			    32 * 1024 * 1024)
+				pc98_data->system |= 0x21;
 			else
-				return 0x06;
+				pc98_data->system |= 0x11;
+			break;
 		}
+		return 1;
+	}
+
+	if (strcasecmp (disk_type->name, MAC_NAME) == 0) {
+		MacPartitionData*	mac_data = part->disk_specific;
+
+		PED_ASSERT (mac_data != NULL, return 0);
+
+		if (mac_data->is_boot)
+			strcpy (mac_data->system_name, "Apple_Bootstrap");
+		else
+			strcpy (mac_data->system_name, "FAT");
+		mac_data->status = 0x33;
+		return 1;
 	}
+
+	return 0;
 }
 
