@@ -1,6 +1,6 @@
 /*
     libparted
-    Copyright (C) 1998-2000  Andrew Clausen  <clausen@gnu.org>
+    Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 
 #include <libintl.h>
 #if ENABLE_NLS
-#  define _(String) gettext (String)
+#  define _(String) dgettext (PACKAGE, String)
 #else
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
@@ -42,7 +42,7 @@
 
 #define BUFFER_SIZE  1024	/* buffer size in sectors (512 bytes) */
 
-typedef __s32			FatCluster;
+typedef __u32			FatCluster;
 typedef __s32			FatFragment;
 
 enum _FatType {
@@ -133,6 +133,7 @@ struct _FatSpecific {
 #define VOLUME_LABEL_ATTR	0x08
 #define VFAT_ATTR		0x0f
 #define DIRECTORY_ATTR		0x10
+#define ARCH_ATTR		0x20
 
 #define MAX_FAT12_CLUSTERS	4086      
 #define MAX_FAT16_CLUSTERS	65526
@@ -140,15 +141,16 @@ struct _FatSpecific {
 #define FAT_ROOT_DIR_ENTRY_COUNT	512
 
 extern int fat_probe (const PedGeometry* geom);
+extern int fat_clobber (PedGeometry* geom);
 extern PedFileSystem* fat_open (const PedGeometry* geom);
-extern PedFileSystem* fat_create (const PedGeometry* geom);
+extern PedFileSystem* fat_create (PedGeometry* geom);
 extern int fat_close (PedFileSystem* fs);
-extern int fat_resize (PedFileSystem* fs, PedGeometry* geom);
-extern int fat_copy (PedFileSystem* fs, PedGeometry* geom);
+extern int fat_resize (PedFileSystem* fs, const PedGeometry* geom);
+extern int fat_copy (const PedFileSystem* fs, PedGeometry* geom);
 extern int fat_check (PedFileSystem* fs);
-extern PedSector fat_get_min_size (const PedFileSystem* fs);
-extern char fat_get_system (const PedFileSystem* fs, const PedPartition* part,
-			    PedDiskType* disk_type);
+extern PedConstraint* fat_get_resize_constraint (const PedFileSystem* fs);
+extern int fat_set_system (const PedFileSystem* fs, PedPartition* part,
+			    const PedDiskType* disk_type);
 extern void fat_print (const PedFileSystem* fs);
 
 extern PedFileSystem* fat_alloc (const PedGeometry* geom);
@@ -158,9 +160,6 @@ extern void fat_free_buffers (PedFileSystem* fs);
 
 extern int fat_set_frag_sectors (PedFileSystem* fs, PedSector frag_sectors);
 
-#define MIN(a,b)	((a<b)? a : b)
-#define MAX(a,b)	(a>b) ? a : b;
-
 inline static PedSector
 div_round_up (PedSector a, PedSector b)
 {
