@@ -1,6 +1,6 @@
 /*
     libparted
-    Copyright (C) 1998-2000  Andrew Clausen  <clausen@gnu.org>
+    Copyright (C) 1998-2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -121,22 +121,25 @@ fat_table_read (FatTable* ft, const PedFileSystem* fs, int table_num)
 
 	PED_ASSERT (ft->raw_size >= fs_info->fat_sectors * 512, return 0);
 
+	memset (ft->table, 0, ft->raw_size);
+
         if (!ped_geometry_read (fs->geom, (void *) ft->table,
 				fs_info->fat_offset
 					+ table_num * fs_info->fat_sectors,
-				ft->raw_size / 512))
+				fs_info->fat_sectors))
 		return 0;
 
         if ( *((unsigned char*) ft->table) != fs_info->boot_sector.media) {
-		ped_exception_throw (PED_EXCEPTION_ERROR,
-				     PED_EXCEPTION_IGNORE_CANCEL,
-				     _("FAT %d media %x doesn't match the "
-				       "boot sector's media %x.  You should "
-				       "probably run scandisk."),
-				     (int) table_num + 1,
-				     (int) *((unsigned char*) ft->table),
-				     (int) fs_info->boot_sector.media);
-		return 0;
+		if (ped_exception_throw (
+			PED_EXCEPTION_ERROR,
+			PED_EXCEPTION_IGNORE_CANCEL,
+			_("FAT %d media %x doesn't match the boot sector's "
+			  "media %x.  You should probably run scandisk."),
+			(int) table_num + 1,
+			(int) *((unsigned char*) ft->table),
+			(int) fs_info->boot_sector.media)
+				!= PED_EXCEPTION_IGNORE)
+			return 0;
         }
 
 	ft->cluster_count = (fs_info->sector_count - fs_info->cluster_offset)
@@ -152,10 +155,12 @@ fat_table_write (const FatTable* ft, PedFileSystem* fs, int table_num)
 {
 	FatSpecific*	fs_info = FAT_SPECIFIC (fs);
 
+	PED_ASSERT (ft->raw_size >= fs_info->fat_sectors * 512, return 0);
+
         if (!ped_geometry_write (fs->geom, (void *) ft->table,
 				 fs_info->fat_offset
 					+ table_num * fs_info->fat_sectors,
-				 ft->raw_size / 512))
+				 fs_info->fat_sectors))
 		return 0;
 	if (!ped_geometry_sync (fs->geom))
 		return 0;
