@@ -1,6 +1,6 @@
 /*
     libparted
-    Copyright (C) 1998-2000  Andrew Clausen  <clausen@gnu.org>
+    Copyright (C) 1998-2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#ifndef PED_FAT_FATTABLE_H_INCLUDED
-#define PED_FAT_FATTABLE_H_INCLUDED
+#ifndef PED_FAT_TABLE_H_INCLUDED
+#define PED_FAT_TABLE_H_INCLUDED
 
 typedef struct _FatTable	FatTable;
 
@@ -44,7 +44,8 @@ extern void fat_table_clear (FatTable* ft);
 extern int fat_table_set_cluster_count (FatTable* ft,
 					FatCluster new_cluster_count);
 
-extern int fat_table_read (FatTable* ft, const PedFileSystem* fs, int table_num);
+extern int fat_table_read (FatTable* ft, const PedFileSystem* fs,
+			   int table_num);
 extern int fat_table_write (const FatTable* ft, PedFileSystem* fs,
 			    int table_num);
 extern int fat_table_write_all (const FatTable* ft, PedFileSystem* fs);
@@ -70,5 +71,5 @@ extern int fat_table_set_bad (FatTable* ft, FatCluster cluster);
 
 extern int fat_table_entry_size (FatType fat_type);
 
-#endif /* PED_FAT_FATTABLE_H_INCLUDED */
+#endif /* PED_FAT_TABLE_H_INCLUDED */
 
