@@ -1,6 +1,6 @@
 /*
     libparted
-    Copyright (C) 1998-2000  Andrew Clausen  <clausen@gnu.org>
+    Copyright (C) 1998-2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -39,12 +39,7 @@ static char tmp_buffer [4096];
 int
 fat_traverse_entries_per_buffer (FatTraverseInfo* trav_info)
 {
-	FatSpecific*		fs_info = FAT_SPECIFIC (trav_info->fs);
-
-	if (trav_info->is_legacy_root_dir)
-		return 512 / sizeof (FatDirEntry);
-	else
-		return fs_info->cluster_size / sizeof (FatDirEntry);
+	return trav_info->buffer_size / sizeof (FatDirEntry);
 }
 
 /* returns 1 if there are no more directory entries in the directory being
@@ -55,21 +50,19 @@ is_last_buffer (FatTraverseInfo* trav_info) {
 	FatSpecific*	fs_info = FAT_SPECIFIC (trav_info->fs);
     
 	if (trav_info->is_legacy_root_dir)
-		return trav_info->this_buffer.sector + 1
-			>= fs_info->root_dir_sector_count;
+		return 1;
 	else
-		return fat_table_is_eof (fs_info->fat,
-					 trav_info->next_buffer.cluster);
+		return fat_table_is_eof (fs_info->fat, trav_info->next_buffer);
 }
 
 static int
-write_root_dir_sector (FatTraverseInfo* trav_info)
+write_root_dir (FatTraverseInfo* trav_info)
 {
 	FatSpecific*	fs_info = FAT_SPECIFIC (trav_info->fs);
 	
 	if (!ped_geometry_write (trav_info->fs->geom, trav_info->dir_entries,
-				 fs_info->root_dir_offset
-					+ trav_info->this_buffer.sector, 1))
+				 fs_info->root_dir_offset,
+				 fs_info->root_dir_sector_count))
 		return 0;
 	if (!ped_geometry_sync (trav_info->fs->geom))
 		return 0;
@@ -82,7 +75,7 @@ write_dir_cluster (FatTraverseInfo* trav_info)
 {
 	if (!fat_write_sync_cluster (trav_info->fs,
 				     (void*) trav_info->dir_entries,
-				     trav_info->this_buffer.cluster))
+				     trav_info->this_buffer))
 		return 0;
 	trav_info->dirty = 0;
 	return 1;
@@ -92,58 +85,36 @@ static int
 write_dir_buffer (FatTraverseInfo* trav_info)
 {
 	if (trav_info->is_legacy_root_dir)
-		return write_root_dir_sector (trav_info);
+		return write_root_dir (trav_info);
 	else
 		return write_dir_cluster (trav_info);
 }
 
-
-static int
-read_next_root_dir_sector (FatTraverseInfo* trav_info)
-{
-	FatSpecific*	fs_info = FAT_SPECIFIC (trav_info->fs);
-
-	trav_info->this_buffer.sector = trav_info->next_buffer.sector;
-	trav_info->next_buffer.sector++;
-
-	return ped_geometry_read (trav_info->fs->geom, trav_info->dir_entries,
-				  fs_info->root_dir_offset
-					+ trav_info->this_buffer.sector, 1);
-}
-
-static int
-read_next_dir_cluster (FatTraverseInfo* trav_info)
-{
-	FatSpecific*	fs_info = FAT_SPECIFIC (trav_info->fs);
-
-	trav_info->this_buffer.cluster = trav_info->next_buffer.cluster;
-
-	if (trav_info->this_buffer.cluster < 2
-	    || trav_info->this_buffer.cluster >= fs_info->cluster_count + 2) {
-		ped_exception_throw (
-			PED_EXCEPTION_ERROR,
-			PED_EXCEPTION_CANCEL,
-			"Cluster %ld in directory %s is outside filesystem!",
-			(long) trav_info->this_buffer.cluster,
-			trav_info->dir_name);
-		return 0;
-	}
-
-	trav_info->next_buffer.cluster
-		= fat_table_get (fs_info->fat, trav_info->this_buffer.cluster);
-
-
-	return fat_read_cluster (trav_info->fs, (void *) trav_info->dir_entries,
-				 trav_info->this_buffer.cluster);
-}
-
 static int
 read_next_dir_buffer (FatTraverseInfo* trav_info)
 {
-	if (trav_info->is_legacy_root_dir)
-		return read_next_root_dir_sector (trav_info);
-	else
-		return read_next_dir_cluster (trav_info);
+	FatSpecific*	fs_info = FAT_SPECIFIC (trav_info->fs);
+
+	PED_ASSERT (!trav_info->is_legacy_root_dir, return 0);
+
+	trav_info->this_buffer = trav_info->next_buffer;
+
+	if (trav_info->this_buffer < 2
+	    || trav_info->this_buffer >= fs_info->cluster_count + 2) {
+		ped_exception_throw (
+			PED_EXCEPTION_ERROR,
+			PED_EXCEPTION_CANCEL,
+			"Cluster %ld in directory %s is outside filesystem!",
+			(long) trav_info->this_buffer,
+			trav_info->dir_name);
+		return 0;
+	}
+
+	trav_info->next_buffer
+		= fat_table_get (fs_info->fat, trav_info->this_buffer);
+
+	return fat_read_cluster (trav_info->fs, (void *) trav_info->dir_entries,
+				 trav_info->this_buffer);
 }
 
 /* FIXME: put into fat_dir_entry_* operations */
@@ -172,12 +143,13 @@ fat_traverse_begin (PedFileSystem* fs, FatCluster start_cluster,
 	trav_info->is_legacy_root_dir
 		= (fs_info->fat_type == FAT_TYPE_FAT16) && (start_cluster == 0);
 	trav_info->dirty = 0;
-	
+	trav_info->eof = 0;
+	trav_info->current_entry = -1;
+
 	if (trav_info->is_legacy_root_dir) {
-		trav_info->next_buffer.sector = 0;
-		trav_info->buffer_size = 512;
+		trav_info->buffer_size = 512 * fs_info->root_dir_sector_count;
 	} else {
-		trav_info->next_buffer.cluster = start_cluster;
+		trav_info->next_buffer = start_cluster;
 		trav_info->buffer_size = fs_info->cluster_size;
 	}
 
@@ -186,11 +158,15 @@ fat_traverse_begin (PedFileSystem* fs, FatCluster start_cluster,
 	if (!trav_info->dir_entries)
 		goto error_free_dir_name;
 
-	if (!read_next_dir_buffer (trav_info))
-		goto error_free_dir_entries;
-
-	trav_info->this_dir_entry = trav_info->dir_entries - 1;
-	trav_info->this_dir_entry_index = -1;
+	if (trav_info->is_legacy_root_dir) {
+		if (!ped_geometry_read (fs->geom, trav_info->dir_entries,
+					fs_info->root_dir_offset,
+					fs_info->root_dir_sector_count))
+			goto error_free_dir_entries;
+	} else {
+		if (!read_next_dir_buffer (trav_info))
+			goto error_free_dir_entries;
+	}
 
 	return trav_info;
 
@@ -232,33 +208,33 @@ fat_traverse_directory (FatTraverseInfo *trav_info, FatDirEntry* parent)
 FatDirEntry*
 fat_traverse_next_dir_entry (FatTraverseInfo *trav_info)
 {
-	trav_info->this_dir_entry++;
-	trav_info->this_dir_entry_index++;
+	if (trav_info->eof)
+		return NULL;
 
-	if (trav_info->this_dir_entry_index
+	trav_info->current_entry++;
+	if (trav_info->current_entry
 			>= fat_traverse_entries_per_buffer (trav_info)) {
 		if (trav_info->dirty) {
 			if (!write_dir_buffer (trav_info))
 				return NULL;
 		}
 
-		trav_info->this_dir_entry = trav_info->dir_entries;
-		trav_info->this_dir_entry_index = 0;
-
-		if (is_last_buffer (trav_info))
+		trav_info->current_entry = 0;
+		if (is_last_buffer (trav_info)) {
+			trav_info->eof = 1;
 			return NULL;
+		}
 		if (!read_next_dir_buffer (trav_info))
 			return NULL;
 	}
-	return trav_info->this_dir_entry;
+	return trav_info->dir_entries + trav_info->current_entry;
 }
 
 FatCluster
 fat_dir_entry_get_first_cluster (FatDirEntry* dir_entry, PedFileSystem *fs)
 {
 	FatSpecific*		fs_info = FAT_SPECIFIC (fs);
-	FatCluster		cluster;
-	
+
 	switch (fs_info->fat_type) {
 	case FAT_TYPE_FAT12:
 	case FAT_TYPE_FAT16:
@@ -299,6 +275,15 @@ fat_dir_entry_get_length (FatDirEntry* dir_entry)
 	return PED_LE32_TO_CPU (dir_entry->length);
 }
 
+int
+fat_dir_entry_is_null_term (const FatDirEntry* dir_entry)
+{
+	FatDirEntry	null_entry;
+
+	memset (&null_entry, 0, sizeof (null_entry));
+	return memcmp (&null_entry, dir_entry, sizeof (null_entry)) == 0;
+}
+
 int
 fat_dir_entry_is_active (FatDirEntry* dir_entry)
 {
