@@ -1,6 +1,6 @@
 /*
     libparted
-    Copyright (C) 1998-2000  Andrew Clausen  <clausen@gnu.org>
+    Copyright (C) 1998-2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,17 +36,13 @@ struct _FatTraverseInfo {
 	char*			dir_name;
 
 	int			is_legacy_root_dir;
-	int			buffer_size;
-	union {
-		FatCluster		cluster;
-		PedSector		sector;
-	} this_buffer, next_buffer;
+	int			dirty;
+	int			eof;
 
 	FatDirEntry*		dir_entries;
-	FatDirEntry*		this_dir_entry;
-	int			this_dir_entry_index;
-
-	int			dirty;
+	int			current_entry;
+	FatCluster		this_buffer, next_buffer;
+	int			buffer_size;
 };
 
 extern int fat_traverse_entries_per_buffer (FatTraverseInfo* trav_info);
@@ -73,6 +69,7 @@ extern void fat_dir_entry_set_first_cluster (FatDirEntry* dir_entry,
 
 extern __u32 fat_dir_entry_get_length (FatDirEntry* dir_entry);
 
+extern int fat_dir_entry_is_null_term (const FatDirEntry* dir_entry);
 extern int fat_dir_entry_is_file (FatDirEntry* dir_entry);
 extern int fat_dir_entry_is_system_file (FatDirEntry* dir_entry);
 extern int fat_dir_entry_is_directory (FatDirEntry* dir_entry);
