@@ -1,10 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998-2000  Andrew Clausen, Lennert Buytenhek and Red Hat Inc.
-
-	Andrew Clausen			<clausen@gnu.org>
-	Lennert Buytenhek		<buytenh@gnu.org>
-	Matt Wilson, Red Hat Inc.	<msw@redhat.com>
+    Copyright (C) 1998-2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,10 +23,16 @@
 
 #include <parted/parted.h>
 #include <parted/endian.h>
+#include <parted/disk_dos.h>
+#include <parted/disk_loop.h>
+#include <parted/disk_pc98.h>
+#include <parted/disk_mac.h>
+#include <parted/disk_bsd.h>
+#include <parted/disk_sun.h>
 
 #include <libintl.h>
 #if ENABLE_NLS
-#  define _(String) gettext (String)
+#  define _(String) dgettext (PACKAGE, String)
 #else
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
@@ -40,7 +42,7 @@
 #include <string.h>
 
 #define SWAP_SPECIFIC(fs) ((SwapSpecific*) (fs->type_specific))
-#define BUFFER_SIZE 32
+#define BUFFER_SIZE 128
 
 typedef struct {
 	char		page_map[1];
@@ -73,26 +75,28 @@ typedef struct {
 } SwapSpecific;
 
 static int swap_probe (const PedGeometry* geom);
+static int swap_clobber (PedGeometry* geom);
 static PedFileSystem* swap_open (const PedGeometry* geom);
-static PedFileSystem* swap_create (const PedGeometry* geom);
+static PedFileSystem* swap_create (PedGeometry* geom);
 static int swap_close (PedFileSystem* fs);
-static int swap_resize (PedFileSystem* fs, PedGeometry* geom);
-static int swap_copy (PedFileSystem* fs, PedGeometry* geom);
+static int swap_resize (PedFileSystem* fs, const PedGeometry* geom);
+static int swap_copy (const PedFileSystem* fs, PedGeometry* geom);
 static int swap_check (PedFileSystem* fs);
-static PedSector swap_get_min_size (const PedFileSystem* fs);
-static char swap_get_system (const PedFileSystem* fs, const PedPartition* part,
-			     PedDiskType* disk_type);
+static PedConstraint* swap_get_resize_constraint (const PedFileSystem* fs);
+static int swap_set_system (const PedFileSystem* fs, PedPartition* part,
+			    const PedDiskType* disk_type);
 
 static PedFileSystemOps swap_ops = {
 	probe:		swap_probe,
+	clobber:	swap_clobber,
 	open:		swap_open,
 	create:		swap_create,
 	close:		swap_close,
 	check:		swap_check,
 	copy:		swap_copy,
 	resize:		swap_resize,
-	get_min_size:	swap_get_min_size,
-	get_system:	swap_get_system
+	get_resize_constraint:	swap_get_resize_constraint,
+	set_system:	swap_set_system
 };
 
 static PedFileSystemType swap_type = {
@@ -125,6 +129,31 @@ swap_probe (const PedGeometry* geom)
 	return 0;
 }
 
+static int
+swap_clobber (PedGeometry* geom)
+{
+	PedFileSystem*		fs;
+	SwapSpecific*		swap_info;
+	char			buf[512];
+
+	fs = swap_open (geom);
+	if (!fs)
+		return 1;
+	swap_info = (SwapSpecific*) fs->type_specific;
+
+	memset (buf, 0, 512);
+	if (!ped_geometry_write (geom, buf, getpagesize() / 512 - 1, 1))
+		goto error_close_fs;
+
+	swap_close (fs);
+	return 1;
+
+error_close_fs:
+	swap_close (fs);
+error:
+	return 0;
+}
+
 static PedFileSystem*
 swap_alloc (const PedGeometry* geom)
 {
@@ -206,7 +235,7 @@ swap_open (const PedGeometry* geom)
 		fs_info->version = 1;
 	} else {
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-			_("Unrecognised linux swap signature %10s."),
+			_("Unrecognised linux swap signature '%10s'."),
 			fs_info->sig);
 		goto error_free_fs;
 	}
@@ -343,24 +372,27 @@ swap_write (PedFileSystem* fs, int rewrite, PedGeometry* old_geom)
 	SwapSpecific*	fs_info = SWAP_SPECIFIC (fs);
 
 	fs_info->page_sectors = getpagesize () / 512;
-	memset (fs_info->header, 0, fs_info->page_sectors * 512);
 	fs_info->sig = (char*) fs_info->header
 			+ fs_info->page_sectors * 512 - 10;
 
 	fs_info->page_count = fs->geom->length / fs_info->page_sectors;
-	if (fs_info->page_count > 8 * (fs_info->page_sectors * 512 - 10) ) {
-		if (rewrite) {
-			if (!ped_geometry_read (old_geom, fs_info->header,
-						0, 2))
-				return 0;
-		} else {
-			ped_exception_throw (PED_EXCEPTION_WARNING,
-				PED_EXCEPTION_OK,
-				_("This swap partition is not compatible with "
-				"Linux version 2.1.117 or earlier.  Use a "
-				"smaller partition (maximum size 128mb) if you "
-				"want to use old versions of Linux."));
-		}
+
+	if (!ped_geometry_read (fs->geom, fs_info->header, 0, 2))
+		return 0;
+	if (!rewrite) {
+/* not many people use 2.0 nowadays.  If they are, then there's no harm done...
+ * they'll just get a bit confused when they can't swap on...
+ */
+#if 0
+		ped_exception_throw (PED_EXCEPTION_WARNING,
+			PED_EXCEPTION_OK,
+			_("This swap partition is not compatible with "
+			"Linux version 2.1.117 or earlier.  Use a "
+			"smaller partition (maximum size 128mb) if you "
+			"want to use old versions of Linux."));
+#endif
+		memset ((char*) fs_info->header + 1024, 0,
+			fs_info->page_sectors * 512 - 1024);
 
 		fs_info->version = 1;
 		fs_info->header->new.version = 1;
@@ -369,10 +401,6 @@ swap_write (PedFileSystem* fs, int rewrite, PedGeometry* old_geom)
 		fs_info->max_bad_pages = (getpagesize()
 						- sizeof (SwapNewHeader)) / 4;
 		memcpy (fs_info->sig, "SWAPSPACE2", 10);
-	} else {
-		fs_info->version = 0;
-		fs_info->max_bad_pages = fs_info->page_count;
-		memcpy (fs_info->sig, "SWAP-SPACE", 10);
 	}
 	if (!swap_check_pages (fs))
 		return 0;
@@ -381,7 +409,7 @@ swap_write (PedFileSystem* fs, int rewrite, PedGeometry* old_geom)
 }
 
 static PedFileSystem*
-swap_create (const PedGeometry* geom)
+swap_create (PedGeometry* geom)
 {
 	PedFileSystem*		fs;
 
@@ -406,7 +434,7 @@ swap_close (PedFileSystem* fs)
 }
 
 static int
-swap_resize (PedFileSystem* fs, PedGeometry* geom)
+swap_resize (PedFileSystem* fs, const PedGeometry* geom)
 {
 	PedGeometry*	old_geom;
 	
@@ -424,7 +452,7 @@ error:
 }
 
 static int
-swap_copy (PedFileSystem* fs, PedGeometry* geom)
+swap_copy (const PedFileSystem* fs, PedGeometry* geom)
 {
 	PedFileSystem*	new_fs;
 
@@ -445,21 +473,102 @@ swap_check (PedFileSystem* fs)
 	return 1;
 }
 
-static PedSector
-swap_get_min_size (const PedFileSystem* fs)
+static PedConstraint*
+swap_get_resize_constraint (const PedFileSystem* fs)
 {
-	return getpagesize() / 512;
+	PedDisk*	disk = fs->geom->disk;
+	PedGeometry	full_disk;
+
+	if (!ped_geometry_init (&full_disk, disk, 0, disk->dev->length - 1))
+		return NULL;
+	
+	return ped_constraint_new (ped_alignment_any, ped_alignment_any,
+				   &full_disk, &full_disk, getpagesize() / 512);
 }
 
-static char
-swap_get_system (const PedFileSystem* fs, const PedPartition* part,
-		 PedDiskType* disk_type)
+static int
+swap_set_system (const PedFileSystem* fs, PedPartition* part,
+		 const PedDiskType* disk_type)
 {
-	if (part->hidden) {
-		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-			_("Linux-swap partitions can not be hidden."));
-		return 0;
+	if (strcasecmp (disk_type->name, LOOP_NAME) == 0)
+		return 1;
+
+	if (strcasecmp (disk_type->name, DOS_NAME) == 0) {
+		DosPartitionData*	dos_data = part->disk_specific;
+
+		PED_ASSERT (dos_data != NULL, return 0);
+
+		if (dos_data->raid) {
+			dos_data->system = 0xfd;
+			return 1;
+		}
+		if (dos_data->lvm) {
+			dos_data->system = 0x8e;
+			return 1;
+		}
+
+		if (dos_data->hidden) {
+			ped_exception_throw (
+				PED_EXCEPTION_ERROR,
+				PED_EXCEPTION_FIX,
+				_("Linux-swap partitions can not be hidden on "
+				  "msdos disk labels."));
+			dos_data->hidden = 0;
+		}
+
+		dos_data->system = 0x82;
+		return 1;
 	}
-	return 0x82;
+
+	if (strcasecmp (disk_type->name, PC98_NAME) == 0) {
+		PC98PartitionData*	pc98_data = part->disk_specific;
+
+		PED_ASSERT (pc98_data != NULL, return 0);
+
+		if (pc98_data->boot) {
+			ped_exception_throw (
+				PED_EXCEPTION_ERROR,
+				PED_EXCEPTION_FIX,
+				_("Linux-swap partitions can not be bootable "
+				  "on pc98 disk labels."));
+			pc98_data->boot = 0;
+		}
+
+		pc98_data->system = 0x20e2;
+		return 1;
+	}
+
+	if (strcmp (disk_type->name, MAC_NAME) == 0) {
+		MacPartitionData*       mac_data = part->disk_specific;
+
+		PED_ASSERT (mac_data != NULL, return 0);
+
+		if (mac_data->is_boot)
+			strcpy (mac_data->system_name, "Apple_Bootstrap");
+		else
+			strcpy (mac_data->system_name, "Apple_UNIX_SVR2");
+
+		if (!mac_data->is_swap) {
+			if (!ped_partition_set_flag (part, PED_PARTITION_SWAP,
+						     1))
+				return 0;
+		}
+		mac_data->status = 0x33;
+		return 1;
+	}
+
+	if (strcmp (disk_type->name, BSD_NAME) == 0) {
+		BSDPartitionData* bsd_data = part->disk_specific;
+		bsd_data->type = 0x1;
+		return 1;
+	}
+
+	if (strcmp (disk_type->name, SUN_NAME) == 0) {
+		SunPartitionData* sun_data = part->disk_specific;
+		sun_data->type = 0x82;
+		return 1;
+	}
+
+	return 0;	
 }
 
