@@ -1,10 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998-2000  Andrew Clausen, Lennert Buytenhek and Red Hat Inc.
-
-	Andrew Clausen			<clausen@gnu.org>
-	Lennert Buytenhek		<buytenh@gnu.org>
-	Matt Wilson, Red Hat Inc.	<msw@redhat.com>
+    Copyright (C) 1999, 2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,28 +23,40 @@
 
 #include <libintl.h>
 #if ENABLE_NLS
-#  define _(String) gettext (String)
+#  define _(String) dgettext (PACKAGE, String)
 #else
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
 
-#define MIN(a,b) ( (a<b)? (a) : (b) )
+int
+ped_geometry_init (PedGeometry* geom, const PedDisk* disk,
+		   PedSector start, PedSector length)
+{
+	PED_ASSERT (geom != NULL, return 0);
+	PED_ASSERT (disk != NULL, return 0);
+
+	geom->disk = (PedDisk*) disk;
+	return ped_geometry_set (geom, start, length);
+}
 
 PedGeometry*
 ped_geometry_new (const PedDisk* disk, PedSector start, PedSector length)
 {
-	PedGeometry*	result;
+	PedGeometry*	geom;
 
 	PED_ASSERT (disk != NULL, return NULL);
 
-	result = (PedGeometry*) ped_malloc (sizeof (PedGeometry));
-	if (!result)
-		return NULL;
+	geom = (PedGeometry*) ped_malloc (sizeof (PedGeometry));
+	if (!geom)
+		goto error;
+	if (!ped_geometry_init (geom, disk, start, length))
+		goto error_free_geom;
+	return geom;
 
-	result->disk = (PedDisk*) disk;
-	ped_geometry_set (result, start, length);
-
-	return result;
+error_free_geom:
+	ped_free (geom);
+error:
+	return NULL;
 }
 
 PedGeometry*
@@ -58,6 +66,26 @@ ped_geometry_duplicate (const PedGeometry* geom)
 	return ped_geometry_new (geom->disk, geom->start, geom->length);
 }
 
+PedGeometry*
+ped_geometry_intersect (const PedGeometry* a, const PedGeometry* b)
+{
+	PedSector	start;
+	PedSector	end;
+
+	PED_ASSERT (a != NULL, return NULL);
+	PED_ASSERT (b != NULL, return NULL);
+
+	if (a->disk->dev != b->disk->dev)
+		return NULL;
+
+	start = PED_MAX (a->start, b->start);
+	end = PED_MIN (a->end, b->end);
+	if (start > end)
+		return NULL;
+
+	return ped_geometry_new (a->disk, start, end - start + 1);
+}
+
 void
 ped_geometry_destroy (PedGeometry* geom)
 {
@@ -67,25 +95,56 @@ ped_geometry_destroy (PedGeometry* geom)
 	ped_free (geom);
 }
 
-void
+int
 ped_geometry_set (PedGeometry* geom, PedSector start, PedSector length)
 {
-	PED_ASSERT (length >= 0, return);
+	PED_ASSERT (geom != NULL, return 0);
+	PED_ASSERT (geom->disk != NULL, return 0);
+	PED_ASSERT (geom->disk->dev != NULL, return 0);
+
+	if (length < 1) {
+		ped_exception_throw (
+			PED_EXCEPTION_ERROR,
+			PED_EXCEPTION_CANCEL,
+			_("Can't have the end before the start!"));
+		return 0;
+	}
+	if (start < 0 || start + length - 1 >= geom->disk->dev->length) {
+		ped_exception_throw (
+			PED_EXCEPTION_ERROR,
+			PED_EXCEPTION_CANCEL,
+			_("Can't have a partition outside the disk!"));
+		return 0;
+ 	}
 
 	geom->start = start;
 	geom->length = length;
 	geom->end = start + length - 1;
-}
 
-void
-ped_geometry_set_start (PedGeometry* geom, PedSector start)
-{
-	ped_geometry_set (geom, start, geom->end - start + 1);
+	return 1;
 }
 
 int
-ped_geometry_test_overlap (PedGeometry* a, PedGeometry* b)
+ped_geometry_set_start (PedGeometry* geom, PedSector start)
 {
+	return ped_geometry_set (geom, start, geom->end - start + 1);
+}
+
+int
+ped_geometry_set_end (PedGeometry* geom, PedSector end)
+{
+	return ped_geometry_set (geom, geom->start, end - geom->start + 1);
+}
+
+int
+ped_geometry_test_overlap (const PedGeometry* a, const PedGeometry* b)
+{
+	PED_ASSERT (a != NULL, return 0);
+	PED_ASSERT (b != NULL, return 0);
+
+	if (a->disk->dev != b->disk->dev)
+		return 0;
+
 	if (a->start < b->start)
 		return a->end >= b->start;
 	else
@@ -93,15 +152,34 @@ ped_geometry_test_overlap (PedGeometry* a, PedGeometry* b)
 }
 
 int
-ped_geometry_test_inside (PedGeometry* a, PedGeometry* b)
+ped_geometry_test_inside (const PedGeometry* a, const PedGeometry* b)
 {
+	PED_ASSERT (a != NULL, return 0);
+	PED_ASSERT (b != NULL, return 0);
+
+	if (a->disk->dev != b->disk->dev)
+		return 0;
+
 	return b->start >= a->start && b->end <= a->end;
 }
 
-void
-ped_geometry_set_end (PedGeometry* geom, PedSector end)
+int
+ped_geometry_test_equal (const PedGeometry* a, const PedGeometry* b)
 {
-	ped_geometry_set (geom, geom->start, end - geom->start + 1);
+	PED_ASSERT (a != NULL, return 0);
+	PED_ASSERT (b != NULL, return 0);
+
+	return a->disk->dev == b->disk->dev
+	       && a->start == b->start
+	       && a->end == b->end;
+}
+
+int
+ped_geometry_test_sector_inside (const PedGeometry* geom, PedSector sector)
+{
+	PED_ASSERT (geom != NULL, return 0);
+
+	return sector >= geom->start && sector <= geom->end;
 }
 
 int
@@ -113,13 +191,15 @@ ped_geometry_read (const PedGeometry* geom, void* buffer, PedSector start,
 
 	PED_ASSERT (geom != NULL, return 0);
 	PED_ASSERT (buffer != NULL, return 0);
+	PED_ASSERT (start >= 0, return 0);
+	PED_ASSERT (count >= 0, return 0);
 
 	if (real_start + count - 1 > geom->end) {
 		exception_status = ped_exception_throw (
 			PED_EXCEPTION_ERROR, PED_EXCEPTION_IGNORE_CANCEL,
 			_("Attempt to read sectors %ld-%ld outside of "
 			  "partition on %s"),
-			(long) start, (long) start + count - 1,
+			(long) start, (long) (start + count - 1),
 			geom->disk->dev->path);
 		return exception_status == PED_EXCEPTION_IGNORE;
 	}
@@ -145,6 +225,8 @@ ped_geometry_write (PedGeometry* geom, const void* buffer, PedSector start,
 
 	PED_ASSERT (geom != NULL, return 0);
 	PED_ASSERT (buffer != NULL, return 0);
+	PED_ASSERT (start >= 0, return 0);
+	PED_ASSERT (count >= 0, return 0);
 
 	if (real_start + count - 1 > geom->end) {
 		exception_status = ped_exception_throw (
@@ -152,7 +234,7 @@ ped_geometry_write (PedGeometry* geom, const void* buffer, PedSector start,
 			PED_EXCEPTION_IGNORE_CANCEL,
 			_("Attempt to write sectors %ld-%ld outside of "
 			  "partition on %s"),
-			(long) start, (long) start + count - 1,
+			(long) start, (long) (start + count - 1),
 			geom->disk->dev->path);
 		return exception_status == PED_EXCEPTION_IGNORE;
 	}
@@ -178,7 +260,7 @@ ped_geometry_check (PedGeometry* geom, void* buffer, PedSector buffer_size,
 retry:
 	ped_exception_fetch_all();
 	for (group = offset; group < offset + count; group += buffer_size) {
-		read_len = MIN (buffer_size, offset + count - group);
+		read_len = PED_MIN (buffer_size, offset + count - group);
 		if (!ped_geometry_read (geom, buffer, group, read_len))
 			goto found_error;
 	}
