@@ -1,10 +1,6 @@
 /*
     parted - a frontend to libparted
-    Copyright (C) 1999-2000  Andrew Clausen, Lennert Buytenhek and Red Hat, Inc.
-
-    Andrew Clausen		<clausen@gnu.org>
-    Lennert Buytenhek		<buytenh@gnu.org>
-    Matt Wilson, Red Hat Inc.	<msw@redhat.com>
+    Copyright (C) 1999, 2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,7 +25,7 @@
 #include <locale.h>
 #define N_(String) String
 #if ENABLE_NLS
-#  define _(String) gettext (String)
+#  define _(String) dgettext (PACKAGE, String)
 #else
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
@@ -41,6 +37,10 @@
 #include <string.h>
 #include <unistd.h>
 
+#ifdef ENABLE_MTRACE
+#include <mcheck.h>
+#endif
+
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 
@@ -65,35 +65,42 @@ static char*	options_help [][2] = {
 int	opt_script_mode;
 
 static char* minor_msg = N_(
-"MINOR is the partition number used by Linux.  The primary partitions number "
-"from 1-4, and logical partitions are 5 onwards.\n");
+"MINOR is the partition number used by Linux.  On msdos disk labels, the "
+"primary partitions number from 1-4, and logical partitions are 5 onwards.\n");
 
 static char* label_type_msg_start = N_("LABEL-TYPE is one of: ");
+static char* flag_msg_start =	N_("FLAG is one of: ");
 static char* part_type_msg =	N_("PART-TYPE is one of: primary, logical, "
 			           "extended\n");
 static char* fs_type_msg_start = N_("FS-TYPE is one of: ");
 static char* start_end_msg =	N_("START and END are in megabytes\n");
+static char* state_msg =	N_("STATE is one of: on, off\n");
 static char* device_msg =	N_("DEVICE is usually /dev/hda or /dev/sda\n");
+static char* name_msg =		N_("NAME is any word you want\n");
 
 static char* label_type_msg;
 static char* fs_type_msg;
+static char* flag_msg;
 
 static Command*	commands [256] = {NULL};
 
-static int do_boot (PedDevice* dev);
-static int do_check (PedDevice* dev);
-static int do_cp (PedDevice* dev);
-int do_help (PedDevice* dev);
-static int do_hide (PedDevice* dev);
-static int do_mklabel (PedDevice* dev);
-static int do_mkfs (PedDevice* dev);
-static int do_mkpart (PedDevice* dev);
-static int do_mkpartfs (PedDevice* dev);
-static int do_print (PedDevice* dev);
-static int do_quit (PedDevice* dev);
-static int do_resize (PedDevice* dev);
-static int do_rm (PedDevice* dev);
-static int do_unhide (PedDevice* dev);
+static int do_boot (PedDevice** dev);
+static int do_check (PedDevice** dev);
+static int do_cp (PedDevice** dev);
+int do_help (PedDevice** dev);
+static int do_hide (PedDevice** dev);
+static int do_mklabel (PedDevice** dev);
+static int do_mkfs (PedDevice** dev);
+static int do_mkpart (PedDevice** dev);
+static int do_mkpartfs (PedDevice** dev);
+static int do_print (PedDevice** dev);
+static int do_quit (PedDevice** dev);
+static int do_resize (PedDevice** dev);
+static int do_rm (PedDevice** dev);
+static int do_unhide (PedDevice** dev);
+static int do_name (PedDevice** dev);
+
+static void _done (PedDevice* dev);
 
 void
 help_on (char* topic)
@@ -107,52 +114,18 @@ help_on (char* topic)
 }
 
 static int
-do_boot (PedDevice* dev)
-{
-	PedDisk*	disk;
-	PedPartition*	part;
-
-	disk = ped_disk_open (dev);
-	if (!disk)
-		goto error;
-
-	if (!is_integer ()) {
-		help_on ("boot");
-		goto error_close_disk;
-	}
-	part = ped_disk_get_partition (disk, get_integer ());
-	if (!part) {
-		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-			_("Partition doesn't exist."));
-		goto error_close_disk;
-	}
-
-	if (!ped_disk_set_boot_partition (disk, part))
-		goto error_close_disk;
-	if (!ped_disk_write (disk))
-		goto error_close_disk;
-	ped_disk_close (disk);
-	return 1;
-
-error_close_disk:
-	ped_disk_close (disk);
-error:
-	return 0;
-}
-
-static int
-do_check (PedDevice* dev)
+do_check (PedDevice** dev)
 {
 	PedDisk*	disk;
 	PedFileSystem*	fs;
 	PedPartition*	part;
 
-	disk = ped_disk_open (dev);
+	disk = ped_disk_open (*dev);
 	if (!disk)
 		goto error;
 
 	if (!is_integer ()) {
-		help_on (_("check"));
+		help_on ("check");
 		goto error_close_disk;
 	}
 	part = ped_disk_get_partition (disk, get_integer ());
@@ -176,22 +149,37 @@ error:
 }
 
 static int
-do_cp (PedDevice* dev)
+do_cp (PedDevice** dev)
 {
 	PedDisk*		src_disk = NULL;
 	PedDisk*		dst_disk = NULL;
-	PedDevice*		dst_device;
+	PedDevice*		src_device;
 	PedPartition*		src = NULL;
 	PedPartition*		dst = NULL;
 	PedFileSystem*		src_fs = NULL;
+	PedFileSystemType*	src_fs_type = NULL;
 
-	src_disk = ped_disk_open (dev);
-	if (!src_disk)
+	dst_disk = ped_disk_open (*dev);
+	if (!dst_disk)
 		goto error;
 
 /* figure out source partition */
 	if (!is_integer ()) {
-		help_on (_("cp"));
+		char*	src_device_name = get_word ();
+		if (!src_device_name) {
+			help_on ("cp");
+			goto error_close_disk;
+		}
+		src_device = ped_device_get (src_device_name);
+		if (!src_device)
+			goto error_close_disk;
+		src_disk = ped_disk_open (src_device);
+	} else {
+		src_device = *dev;
+		src_disk = dst_disk;
+	}
+	if (!is_integer ()) {
+		help_on ("cp");
 		goto error_close_disk;
 	}
 	src = ped_disk_get_partition (src_disk, get_integer ());
@@ -208,15 +196,7 @@ do_cp (PedDevice* dev)
 
 /* figure out target partition */
 	if (!is_integer ()) {
-		dst_device = ped_device_get (get_word ());
-		if (!dst_device)
-			goto error_close_disk;
-		dst_disk = ped_disk_open (dst_device);
-	} else {
-		dst_disk = src_disk;
-	}
-	if (!is_integer ()) {
-		help_on (_("cp"));
+		help_on ("cp");
 		goto error_close_disk;
 	}
 	dst = ped_disk_get_partition (dst_disk, get_integer ());
@@ -233,16 +213,23 @@ do_cp (PedDevice* dev)
 				!= PED_EXCEPTION_IGNORE)
 			goto error_close_disk;
 	}
-	dst->type = src->type;
 
 /* do the copy */
 	src_fs = ped_file_system_open (&src->geom);
 	if (!src_fs)
 		goto error_close_disk;
+	src_fs_type = src_fs->type;
 	if (!ped_file_system_copy (src_fs, &dst->geom))
 		goto error_close_fs;
 	ped_file_system_close (src_fs);
-	if (dst_disk && src_disk != dst_disk)
+
+
+/* update the partition table, close disks */
+	if (!ped_partition_set_system (dst, src_fs_type))
+		goto error_close_disk;
+	if (!ped_disk_write (dst_disk))
+		goto error_close_disk;
+	if (dst_disk != src_disk)
 		ped_disk_close (dst_disk);
 	ped_disk_close (src_disk);
 	return 1;
@@ -250,9 +237,9 @@ do_cp (PedDevice* dev)
 error_close_fs:
 	ped_file_system_close (src_fs);
 error_close_disk:
-	if (dst_disk && src_disk != dst_disk)
-		ped_disk_close (dst_disk);
-	ped_disk_close (src_disk);
+	if (src_disk && src_disk != dst_disk)
+		ped_disk_close (src_disk);
+	ped_disk_close (dst_disk);
 error:
 	return 0;
 }
@@ -280,7 +267,7 @@ print_options_help ()
 }
 
 int
-do_help (PedDevice* dev)
+do_help (PedDevice** dev)
 {
 	if (peek_word ())
 		help_on (get_word ());
@@ -290,47 +277,13 @@ do_help (PedDevice* dev)
 }
 
 static int
-do_hide (PedDevice* dev)
-{
-	PedDisk*	disk;
-	PedPartition*	part;
-
-	disk = ped_disk_open (dev);
-	if (!disk)
-		goto error;
-
-	if (!is_integer ()) {
-		help_on (_("hide"));
-		goto error_close_disk;
-	}
-	part = ped_disk_get_partition (disk, get_integer ());
-	if (!part) {
-		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-			_("Partition doesn't exist."));
-		goto error_close_disk;
-	}
-
-	if (!ped_partition_set_hide_state (part, 1))
-		goto error_close_disk;
-	if (!ped_disk_write (disk))
-		goto error_close_disk;
-	ped_disk_close (disk);
-	return 1;
-
-error_close_disk:
-	ped_disk_close (disk);
-error:
-	return 0;
-}
-
-static int
-do_mklabel (PedDevice* dev)
+do_mklabel (PedDevice** dev)
 {
 	PedDisk*	disk;
 	PedDiskType*	type;
 
 	ped_exception_fetch_all ();
-	disk = ped_disk_open (dev);
+	disk = ped_disk_open (*dev);
 	if (!disk) ped_exception_catch ();
 	ped_exception_leave_all ();
 
@@ -356,23 +309,27 @@ do_mklabel (PedDevice* dev)
 	if (!type)
 		goto error_syntax;
 
-	ped_disk_create (dev, type);
+	disk = ped_disk_create (*dev, type);
+	if (!disk)
+		goto error;
+	ped_disk_close (disk);
 	return 1;
 
 error_syntax:
-	help_on (_("mklabel"));
+	help_on ("mklabel");
+error:
 	return 0;
 }
 
 static int
-do_mkfs (PedDevice* dev)
+do_mkfs (PedDevice** dev)
 {
 	PedDisk*		disk;
 	PedPartition*		part;
 	PedFileSystemType*	type;
 	PedFileSystem*		fs;
 
-	disk = ped_disk_open (dev);
+	disk = ped_disk_open (*dev);
 	if (!disk)
 		goto error;
 
@@ -415,7 +372,7 @@ do_mkfs (PedDevice* dev)
 	return 1;
 
 error_syntax:
-	help_on (_("mkfs"));
+	help_on ("mkfs");
 error_close_disk:
 	ped_disk_close (disk);
 error:
@@ -423,35 +380,39 @@ error:
 }
 
 static int
-do_mkpart (PedDevice* dev)
+do_mkpart (PedDevice** dev)
 {
 	PedDisk*			disk;
 	PedPartition*			part;
 	PedPartitionType		part_type;
 	PedFileSystemType*		fs_type;
 	PedSector			start, end;
+	PedConstraint*			constraint;
 
-	disk = ped_disk_open (dev);
+	disk = ped_disk_open (*dev);
 	if (!disk)
 		goto error;
+	constraint = ped_constraint_any (disk);
+	if (!constraint)
+		goto error_close_disk;
 
 	if (count_words () < 2) {
-		help_on (_("mkpart"));
-		goto error_close_disk;
+		help_on ("mkpart");
+		goto error_destroy_constraint;
 	}
 	switch (get_word() [0]) {
 		case 'p': part_type = PED_PARTITION_PRIMARY; break;
 		case 'e': part_type = PED_PARTITION_EXTENDED; break;
 		case 'l': part_type = PED_PARTITION_LOGICAL; break;
-		default: help_on (_("mkpart")); goto error_close_disk;
+		default: help_on ("mkpart"); goto error_destroy_constraint;
 	}
 
 	if (part_type == PED_PARTITION_EXTENDED) {
 		fs_type = NULL;
 	} else {
 		if (!peek_word ()) {
-			help_on (_("mkpart"));
-			goto error_close_disk;
+			help_on ("mkpart");
+			goto error_destroy_constraint;
 		}
 
 		fs_type = ped_file_system_type_get (get_word ());
@@ -459,24 +420,24 @@ do_mkpart (PedDevice* dev)
 			ped_exception_throw (PED_EXCEPTION_ERROR,
 				PED_EXCEPTION_CANCEL,
 				_("Unknown file system type."));
-			goto error_close_disk;
+			goto error_destroy_constraint;
 		}
 	}
 
 	if (!is_sector ()) {
-		help_on (_("mkpart"));
-		goto error_close_disk;
+		help_on ("mkpart");
+		goto error_destroy_constraint;
 	}
 	start = get_sector ();
 	if (!is_sector ()) {
-		help_on (_("mkpart"));
-		goto error_close_disk;
+		help_on ("mkpart");
+		goto error_destroy_constraint;
 	}
 	end = get_sector ();
 	part = ped_partition_new (disk, part_type, fs_type, start, end);
 	if (!part)
-		goto error_close_disk;
-	if (!ped_disk_add_partition (disk, part))
+		goto error_destroy_constraint;
+	if (!ped_disk_add_partition (disk, part, constraint))
 		goto error_destroy_part;
 
 	/* FIXME:  ped_partition_new() doesn't probe the file system for
@@ -488,11 +449,14 @@ do_mkpart (PedDevice* dev)
        	ped_partition_set_system (part, fs_type);
 
 	ped_disk_write (disk);
+	ped_constraint_destroy (constraint);
 	ped_disk_close (disk);
 	return 1;
 
 error_destroy_part:
 	ped_partition_destroy (part);
+error_destroy_constraint:
+	ped_constraint_destroy (constraint);
 error_close_disk:
 	ped_disk_close (disk);
 error:
@@ -501,36 +465,40 @@ error:
 
 /* FIXME - this is a mess */
 static int
-do_mkpartfs (PedDevice* dev)
+do_mkpartfs (PedDevice** dev)
 {
 	PedDisk*			disk;
 	PedPartition*			part;
 	PedPartitionType		part_type;
 	PedFileSystemType*		fs_type;
 	PedFileSystem*			fs;
+	PedConstraint*			constraint;
 	PedSector			start, end;
 
-	disk = ped_disk_open (dev);
+	disk = ped_disk_open (*dev);
 	if (!disk)
 		goto error;
+	constraint = ped_constraint_any (disk);
+	if (!constraint)
+		goto error_close_disk;
 
 	if (count_words () < 2) {
-		help_on (_("mkpartfs"));
-		goto error_close_disk;
+		help_on ("mkpartfs");
+		goto error_destroy_constraint;
 	}
 	switch (get_word() [0]) {
 		case 'p': part_type = PED_PARTITION_PRIMARY; break;
 		case 'l': part_type = PED_PARTITION_LOGICAL; break;
 		case 'e':
-		default: help_on (_("mkpartfs")); goto error_close_disk;
+		default: help_on ("mkpartfs"); goto error_destroy_constraint;
 	}
 
 	if (part_type == PED_PARTITION_EXTENDED) {
 		fs_type = NULL;
 	} else {
 		if (!peek_word ()) {
-			help_on (_("mkpartfs"));
-			goto error_close_disk;
+			help_on ("mkpartfs");
+			goto error_destroy_constraint;
 		}
 
 		fs_type = ped_file_system_type_get (get_word ());
@@ -538,24 +506,24 @@ do_mkpartfs (PedDevice* dev)
 			ped_exception_throw (PED_EXCEPTION_ERROR,
 				PED_EXCEPTION_CANCEL,
 				_("Unknown file system type."));
-			goto error_close_disk;
+			goto error_destroy_constraint;
 		}
 	}
 
 	if (!is_sector ()) {
-		help_on (_("mkpartfs"));
-		goto error_close_disk;
+		help_on ("mkpartfs");
+		goto error_destroy_constraint;
 	}
 	start = get_sector ();
 	if (!is_sector ()) {
-		help_on (_("mkpartfs"));
-		goto error_close_disk;
+		help_on ("mkpartfs");
+		goto error_destroy_constraint;
 	}
 	end = get_sector ();
 	part = ped_partition_new (disk, part_type, fs_type, start, end);
 	if (!part)
-		goto error_close_disk;
-	if (!ped_disk_add_partition (disk, part))
+		goto error_destroy_constraint;
+	if (!ped_disk_add_partition (disk, part, constraint))
 		goto error_destroy_part;
 
 	fs = ped_file_system_create (&part->geom, fs_type);
@@ -569,6 +537,7 @@ do_mkpartfs (PedDevice* dev)
 	ped_partition_set_system (part, fs_type);
 
 	ped_disk_write (disk);
+	ped_constraint_destroy (constraint);
 	ped_disk_close (disk);
 	return 1;
 
@@ -577,6 +546,8 @@ error_remove_part:
 	goto error_close_disk;
 error_destroy_part:
 	ped_partition_destroy (part);
+error_destroy_constraint:
+	ped_constraint_destroy (constraint);
 error_close_disk:
 	ped_disk_close (disk);
 error:
@@ -584,47 +555,201 @@ error:
 }
 
 static int
-do_print (PedDevice* dev)
+do_move (PedDevice** dev)
 {
-	PedDisk*	disk;
-	PedPartition*	part;
-	int		first_flag;
+	PedDisk*		disk = NULL;
+	PedPartition*		part = NULL;
+	PedSector		start, end;
+	PedGeometry		old_geom;
+	PedConstraint*		constraint;
+	PedFileSystem*		fs = NULL;
 
-	disk = ped_disk_open (dev);
+	disk = ped_disk_open (*dev);
 	if (!disk)
 		goto error;
 
-	printf (_("Disk geometry: 0-%.1f megabytes\n"),
-		disk->dev->length * 1.0 / MEGABYTE_SECTORS);
+	if (!is_integer ()) {
+		help_on ("move");
+		goto error_close_disk;
+	}
+	part = ped_disk_get_partition (disk, get_integer ());
+	if (!part) {
+		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
+			_("Partition doesn't exist."));
+		goto error_close_disk;
+	}
+	if (part->type == PED_PARTITION_EXTENDED) {
+		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
+			_("Can't move extended partitions."));
+		goto error_close_disk;
+	}
 
-	printf (
-_("Minor   Start     End    Type            Filesystem     Flags\n"));
+	if (ped_partition_is_busy (part)) {
+		if (ped_exception_throw (
+			PED_EXCEPTION_WARNING,
+			PED_EXCEPTION_IGNORE_CANCEL,
+			_("Partition is being used."))
+				!= PED_EXCEPTION_IGNORE)
+			goto error_close_disk;
+	}
+
+	/* NOTE: this gets aligned by ped_disk_set_partition_geom() */
+	if (!is_sector ()) {
+		help_on ("move");
+		goto error_close_disk;
+	}
+	start = get_sector ();
+	if (!is_sector ()) {
+		help_on ("move");
+		goto error_close_disk;
+	}
+	end = get_sector ();
+
+	old_geom = part->geom;
+	constraint = ped_constraint_any (disk);
+	if (!constraint)
+		goto error_close_disk;
+	if (!ped_disk_set_partition_geom (disk, part, constraint, start, end)) {
+		ped_constraint_destroy (constraint);
+		goto error_close_disk;
+	}
+	ped_constraint_destroy (constraint);
+
+	if (ped_geometry_test_overlap (&old_geom, &part->geom)) {
+		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
+			_("Can't move a partition onto itself.  Try using "
+			  "resize, perhaps?"));
+		goto error_close_disk;
+	}
+
+/* do the move */
+	fs = ped_file_system_open (&old_geom);
+	if (!fs)
+		goto error_close_disk;
+	if (!ped_file_system_copy (fs, &part->geom))
+		goto error_close_fs;
+	ped_file_system_close (fs);
+	if (!ped_disk_write (disk))
+		goto error_close_disk;
+	ped_disk_close (disk);
+	return 1;
+
+error_close_fs:
+	ped_file_system_close (fs);
+error_close_disk:
+	ped_disk_close (disk);
+error:
+	return 0;
+}
+	
+static int
+do_name (PedDevice** dev)
+{
+	PedDisk*	disk;
+	PedPartition*	part;
+
+	disk = ped_disk_open (*dev);
+	if (!disk)
+		goto error;
+
+	if (!is_integer ()) {
+		help_on ("name");
+		goto error_close_disk;
+	}
+
+	part = ped_disk_get_partition (disk, get_integer ());
+	if (!part) {
+		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
+			_("Partition doesn't exist."));
+		goto error_close_disk;
+	}
+
+	if (!peek_word ()) {
+		help_on ("name");
+		goto error_close_disk;
+	}
+
+	if (!ped_partition_set_name (part, get_word()))
+		goto error_close_disk;
+	if (!ped_disk_write (disk))
+		goto error_close_disk;
+	ped_disk_close (disk);
+	return 1;
+
+error_close_disk:
+	ped_disk_close (disk);
+error:
+	return 0;
+}
+
+static int
+do_print (PedDevice** dev)
+{
+	PedDisk*		disk;
+	PedPartition*		part;
+	PedPartitionFlag	flag;
+	int			first_flag;
+	int			has_extended;
+	int			has_name;
+
+	disk = ped_disk_open (*dev);
+	if (!disk)
+		goto error;
+
+	printf (_("Disk geometry for %s: 0.000-%.3f megabytes\n"),
+		disk->dev->path,
+		(disk->dev->length-1) * 1.0 / MEGABYTE_SECTORS);
+	printf (_("Disk label type: %s\n"), disk->type->name);
+
+	has_extended = ped_disk_type_check_feature (disk->type,
+			       		 PED_DISK_TYPE_EXTENDED);
+	has_name = ped_disk_type_check_feature (disk->type,
+			       		 PED_DISK_TYPE_PARTITION_NAME);
+
+	printf (_("Minor    Start       End     "));
+	if (has_extended)
+		printf (_("Type      "));
+	printf (_("Filesystem  "));
+	if (has_name)
+		printf (_("Name                  "));
+	printf (_("Flags"));
+	printf ("\n");
 
 	for (part = ped_disk_next_partition (disk, NULL); part;
 	     part = ped_disk_next_partition (disk, part)) {
-		if (part->type == PED_PARTITION_PRIMARY
-		    || part->type == PED_PARTITION_LOGICAL
-		    || part->type == PED_PARTITION_EXTENDED) {
-			printf ("%-5d %8.1f %8.1f  %-15s %-15s",
-				(int) part->num,
-				(int) part->geom.start * 1.0 / MEGABYTE_SECTORS,
-				(int) part->geom.end * 1.0 / MEGABYTE_SECTORS,
-				ped_partition_type_get_name (part->type),
-				part->fs_type ? part->fs_type->name : "");
+		
+		if (part->type != PED_PARTITION_PRIMARY
+		    && part->type != PED_PARTITION_LOGICAL
+		    && part->type != PED_PARTITION_EXTENDED)
+	       		continue;
 
-			first_flag = 1;
-			if (part->bootable) {
-				printf (_("boot"));
-				first_flag = 0;
-			}
-			if (part->hidden) {
-				if (!first_flag)
+		printf ("%-5d ", part->num);
+
+		printf ("%10.3f %10.3f  ",
+			(int) part->geom.start * 1.0 / MEGABYTE_SECTORS,
+			(int) part->geom.end * 1.0 / MEGABYTE_SECTORS);
+
+		if (has_extended)
+			printf ("%-9s ",
+				ped_partition_type_get_name (part->type));
+
+		printf ("%-12s", part->fs_type ? part->fs_type->name : "");
+
+		if (has_name)
+			printf ("%-22s", ped_partition_get_name (part));
+
+		first_flag = 1;
+		for (flag = ped_partition_flag_next (0); flag;
+		     flag = ped_partition_flag_next (flag)) {
+			if (ped_partition_get_flag (part, flag)) {
+				if (first_flag)
+					first_flag = 0;
+				else
 					printf (", ");
-				printf (_("hidden"));
-				first_flag = 0;
+				printf (_(ped_partition_flag_get_name (flag)));
 			}
-			printf ("\n");
 		}
+		printf ("\n");
 	}
 
 	ped_disk_close (disk);
@@ -635,29 +760,28 @@ error:
 }
 
 static int
-do_quit (PedDevice* dev)
+do_quit (PedDevice** dev)
 {
-	ped_device_close (dev);
-	ped_done ();
-	done_ui ();
+	_done (*dev);
 	exit (0);
 }
 
 static int
-do_resize (PedDevice* dev)
+do_resize (PedDevice** dev)
 {
 	PedDisk*		disk;
 	PedPartition*		part;
 	PedFileSystem*		fs;
 	PedFileSystemType*	fs_type;
+	PedConstraint*		constraint;
 	PedSector		start, end;
 
-	disk = ped_disk_open (dev);
+	disk = ped_disk_open (*dev);
 	if (!disk)
 		goto error;
 
 	if (!is_integer ()) {
-		help_on (_("resize"));
+		help_on ("resize");
 		goto error_close_disk;
 	}
 	part = ped_disk_get_partition (disk, get_integer ());
@@ -675,30 +799,33 @@ do_resize (PedDevice* dev)
 			goto error_close_disk;
 	}
 
+	/* NOTE: this gets aligned by ped_disk_set_partition_geom() */
 	if (!is_sector ()) {
-		help_on (_("resize"));
+		help_on ("resize");
 		goto error_close_disk;
 	}
 	start = get_sector ();
 	if (!is_sector ()) {
-		help_on (_("resize"));
+		help_on ("resize");
 		goto error_close_disk;
 	}
-
-	/* NOTE: this gets aligned by ped_disk_set_partition_geom() */
 	end = get_sector ();
 
 	if (part->type == PED_PARTITION_EXTENDED) {
-		if (!ped_disk_set_partition_geom (disk, part, start, end))
-			goto error_close_disk;
+		constraint = ped_constraint_any (disk);
+		if (!ped_disk_set_partition_geom (disk, part, constraint,
+						  start, end))
+			goto error_destroy_constraint;
 		ped_partition_set_system (part, NULL);
 	} else {
 		fs = ped_file_system_open (&part->geom);
 		if (!fs)
 			goto error_close_disk;
 		fs_type = fs->type;
-		if (!ped_disk_set_partition_geom (disk, part, start, end))
-			goto error_close_disk;
+		constraint = ped_file_system_get_resize_constraint (fs);
+		if (!ped_disk_set_partition_geom (disk, part, constraint,
+						  start, end))
+			goto error_close_fs;
 		if (!ped_file_system_resize (fs, &part->geom))
 			goto error_close_fs;
 		ped_file_system_close (fs);
@@ -706,11 +833,14 @@ do_resize (PedDevice* dev)
 	}
 
 	ped_disk_write (disk);
+	ped_constraint_destroy (constraint);
 	ped_disk_close (disk);
 	return 1;
 
 error_close_fs:
 	ped_file_system_close (fs);
+error_destroy_constraint:
+	ped_constraint_destroy (constraint);
 error_close_disk:
 	ped_disk_close (disk);
 error:
@@ -718,12 +848,12 @@ error:
 }
 
 static int
-do_rm (PedDevice* dev)
+do_rm (PedDevice** dev)
 {
 	PedDisk*		disk;
 	PedPartition*		part;
 
-	disk = ped_disk_open (dev);
+	disk = ped_disk_open (*dev);
 	if (!disk)
 		goto error;
 
@@ -751,7 +881,7 @@ do_rm (PedDevice* dev)
 	return 1;
 
 error_syntax:
-	help_on (_("rm"));
+	help_on ("rm");
 error_close_disk:
 	ped_disk_close (disk);
 error:
@@ -759,19 +889,44 @@ error:
 }
 
 static int
-do_unhide (PedDevice* dev)
+do_select (PedDevice** dev)
 {
-	PedDisk*	disk;
-	PedPartition*	part;
+	PedDevice*	new_dev;
 
-	disk = ped_disk_open (dev);
+	if (!peek_word())
+		goto error_syntax;
+
+	new_dev = ped_device_get (get_word ());
+	if (!new_dev)
+		goto error;
+	if (!ped_device_open (new_dev))
+		goto error;
+
+	ped_device_close (*dev);
+	*dev = new_dev;
+	print_using_dev (*dev);
+	return 1;
+
+error_syntax:
+	help_on ("select");
+error:
+	return 0;
+}
+
+static int
+do_set (PedDevice** dev)
+{
+	PedDisk*		disk;
+	PedPartition*		part;
+	PedPartitionFlag	flag;
+	int			state;
+
+	disk = ped_disk_open (*dev);
 	if (!disk)
 		goto error;
 
-	if (!is_integer ()) {
-		help_on (_("unhide"));
-		goto error_close_disk;
-	}
+	if (!is_integer ())
+       		goto error_help;
 	part = ped_disk_get_partition (disk, get_integer ());
 	if (!part) {
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
@@ -779,13 +934,25 @@ do_unhide (PedDevice* dev)
 		goto error_close_disk;
 	}
 
-	if (!ped_partition_set_hide_state (part, 0))
+	if (!peek_word ())
+       		goto error_help;
+	flag = ped_partition_flag_get_by_name (get_word ());
+	if (!flag)
+       		goto error_help;
+
+	if (!peek_word ())
+		goto error_help;
+	state = get_state ();
+
+	if (!ped_partition_set_flag (part, flag, state))
 		goto error_close_disk;
 	if (!ped_disk_write (disk))
 		goto error_close_disk;
 	ped_disk_close (disk);
 	return 1;
 
+error_help:
+	help_on ("set");
 error_close_disk:
 	ped_disk_close (disk);
 error:
@@ -799,6 +966,24 @@ _init_messages ()
 	int			first;
 	PedFileSystemType*	fs_type;
 	PedDiskType*		disk_type;
+	PedPartitionFlag	part_flag;
+
+/* flags */
+	first = 1;
+	list = str_list_create (_(flag_msg_start), NULL);
+	for (part_flag = ped_partition_flag_next (0); part_flag;
+	     		part_flag = ped_partition_flag_next (part_flag)) {
+		if (first)
+			first = 0;
+		else
+			str_list_append (list, ", ");
+		str_list_append (list,
+				 _(ped_partition_flag_get_name (part_flag)));
+	}
+	str_list_append (list, "\n");
+
+	flag_msg = str_list_convert (list);
+	str_list_destroy (list);
 
 /* disk type */
 	list = str_list_create (_(label_type_msg_start), NULL);
@@ -838,14 +1023,6 @@ _init_messages ()
 static void
 _init_commands ()
 {
-	command_register (commands, command_create (
-		str_list_create_unique ("boot", _("boot"), NULL),
-		do_boot,
-		str_list_create (
-_("boot MINOR                    set the boot partition to be MINOR"),
-NULL),
-		str_list_create (_(minor_msg), NULL)));
-
 	command_register (commands, command_create (
 		str_list_create_unique ("check", _("check"), NULL),
 		do_check,
@@ -858,7 +1035,8 @@ NULL),
 		str_list_create_unique ("cp", _("cp"), NULL),
 		do_cp,
 		str_list_create (
-_("cp MINOR [DEVICE] MINOR       copy filesystem to another partition"),
+_("cp [FROM-DEVICE] FROM-MINOR TO-MINOR      copy filesystem to another "
+  "partition"),
 NULL),
 		str_list_create (_(minor_msg), _(device_msg), NULL)));
 
@@ -870,14 +1048,6 @@ _("help [COMMAND]                prints general help, or help on COMMAND"),
 NULL),
 		NULL));
 
-	command_register (commands, command_create (
-		str_list_create_unique ("hide", _("hide"), NULL),
-		do_hide,
-		str_list_create (
-_("hide MINOR                    hides partition MINOR"),
-NULL),
-		str_list_create (_(minor_msg), NULL)));
-
 	command_register (commands, command_create (
 		str_list_create_unique ("mklabel", _("mklabel"), NULL),
 		do_mklabel,
@@ -921,6 +1091,24 @@ _("mkpartfs PART-TYPE FS-TYPE START END      make a partition with a "
 NULL),
 		str_list_create (_(part_type_msg), _(start_end_msg), NULL)));
 
+
+	command_register (commands, command_create (
+		str_list_create_unique ("move", _("move"), NULL),
+		do_move,
+		str_list_create (
+_("move MINOR START END          move partition MINOR"),
+NULL),
+		str_list_create (_(minor_msg), _(start_end_msg), NULL)));
+
+	command_register (commands, command_create (
+		str_list_create_unique ("name", _("name"), NULL),
+		do_name,
+		str_list_create (
+_("name MINOR NAME               name partition MINOR NAME"),
+NULL),
+		str_list_create (_(minor_msg), _(name_msg), NULL)));
+
+
 	command_register (commands, command_create (
 		str_list_create_unique ("print", _("print"), NULL),
 		do_print,
@@ -954,13 +1142,21 @@ NULL),
 		str_list_create (_(minor_msg), NULL)));
 
 	command_register (commands, command_create (
-		str_list_create_unique ("unhide", _("unhide"), NULL),
-		do_unhide,
+		str_list_create_unique ("select", _("select"), NULL),
+		do_select,
 		str_list_create (
-_("unhide MINOR                  unhides partition MINOR"),
+_("select DEVICE                 choose the device to edit"),
 NULL),
-		str_list_create (_(minor_msg), NULL)));
+		str_list_create (_(device_msg), NULL)));
 
+	command_register (commands, command_create (
+		str_list_create_unique ("set", _("set"), NULL),
+		do_set,
+		str_list_create (
+_("set MINOR FLAG STATE          change a flag on partition MINOR"),
+NULL),
+		str_list_create (_(minor_msg), _(flag_msg), _(state_msg),
+				 NULL)));
 }
 
 static void
@@ -968,8 +1164,8 @@ _init_i18n ()
 {
 /* intialize i18n */
 #ifdef ENABLE_NLS
-	language = getenv ("LANG");	/* FIXME */
 	setlocale(LC_ALL, "");
+	setlocale(LC_NUMERIC, "en_US");		/* FIXME */
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 #endif /* ENABLE_NLS */
@@ -1051,6 +1247,11 @@ _init (int* argc_ptr, char*** argv_ptr)
 {
 	PedDevice*	dev;
 
+#ifdef ENABLE_MTRACE
+	if (getenv ("MALLOC_TRACE"))
+		mtrace();
+#endif
+
 	_init_i18n ();
 	if (!init_ui ())
 		goto error;
@@ -1074,11 +1275,11 @@ error:
 }
 
 static void
-_done (PedDevice *dev)
+_done (PedDevice* dev)
 {
 	ped_device_close (dev);
-	ped_done ();
-	done_ui ();
+	ped_done();
+	done_ui();
 }
 
 int
