@@ -1,10 +1,6 @@
 /*
     parted - a frontend to libparted
-    Copyright (C) 1999  Andrew Clausen, Lennert Buytenhek and Red Hat, Inc.
-
-    Andrew Clausen		<clausen@gnu.org>
-    Lennert Buytenhek		<buytenh@gnu.org>
-    Matt Wilson, Red Hat Inc.	<msw@redhat.com>
+    Copyright (C) 1999, 2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,10 +27,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 
 #ifdef ENABLE_NLS
 
-#include <iconv.h>
 #include <wchar.h>
 #include <wctype.h>
 
@@ -52,16 +48,6 @@
 
 #define MIN(a,b)	( (a<b)?  a : b )
 
-#ifdef ENABLE_NLS
-
-char*			language;
-static char*		gettext_charset;
-static char*		term_charset;
-static iconv_t		gettext_to_wchar_handle;
-static iconv_t		wchar_to_term_handle;
-
-#endif /* ENABLE_NLS */
-
 int
 wchar_strlen (const wchar_t* str)
 {
@@ -76,7 +62,7 @@ wchar_t*
 wchar_strchr (const wchar_t* str, char ch)
 {
 #ifdef ENABLE_NLS
-	return wcschr (str, ch << (3 * 8));
+	return wcschr (str, ch);
 #else
 	return strchr (str, ch);
 #endif
@@ -102,112 +88,42 @@ wchar_strncasecmp (const wchar_t* a, const wchar_t* b, int n)
 #endif
 }
 
-static const char*
-get_gettext_encoding ()
-{
-#ifdef ENABLE_NLS
-	if (!language)
-		return "ISO-8859-1";
-	if (strncasecmp (language, "ja", 2) == 0)
-		return "EUC-JP";
-	if (strncasecmp (language, "ru", 2) == 0)
-		return "KOI8-R";
-#endif
-	return "ISO-8859-1";
-}
-
-static const char*
-get_terminal_encoding ()
-{
-#ifdef ENABLE_NLS
-	if (!language)
-		return "ISO-8859-1";
-	if (strncasecmp (language, "ja", 2) == 0)
-		return "EUC-JP";
-	if (strncasecmp (language, "ru", 2) == 0)
-		return "KOI8-R";
-#endif
-	return "ISO-8859-1";
-}
-
 int
 init_str_list ()
 {
-#ifdef ENABLE_NLS
-	gettext_to_wchar_handle = iconv_open ("UCS4", get_gettext_encoding ());
-	if (gettext_to_wchar_handle == (iconv_t) -1)
-		goto error;
-
-	wchar_to_term_handle = iconv_open (get_terminal_encoding (), "UCS4");
-	if (wchar_to_term_handle == (iconv_t) -1)
-		goto error_close_handle;
-
 	return 1;
-
-error_close_handle:
-	iconv_close (gettext_to_wchar_handle);
-error:
-	printf ("Error initializing character set conversion: %s\n",
-		strerror (errno));
-	return 0;
-
-#else /* ENABLE_NLS */
-	return 1;
-#endif /* !ENABLE_NLS */
 }
 
 int
 done_str_list ()
 {
-#ifdef ENABLE_NLS
-	iconv_close (gettext_to_wchar_handle);
-	iconv_close (wchar_to_term_handle);
-#endif
 	return 1;
 }
 
 /* converts a string from the encoding in the gettext catalogues to wide
- * character strings (of type wchar_t*).  Wide character strings are encoded
- * with UCS4.
+ * character strings (of type wchar_t*).
  */
 #ifdef ENABLE_NLS
 static wchar_t*
 gettext_to_wchar (const char* str)
 {
-	size_t		in_bytes_left = strlen (str);
-	wchar_t*	result = malloc (64);
-	char*		out_buf = (char*) result;
-	size_t		out_buf_size = 64;
-	size_t		out_bytes_left = 60;	/* leave room for terminator */
+	int		count;
+	wchar_t*	result;
 	size_t		status;
+	mbstate_t	ps;
 
-/* FIXME: add this code when glibc is fixed */
-/*
-	status = iconv (gettext_to_wchar_handle,
-			NULL, NULL,
-			&out_buf, &out_bytes_left);
-	if (status == (iconv_t) -1)
+	count = strlen (str) + 1;
+	result = malloc (count * sizeof (wchar_t));
+	if (!result)
 		goto error;
-*/
 
-	while (1) {
-		status = iconv (gettext_to_wchar_handle,
-			   &str, &in_bytes_left,
-			   &out_buf, &out_bytes_left);
-		if (status == -1 && errno != E2BIG)
-			goto error;
+	memset(&ps, 0, sizeof (ps));
+	status = mbsrtowcs(result, &str, count, &ps);
+	if (status == (size_t) -1)
+		goto error;
 
-		if (!in_bytes_left)
-			break;
-
-		out_buf_size += 64;
-		out_bytes_left += 64;
-		result = realloc (result, out_buf_size);
-		out_buf = (char*) result + (out_buf_size-4) - out_bytes_left;
-	}
-
-	* (wchar_t*) out_buf = 0;
-	return (wchar_t*) result;
+	result = realloc (result, (wcslen (result) + 1) * sizeof (wchar_t));
+	return result;
 
 error:
 	printf ("Error during translation: %s\n", strerror (errno));
@@ -229,40 +145,33 @@ gettext_to_wchar (const char* str)
 static char*
 wchar_to_str (const wchar_t* str, int count)
 {
-	size_t		in_bytes_left = ((count) ? count : wcslen (str))
-					* (sizeof (wchar_t));
-	char*		result = malloc (64);
-	char*		out_buf = result;
-	size_t		out_buf_size = 64;
-	size_t		out_bytes_left = 63;	/* leave room for terminator */
+	char*		result;
+	char*		out_buf;
 	size_t		status;
+	mbstate_t	ps;
+	int		i;
 
-/* FIXME: add this code when glibc is fixed */
-/*
-	status = iconv (wchar_to_term_handle,
-			NULL, NULL,
-			&out_buf, &out_bytes_left);
-	if (status == (iconv_t) -1)
+	if (count == 0 || wcslen(str) < count)
+		count = wcslen (str);
+
+	out_buf = result = malloc ((count + 1) *  MB_LEN_MAX);
+	if (!result)
 		goto error;
-*/
 
-	while (1) {
-		status = iconv (wchar_to_term_handle,
-			   (const char**) &str, &in_bytes_left,
-			   &out_buf, &out_bytes_left);
-		if (status == -1 && errno != E2BIG)
+	memset(&ps, 0, sizeof(ps));
+
+	for (i = 0; i < count; i++) {
+		status = wcrtomb (out_buf, str[i], &ps);
+		if (status == (size_t) -1)
 			goto error;
-
-		if (!in_bytes_left)
-			break;
-
-		out_buf_size += 64;
-		out_bytes_left += 64;
-		result = realloc (result, out_buf_size);
-		out_buf = result + (out_buf_size-1) - out_bytes_left;
+		out_buf += status;
 	}
 
-	*out_buf = 0;
+	status = wcrtomb (out_buf, 0, &ps);
+	if (status == (size_t) -1)
+		goto error;
+
+	result = realloc (result, strlen (result) + 1);
 	return result;
 
 error:
@@ -453,15 +362,9 @@ str_search (const wchar_t* str, int n, wchar_t c)
 {
 	int	i;
 
-#ifdef ENABLE_NLS
-	for (i=0; i<n; i++)
-		if (str [i] == c << (3 * 8))
-			return i;
-#else
 	for (i=0; i<n; i++)
 		if (str [i] == c)
 			return i;
-#endif
 	return -1;
 }
 
@@ -472,14 +375,15 @@ str_search (const wchar_t* str, int n, wchar_t c)
  * need to include them.  Best not to avoid side effects, like 3.
 14159 :-)
  * FIXME: how do we exclude "." and "(" ?
- * FIXME: glibc doesn't like umlaute.  i.e. \"o (TeX equivalent)
+ * FIXME: glibc doesn't like umlaute.  i.e. \"o (TeX notation), which should
+ * look like: 
  */
 
 static int
 is_break_point (wchar_t c)
 {
 #ifdef ENABLE_NLS
-	return !iswalnum (c >> (3 * 8)) && !iswpunct (c >> (3 * 8));
+	return !iswalnum (c) && !iswpunct (c);
 #else
 	return !isalnum (c) && !ispunct (c);
 #endif
@@ -492,7 +396,7 @@ static int
 is_space (wchar_t c)
 {
 #ifdef ENABLE_NLS
-	return (c >> (3 * 8)) == ' ';		/* FIXME: why big-endian? */
+	return c == (wchar_t) btowc(' ');
 #else
 	return c == ' ';
 #endif
@@ -548,7 +452,7 @@ str_list_print_wrap (const StrList* list, int line_length, int offset,
 				line_break = 1;
 			}
 
-			for (cut_right = cut_left + 2;
+			for (cut_right = cut_left + (line_break ? 2 : 1);
 			     cut_right < str_len && is_space (str [cut_right]);
 			     cut_right++);
 
