@@ -1,10 +1,6 @@
 /*
     parted - a frontend to libparted
-    Copyright (C) 1999  Andrew Clausen, Lennert Buytenhek and Red Hat, Inc.
-
-    Andrew Clausen		<clausen@gnu.org>
-    Lennert Buytenhek		<buytenh@gnu.org>
-    Matt Wilson, Red Hat Inc.	<msw@redhat.com>
+    Copyright (C) 1999, 2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,7 +28,7 @@
 #include <locale.h>
 #define N_(String) String
 #if ENABLE_NLS
-#  define _(String) gettext (String)
+#  define _(String) dgettext (PACKAGE, String)
 #else
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
@@ -48,21 +44,24 @@
 
 #endif /* HAVE_LIBREADLINE */
 
+#include <ctype.h>
 #include <signal.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
 char* prog_name = "GNU Parted " VERSION "\n";
+
 static char* banner_msg = N_(
-"Copyright (C) 1999-2000 Andrew Clausen, Lennert Buytenhek and Red Hat Inc.\n"
-"Translations are Copyright (C) 1999-2000 Free Software Foundation, Inc\n"
+"Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.\n"
 "This program is free software, covered by the GNU General Public License.\n"
 "\n"
 "This program is distributed in the hope that it will be useful, "
 "but WITHOUT ANY WARRANTY; without even the implied warranty of "
 "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
-"GNU General Public License for more details.\n");
+"GNU General Public License for more details.\n"
+"\n"
+"GNU Parted's development is supported by Conectiva, www.conectiva.com\n");
 
 static char* usage_msg = N_(
 "Usage: parted [OPTION]... [DEVICE [COMMAND [PARAMETERS]...]...]\n"
@@ -265,7 +264,6 @@ static StrList* last_match_str;
 static char*
 command_generator (char* text, int state)
 {
-	int		text_length = strlen (text);
 	int		i = (state)? last_match_cmd : 0;
 	int		skip = state;
 	StrList*	walk;
@@ -348,6 +346,17 @@ get_sector ()
 	return atof (words [word_num++]) * MEGABYTE_SECTORS;
 }
 
+int
+get_state ()
+{
+	char*	word;
+
+	word = get_word();
+	if (strcasecmp (_("on"), word) == 0 || strcasecmp ("on", word) == 0)
+		return 1;
+	return 0;
+}
+
 char*
 peek_word ()
 {
@@ -388,7 +397,55 @@ count_words ()
 	int	i;
 
 	for (i=0; words [i]; i++);
-	return i;
+	return i - word_num;
+}
+
+void
+free_line ()
+{
+	int	i;
+
+	for (i=0; words [i]; i++)
+		free (words[i]);
+}
+
+void
+parse_line (char* line)
+{
+	int	quoted = 0;
+	char	quote_char;
+	char	this_word [256];
+	int	i;
+
+	free_line ();
+
+	word_num = 0;
+	this_word [i=0] = 0;
+
+	for (; 1; line++) {
+		if (!*line || (!quoted && *line == ' ')) {
+			this_word [i] = 0;
+			words [word_num++] = strdup (this_word);
+			while (*line == ' ')
+			       	line++;
+			if (!*line)
+				break;
+			i=0;
+		}
+		if (quoted && *line == quote_char) {
+			quoted = 0;
+			continue;
+		}
+		if (!quoted && (*line == '\'' || *line == '"')) {
+			quoted = 1;
+			quote_char = *line;
+			continue;
+		}
+		this_word [i++] = *line;
+	}
+
+	words [word_num] = NULL;
+	word_num = 0;
 }
 
 void
@@ -459,11 +516,47 @@ help_msg ()
 	exit (0);
 }
 
+void
+print_using_dev (PedDevice* dev)
+{
+	printf (_("\nUsing %s\n"), dev->path);
+
+#if defined (__i386__)
+	if (dev->type != PED_DEVICE_UNKNOWN
+	    && dev->type != PED_DEVICE_FILE) {
+		if (dev->cylinders > 1024)
+			ped_exception_throw (
+				PED_EXCEPTION_WARNING,
+				PED_EXCEPTION_OK,
+			_("The operating system thinks the geometry on %s is "
+			  "%d/%d/%d.  Therefore, cylinder 1024 ends at %.3fM.  "
+			  "You should check that this matches the BIOS "
+			  "geometry before using this program."),
+				dev->path,
+				dev->cylinders,
+				dev->heads,
+				dev->sectors,
+				(1024 * dev->heads * dev->sectors - 1)
+					/ 2 / 1024.0);
+		else
+			ped_exception_throw (
+				PED_EXCEPTION_WARNING,
+				PED_EXCEPTION_OK,
+			_("The operating system thinks the geometry on %s is "
+			  "%d/%d/%d.  You should check that this matches the "
+			  "BIOS geometry before using this program."),
+				dev->path,
+				dev->cylinders,
+				dev->heads,
+				dev->sectors);
+	}
+#endif
+}
+
 int
 interactive_mode (PedDevice* dev, Command* cmd_list[])
 {
 	char*		line;
-	int		i;
 	Command*	cmd;
 	StrList*	list;
 
@@ -475,24 +568,12 @@ interactive_mode (PedDevice* dev, Command* cmd_list[])
 	str_list_print_wrap (list, screen_width (), 0, 0);
 	str_list_destroy (list);
 
-	printf (_("\nUsing %s\n"), dev->path);
-
-	ped_exception_throw (PED_EXCEPTION_WARNING, PED_EXCEPTION_OK,
-		_("The operating system thinks the geometry on %s is "
-		  "%d/%d/%d.  You should check that this matches "
-		  "the BIOS geometry before using this program."),
-		dev->path,
-		dev->cylinders,
-		dev->heads,
-		dev->sectors);
+	print_using_dev (dev);
 
 	while (1) {
 		line = _readline ("(parted) ");
 		if (!line) break;
-
-		word_num = 0;
-		words [0] = strtok (line, " ,\n");
-		for (i=1; (words [i] = strtok (NULL, " ,\n")); i++);
+		parse_line (line);
 
 		if (!peek_word ()) {
 			print_commands_help ();
@@ -501,7 +582,7 @@ interactive_mode (PedDevice* dev, Command* cmd_list[])
 
 		cmd = command_get (commands, get_word ());
 		if (cmd)
-			command_run (cmd, dev);
+			command_run (cmd, &dev);
 		else
 			print_commands_help ();
 
@@ -532,7 +613,7 @@ non_interactive_mode (PedDevice* dev, Command* cmd_list[],
 			help_msg ();
 			goto error;
 		}
-		if (!command_run (cmd, dev))
+		if (!command_run (cmd, &dev))
 			goto error;
 	}
 
